;*****************************************************************************
;  FP-15 and FP-16 Static SMWIR Xtalk test (OOB only).
;  Reads and formats the response. Uses all detectors.
;
;  INPUTS:
;  1) infile - file with input, output paths, sender and uaid name, and fpa

;  OUTPUTS:
;  1) dn[collects,band,detector,subsample] -- dn.
;  2) L[collects,band,detector,subsample] -- radiance.
;  3) std_temp[collects,band,detector,subsample,0=closed 1=open]
;      -- standard deviation averaged of both open and closed scans.
;  
;  v1.0 - 11/26/2008 - Jeff McIntire
;  v2.0 - 05/30/2014 - Jeff McIntire
;       - rewritten for F2
;*****************************************************************************

PRO fp15_16_oob, infile

;*****************************************************************************
;  Read in input file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
inpath2=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
uaidname=strarr(1)
sender=strarr(1)
fpa=strarr(1)
odd_even=strarr(1)
header_size=intarr(1)
list_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,inpath2
readf,lun,outpath
readf,lun,uaidname
readf,lun,sender
readf,lun,fpa
readf,lun,odd_even
readf,lun,header_size
readf,lun,list_size
Free_Lun,lun

;*****************************************************************************
;  Get the list of dump files.
;*****************************************************************************

IF list_size EQ 0 THEN spawn,'ls -C1 '+inpath+'*EV*.bin',list_files
IF list_size EQ 1 THEN BEGIN
   spawn,'ls -C1 '+inpath+uaidname+'_EV_C0*',list_files1
   spawn,'ls -C1 '+inpath+uaidname+'_EV_C1*',list_files2
   spawn,'ls -C1 '+inpath+uaidname+'_EV_C2*',list_files3
   
   list_files=strarr(n_elements(list_files1)+n_elements(list_files2)+n_elements(list_files3))
   list_files[0:n_elements(list_files1)-1]=list_files1[0:n_elements(list_files1)-1]
   list_files[n_elements(list_files1):n_elements(list_files1)+n_elements(list_files2)-1]=list_files2[0:n_elements(list_files2)-1]
   list_files[n_elements(list_files1)+n_elements(list_files2):n_elements(list_files1)+n_elements(list_files2)+n_elements(list_files3)-1] $
      =list_files3[0:n_elements(list_files3)-1]
ENDIF

;*****************************************************************************
;  Select focal plane dependent parameters and restore data from
;  previous routines.
;*****************************************************************************

IF fpa EQ 'DNB' THEN restore,'general.DNB.2.sav'
IF fpa EQ 'VisNIR' THEN restore,'general.VisNIR.2.sav'
IF fpa EQ 'SMWIR' THEN restore,'general.SMWIR.2.sav'
IF fpa EQ 'LWIR' THEN restore,'general.LWIR.2.sav'

IF fpa EQ 'DNB' THEN maxgain=3

restore,outpath+uaidname+'.dn.1.sav'
restore,outpath+uaidname+'.lambda.sav'

lambda_center[senderband]=lambda_center[senderband]*1000

;*****************************************************************************
;  Definitions.
;*****************************************************************************

abs_scan_number=lonarr(ncollects)
scan_b_coll=lonarr(ncollects)
change_order=fltarr(ncollects)

;*****************************************************************************
;  Read data from dump files.
;*****************************************************************************

FOR col=0, ncollects-1 DO BEGIN
   print,list_files[col],' ',col
   nlines=file_lines(list_files[col])
   IF nlines EQ 0 THEN continue
   
   read_dump_bin,list_files[col],scan,ham,det,frame,gain_type,dns,mscan_num=128,abs_scan=abs_scan,msamp_num=6400
   abs_scan_number[col]=abs_scan[0]

;*****************************************************************************
;  Subtract the total scan numbers between the first scan of each
;  collect and the change in shutter order.
;*****************************************************************************

   IF col NE 0 THEN BEGIN
      scan_b_coll[col]=abs_scan_number[col]-abs_scan_number[col-1]-max(scan)
      change_order[col]=fix(4*(float(scan_b_coll[col])/4-long(scan_b_coll[col]/4)))
      
      IF abs_scan_number[col-1] GT abs_scan_number[col] THEN BEGIN
         scan_b_coll[col]=abs_scan_number[col]+65535-abs_scan_number[col-1]-max(scan)
         change_order[col]=fix(4*(float(scan_b_coll[col])/4-long(scan_b_coll[col]/4)))
      ENDIF
   ENDIF
   free_lun,lun
ENDFOR

;*****************************************************************************
;  Definitions.
;*****************************************************************************

; Records shutter order per collect from dn.
shutter=fltarr(ncollects)
det_shutter=fltarr(ncollects,maxdet)
shutter_tel=fltarr(ncollects,3)

; Marks collects under the 50% shutter confidence level.
collect_mark=fltarr(ncollects,3)

; Marks collects at the edge of the measured range.
edge=fltarr(ncollects,3)

; DN values per collect.
dn_collect=dblarr(ncollects,nbands,nham,maxdet,maxsubsets)
dn=dblarr(ncollects,nbands,maxdet,maxsubsets)

; Open and Closed noise (0=open and 1=closed)
std_collect=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,2)
std=dblarr(ncollects,nbands,maxdet,maxsubsets,2)

;*****************************************************************************
;  Determine shutter order.
;*****************************************************************************

FOR col=0, ncollects-1 DO BEGIN
   IF fpa EQ 'DNB' AND senderband EQ 0 THEN igain=2 ELSE igain=1

   IF mean(dn_temp[col,senderband,0,0,0,igain,*]) EQ 0 THEN continue
   
   temp3=[0,0,0,0]
   FOR idet=0, detectors[senderband]-1 DO BEGIN
      IF fpa EQ 'SMWIR' AND idet/2-idet/2.+.5*odd_even NE 0 THEN continue
      IF fpa EQ 'LWIR' AND idet/2-idet/2.+.5*odd_even NE 0 THEN continue
        
      ; Calculate the total dn in all bands and detectors for a particular scan.
      FOR iham=0, nham-1 DO BEGIN
         ; Compare the first and third scans, determine the larger.
         IF dn_temp[col,senderband,iham,idet,0,igain,0] NE 0 THEN BEGIN
            IF dn_temp[col,senderband,iham,idet,0,igain,0]-dn_temp[col,senderband,iham,idet,0,igain,2] GT 0 THEN temp1=0 ELSE temp1=1
         ENDIF
         ; Compare the second and fourth scans, determine the larger.
         IF dn_temp[col,senderband,iham,idet,0,igain,1] NE 0 THEN BEGIN
            IF dn_temp[col,senderband,iham,idet,0,igain,1]-dn_temp[col,senderband,iham,idet,0,igain,3] GT 0 THEN temp2=0 ELSE temp2=1 
         ENDIF
      ENDFOR
      IF temp1 EQ 0 AND temp2 EQ 0 THEN det_shutter[col,idet]=1
      IF temp1 EQ 0 AND temp2 EQ 1 THEN det_shutter[col,idet]=2
      IF temp1 EQ 1 AND temp2 EQ 0 THEN det_shutter[col,idet]=3
      IF temp1 EQ 1 AND temp2 EQ 1 THEN det_shutter[col,idet]=4
        
      ; Add up the frequency of each shutter order.
      IF det_shutter[col,idet] EQ 1 THEN temp3[0]=temp3[0]+1
      IF det_shutter[col,idet] EQ 2 THEN temp3[1]=temp3[1]+1
      IF det_shutter[col,idet] EQ 3 THEN temp3[2]=temp3[2]+1
      IF det_shutter[col,idet] EQ 4 THEN temp3[3]=temp3[3]+1
   ENDFOR

   ; Determine the shutter order.
   high_val=MAX(temp3[*])
   high_shutter=WHERE(temp3[*] EQ high_val)
   shutter[col]=high_shutter[0]+1

   IF fpa EQ 'VisNIR' OR fpa EQ 'DNB' THEN BEGIN
      IF band_type[senderband] EQ 1 THEN BEGIN
         IF high_val LT 16 OR n_elements(high_shutter) GT 1 THEN collect_mark[col,0]=1
      ENDIF
      IF band_type[senderband] EQ 0 THEN BEGIN
         IF high_val LT 8 OR n_elements(high_shutter) GT 1 THEN collect_mark[col,0]=1
      ENDIF
   ENDIF
   IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
      IF band_type[senderband] EQ 1 THEN BEGIN
         IF high_val LT 8 OR n_elements(high_shutter) GT 1 THEN collect_mark[col,0]=1
      ENDIF
      IF band_type[senderband] EQ 0 THEN BEGIN
         IF high_val LT 4 OR n_elements(high_shutter) GT 1 THEN collect_mark[col,0]=1
      ENDIF
   ENDIF
   print,'Collect=',col,' Shutter=',shutter[col],collect_mark[col,0],lambda[col],high_val
ENDFOR

;*****************************************************************************
;  Determine shutter order using telemetry from good collect to next good collect.
;*****************************************************************************

; Treat the edge collects that are not bounded by collects with high
; confidence seperately.
FOR col=ncollects-1, 0, -1 DO BEGIN
   IF lambda[col] LE lambda_center[senderband] THEN continue
   
   IF collect_mark[ncollects-1,0] NE 0 THEN edge[col,0]=1
   IF collect_mark[ncollects-1,0] NE 0 AND collect_mark[col,0] NE 0 THEN edge[col,0]=1
   IF collect_mark[ncollects-1,0] NE 0 AND collect_mark[col,0] EQ 0 THEN break
ENDFOR

; Work from the center wavelength to collects with higher wavelength.
FOR col=0, ncollects-1 DO BEGIN
   IF lambda[col] LE lambda_center[senderband] THEN continue
    
   ; Find the collects with low confidence.
   IF collect_mark[col,0] NE 0 THEN BEGIN
      IF edge[col,0] EQ 1 THEN collect_mark[col,1]=1
      IF edge[col,0] EQ 1 THEN continue

      ; Mark all collects with low confidence.
      IF col+1 GT ncollects-1 THEN BEGIN
         FOR col_2=col_1, col+1 DO BEGIN
            IF collect_mark[col_2,0] NE 1 THEN continue
            collect_mark[col_2,1]=1
         ENDFOR
         continue
      ENDIF
      ; Find the shutter order from one collect back and telemetry map to the present collect.
      IF collect_mark[col-1,0] EQ 0 AND col NE ncollects-1 THEN BEGIN
         col_1=col
         shutter_tel[col,0]=shutter_map(change_order[col],shutter[col-1],0)
         
         print,'START ',col-1,shutter[col-1],shutter[col-1],change_order[col-1],collect_mark[col-1,0]
         print,'MIDDLE',col,shutter_tel[col,0],shutter[col],change_order[col],collect_mark[col,0]
      ENDIF
      ; If multiple collects in a row with low confidence, telemetry map through.
      IF collect_mark[col,0] NE 0 AND collect_mark[col-1,0] NE 0 THEN BEGIN
         shutter_tel[col,0]=shutter_map(change_order[col],shutter_tel[col-1,0],0)
         
         print,'MIDDLE',col,shutter_tel[col,0],shutter[col],change_order[col],collect_mark[col,0]
      ENDIF
      ; Telemetry map to one collect forward.
      IF collect_mark[col+1,0] EQ 0 THEN BEGIN
         shutter_tel[col+1,0]=shutter_map(change_order[col+1],shutter_tel[col,0],0)
         
         print,'STOP  ',col+1,shutter_tel[col+1,0],shutter[col+1],change_order[col+1],collect_mark[col+1,0]
         print,''
            
         ; Mark all collects with low confidence that were not telemetry corrected.
         IF shutter_tel[col+1,0] NE shutter[col+1] THEN BEGIN
            FOR col_2=col_1, col DO collect_mark[col_2,1]=1
         ENDIF
         IF shutter_tel[col+1,0] EQ shutter[col+1] THEN BEGIN
            FOR col_2=col_1, col+1 DO BEGIN
               print,col_2,shutter_tel[col_2,0],shutter[col_2]
               shutter[col_2]=shutter_tel[col_2,0]
            ENDFOR
         ENDIF
         print,''
      ENDIF
   ENDIF
ENDFOR

; Treat the edge collects that are not bounded by collects with high
; confidence seperately.
FOR col=0, ncollects-1 DO BEGIN
   IF lambda[col] GT lambda_center[senderband] THEN continue
    
   IF collect_mark[0,0] NE 0 THEN edge[col,0]=1
   IF collect_mark[0,0] NE 0 AND collect_mark[col,0] NE 0 THEN edge[col,0]=1
   IF collect_mark[0,0] NE 0 AND collect_mark[col,0] EQ 0 THEN break
ENDFOR

; Work from the center wavelength to collects with lower wavelength.
FOR col=ncollects-1, 0, -1 DO BEGIN
   IF lambda[col] GT lambda_center[senderband] THEN continue

   ; Find the collects with low confidence.
   IF collect_mark[col,0] NE 0 THEN BEGIN
      IF edge[col,0] EQ 1 THEN collect_mark[col,1]=1
      IF edge[col,0] EQ 1 THEN continue
        
      ; Mark all collects with low confidence.
      IF col-1 LT 0 THEN BEGIN
         FOR col_2=col, col_1 DO BEGIN
            IF collect_mark[col_2,0] NE 1 THEN continue
            collect_mark[col_2,1]=1
         ENDFOR
         continue
      ENDIF
      ; Find the shutter order from one collect back and telemetry map to the present collect.
      IF collect_mark[col+1,0] EQ 0 AND col NE 0 THEN BEGIN
         col_1=col
         shutter_tel[col,1]=shutter_map(change_order[col+1],shutter[col+1],1)
         
         print,'START ',col+1,shutter[col+1],shutter[col+1],change_order[col+1],collect_mark[col+1,0]
         print,'MIDDLE',col,shutter_tel[col,1],shutter[col],change_order[col],collect_mark[col,0]
      ENDIF
      ; If multiple collects in a row with low confidence, telemetry map through.
      IF collect_mark[col,0] NE 0 AND collect_mark[col+1,0] NE 0 THEN BEGIN
         shutter_tel[col,1]=shutter_map(change_order[col+1],shutter_tel[col+1,1],1)
         
         print,'MIDDLE',col,shutter_tel[col,1],shutter[col],change_order[col],collect_mark[col,0]
      ENDIF
      ; Telemetry map to one collect forward.
      IF collect_mark[col-1,0] EQ 0 THEN BEGIN
         shutter_tel[col-1,1]=shutter_map(change_order[col],shutter_tel[col,1],1)
         
         print,'STOP  ',col-1,shutter_tel[col-1,1],shutter[col-1],change_order[col-1],collect_mark[col-1,0]
         print,''
            
         ; Mark all collects with low confidence that were not telemetry corrected.
         IF shutter_tel[col-1,1] NE shutter[col-1] THEN BEGIN
            FOR col_2=col, col_1 DO collect_mark[col_2,1]=1
         ENDIF
         IF shutter_tel[col-1,1] EQ shutter[col-1] THEN BEGIN
            FOR col_2=col-1, col_1 DO BEGIN
               print,col_2,shutter_tel[col_2,1],shutter[col_2],change_order[col_2]
               shutter[col_2]=shutter_tel[col_2,1]
            ENDFOR
         ENDIF
         print,''
      ENDIF
   ENDIF
ENDFOR

; The number of collects with low confidence with and without the
; telemetry correction.
x=WHERE(collect_mark[*,0] EQ 1,x1)
y=WHERE(collect_mark[*,1] EQ 1,y1)
print,x1,y1
print,x
print,y

;*****************************************************************************
;  Same as above but from 2 collects before bad collect to 2 collects after.
;*****************************************************************************

; Work from the center wavelength to collects with higher wavelength.
FOR col=0, ncollects-1 DO BEGIN
   IF lambda[col] LE lambda_center[senderband] THEN continue
    
   ; Find the collects with low confidence.
   IF collect_mark[col,1] NE 0 THEN BEGIN
      IF edge[col,0] EQ 1 THEN collect_mark[col,2]=1
      IF edge[col,0] EQ 1 THEN continue
        
      ; Mark all collects with low confidence.
      IF col+2 GT ncollects-1 THEN BEGIN
         FOR col_2=col_1, col+1 DO BEGIN
            IF collect_mark[col_2,1] NE 1 THEN continue
            collect_mark[col_2,2]=1
         ENDFOR
         continue
      ENDIF
      ; Find the shutter order from two collects back and telemetry map to the present collect.
      IF collect_mark[col-1,1] EQ 0 AND collect_mark[col-2,1] EQ 0 THEN BEGIN
         col_1=col
         shutter_tel[col-1,0]=shutter_map(change_order[col-1],shutter[col-2],0)
         shutter_tel[col,0]=shutter_map(change_order[col],shutter_tel[col-1,0],0)
         
         print,'START ',col-2,shutter[col-2],shutter[col-2],change_order[col-2],collect_mark[col-2,1]
         print,'START ',col-1,shutter_tel[col-1],shutter[col-1],change_order[col-1],collect_mark[col-1,1]
         print,'MIDDLE',col,shutter_tel[col,0],shutter[col],change_order[col],collect_mark[col,1]
      ENDIF
      ; If multiple collects in a row with low confidence, telemetry map through.
      IF collect_mark[col,1] NE 0 AND collect_mark[col-1,1] NE 0 THEN BEGIN
         shutter_tel[col,0]=shutter_map(change_order[col],shutter_tel[col-1,0],0)
         
         print,'MIDDLE',col,shutter_tel[col,0],shutter[col],change_order[col],collect_mark[col,0]
      ENDIF
      IF collect_mark[col+1,1] EQ 0 AND collect_mark[col+2,1] NE 0 THEN BEGIN
         shutter_tel[col+1,0]=shutter_map(change_order[col+1],shutter_tel[col,0],0)
         shutter_tel[col+2,0]=shutter_map(change_order[col+2],shutter_tel[col+1,0],0)
         
         print,'MIDDLE',col+1,shutter_tel[col+1,1],shutter[col+1],change_order[col+1],collect_mark[col+1,1]
         print,'MIDDLE',col+2,shutter_tel[col+2,1],shutter[col+2],change_order[col+2],collect_mark[col+2,1]
      ENDIF
      ; Telemetry map to two collects forward.
      IF collect_mark[col+1,1] EQ 0 AND collect_mark[col+2,1] EQ 0 THEN BEGIN
         shutter_tel[col+1,0]=shutter_map(change_order[col+1],shutter_tel[col,0],0)
         shutter_tel[col+2,0]=shutter_map(change_order[col+2],shutter_tel[col+1,0],0)
         
         print,'STOP  ',col+1,shutter_tel[col+1,0],shutter[col+1],change_order[col+1],collect_mark[col+1,1]
         print,'STOP  ',col+2,shutter_tel[col+2,0],shutter[col+2],change_order[col+2],collect_mark[col+2,1]
         print,''
            
         ; Mark all collects with low confidence that were not telemetry corrected.
         IF shutter_tel[col+2,0] NE shutter[col+2] THEN BEGIN
            FOR col_2=col_1, col DO BEGIN
               IF collect_mark[col_2,1] NE 1 THEN continue
               collect_mark[col_2,2]=1
            ENDFOR
         ENDIF
         IF shutter_tel[col+2,0] EQ shutter[col+2] THEN BEGIN
            FOR col_2=col_1+1, col+1 DO BEGIN
               print,col_2,shutter_tel[col_2,0],shutter[col_2],change_order[col_2]
               shutter[col_2]=shutter_tel[col_2,0]
            ENDFOR
         ENDIF
         print,''
      ENDIF
   ENDIF
ENDFOR

; Work from the center wavelength to collects with lower wavelength.
FOR col=ncollects-1, 0, -1 DO BEGIN
   IF lambda[col] GT lambda_center[senderband] THEN continue

   ; Find the collects with low confidence.
   IF collect_mark[col,1] NE 0 THEN BEGIN
      IF edge[col,0] EQ 1 THEN collect_mark[col,2]=1
      IF edge[col,0] EQ 1 THEN continue
      
      ; Mark all collects with low confidence.
      IF col-3 LT 0 THEN BEGIN
         FOR col_2=col, col_1 DO BEGIN
            IF collect_mark[col_2,1] NE 1 THEN continue
            collect_mark[col_2,2]=1
         ENDFOR
      ENDIF
      IF col-2 LT 0 THEN BEGIN
         FOR col_2=col, col_1 DO BEGIN
            IF collect_mark[col_2,1] NE 1 THEN continue
            collect_mark[col_2,2]=1
         ENDFOR
         continue
      ENDIF
      ; Find the shutter order from two collects back and telemetry map to the present collect.
      IF collect_mark[col+1,1] EQ 0 AND collect_mark[col+2,1] EQ 0 THEN BEGIN
         col_1=col
         shutter_tel[col+1,1]=shutter_map(change_order[col+2],shutter[col+2],1)
         shutter_tel[col,1]=shutter_map(change_order[col+1],shutter_tel[col+1,1],1)
         
         print,'START ',col+2,shutter[col+2],shutter[col+2],change_order[col+3],collect_mark[col+2,1]
         print,'START ',col+1,shutter_tel[col+1,1],shutter[col+1],change_order[col+2],collect_mark[col+1,1]
         print,'MIDDLE',col,shutter_tel[col,1],shutter[col],change_order[col+1],collect_mark[col,1]
      ENDIF
      ; If multiple collects in a row with low confidence, telemetry map through.
      IF collect_mark[col,1] NE 0 AND collect_mark[col+1,1] NE 0 THEN BEGIN
         shutter_tel[col,1]=shutter_map(change_order[col+1],shutter_tel[col+1,1],1)
         
         print,'MIDDLE',col,shutter_tel[col,1],shutter[col],change_order[col+1],collect_mark[col,1]
      ENDIF
      IF collect_mark[col-1,1] EQ 0 AND collect_mark[col-2,1] NE 0 THEN BEGIN
         shutter_tel[col-1,1]=shutter_map(change_order[col],shutter_tel[col,1],1)
         shutter_tel[col-2,1]=shutter_map(change_order[col-1],shutter_tel[col-1,1],1)
         
         print,'MIDDLE',col-1,shutter_tel[col-1,1],shutter[col-1],change_order[col],collect_mark[col-1,1]
         print,'MIDDLE',col-2,shutter_tel[col-2,1],shutter[col-2],change_order[col-1],collect_mark[col-2,1]
      ENDIF
      ; Telemetry map to two collects forward.
      IF collect_mark[col-1,1] EQ 0 AND collect_mark[col-2,1] EQ 0 THEN BEGIN
         shutter_tel[col-1,1]=shutter_map(change_order[col],shutter_tel[col,1],1)
         shutter_tel[col-2,1]=shutter_map(change_order[col-1],shutter_tel[col-1,1],1)
         
         print,'STOP  ',col-1,shutter_tel[col-1,1],shutter[col-1],change_order[col],collect_mark[col-1,1]
         print,'STOP  ',col-2,shutter_tel[col-2,1],shutter[col-2],change_order[col-1],collect_mark[col-2,1]
         print,''
         
         ; Mark all collects with low confidence that were not telemetry corrected.
         IF shutter_tel[col-2,1] NE shutter[col-2] THEN BEGIN
            FOR col_2=col, col_1 DO BEGIN
               IF collect_mark[col_2,1] NE 1 THEN continue
               collect_mark[col_2,2]=1
            ENDFOR
         ENDIF
         IF shutter_tel[col-2,1] EQ shutter[col-2] THEN BEGIN
            FOR col_2=col-1, col_1+1 DO BEGIN
               print,col_2,shutter_tel[col_2,1],shutter[col_2],change_order[col_2]
               
               IF collect_mark[col_2,1] NE 1 THEN continue
               shutter[col_2]=shutter_tel[col_2,1]
               collect_mark[col_2,2]=0
            ENDFOR
         ENDIF
         print,''
      ENDIF
   ENDIF
ENDFOR

; The number of collects with low confidence with and without the two telemetry corrections.
x=WHERE(collect_mark[*,0] EQ 1,x1)
y=WHERE(collect_mark[*,1] EQ 1,y1)
z=WHERE(collect_mark[*,2] EQ 1,z1)
print,x1,y1,z1
print,y
print,z

;*****************************************************************************
;  Background subtract and average over HAM sides (since it is a staring test).
;*****************************************************************************

FOR col=0, ncollects-1 DO BEGIN

    ; Translate the shutter order into the appropriate background subtraction.
   IF shutter[col] EQ 1 THEN alpha=[1,1,-1,-1]
   IF shutter[col] EQ 2 THEN alpha=[1,-1,-1,1]
   IF shutter[col] EQ 3 THEN alpha=[-1,1,1,-1]
   IF shutter[col] EQ 4 THEN alpha=[-1,-1,1,1]
   
   IF mean(dn_temp[col,senderband,*,*,0,*,*]) EQ 0 THEN continue
   
   IF fpa EQ 'DNB' AND senderband EQ 0 THEN igain=2 ELSE igain=1

   FOR b=0, nbands-1 DO BEGIN
      FOR id=0, detectors[b]-1 DO BEGIN
         FOR is=0, subsets[b]-1 DO BEGIN
            FOR ih=0, nham-1 DO BEGIN
               dn_collect[col,b,ih,id,is] $
                  =alpha[0]*dn_temp[col,b,ih,id,is,igain,0] $
                  +alpha[1]*dn_temp[col,b,ih,id,is,igain,1] $
                  +alpha[2]*dn_temp[col,b,ih,id,is,igain,2] $
                  +alpha[3]*dn_temp[col,b,ih,id,is,igain,3]

               ; Dark noise.
               std_collect[col,b,ih,id,is,0] $
                  =(1-alpha[0])/2*std_temp[col,b,ih,id,is,igain,0] $
                  +(1-alpha[1])/2*std_temp[col,b,ih,id,is,igain,1] $
                  +(1-alpha[2])/2*std_temp[col,b,ih,id,is,igain,2] $
                  +(1-alpha[3])/2*std_temp[col,b,ih,id,is,igain,3]
               
               ; Open noise.
               std_collect[col,b,ih,id,is,1] $
                  =(1+alpha[0])/2*std_temp[col,b,ih,id,is,igain,0] $
                  +(1+alpha[1])/2*std_temp[col,b,ih,id,is,igain,1] $
                  +(1+alpha[2])/2*std_temp[col,b,ih,id,is,igain,2] $
                  +(1+alpha[3])/2*std_temp[col,b,ih,id,is,igain,3]
            ENDFOR
            
            ; HAM average.
            dn[col,b,id,is]=(dn_collect[col,b,0,id,is]+dn_collect[col,b,1,id,is])/2
            std[col,b,id,is,0:1]=(std_collect[col,b,0,id,is,0:1]+std_collect[col,b,1,id,is,0:1])/2
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Output dn and std to files.
;*****************************************************************************

save,/compress,ncollects,senderband,shutter,collect_mark,dn,std,filename=outpath+uaidname+'.oob.sav'

outfile2=outpath+uaidname+'.temp.txt'

fmt_1='(i3, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, i2)'

openw,lun,outfile2,/get_lun
FOR col=0, ncollects-1 DO BEGIN
   printf,lun,format=fmt_1,col,collect_mark[col,0],collect_mark[col,1],collect_mark[col,2],shutter[col]
ENDFOR
free_lun,lun
close,lun,/file,/all

END
