;*****************************************************************************
;  FP-15 and FP-16 RSR test. Reads and formats the response. Uses binary extracted data.
;
;  INPUTS:
;  1) infile - file with input, output paths, sender and uaid name, and fpa

;  OUTPUTS:
;  1) dn_temp[collects,band,HAM,detector,subsample,four scan cycle]
;      -- DN averaged over every fourth scan.
;  2) dn_temp_scan[collects,band,HAM,detector,subsample,scan]
;      -- scan dependent DN.
;  3) std_temp[collects,band,HAM,detector,subsample,four scan cycle]
;      -- standard deviation averaged over every fourth scan.
;  4) senderband -- number associated with sender band.
;  5) ncollects -- number of collects.
;  
;  v1.0 - 11/21/2008 - Jeff McIntire
;  v2.0 - 05/30/2014 - Jeff McIntire
;       - rewritten for F2
;*****************************************************************************

PRO fp15_16_read_dn_3, infile

;*****************************************************************************
;  Read in input file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
inpath2=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
uaidname=strarr(1)
sender=strarr(1)
fpa=strarr(1)
odd_even=strarr(1)
header_size=intarr(1)
list_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath	
readf,lun,inpath2
readf,lun,outpath
readf,lun,uaidname
readf,lun,sender
readf,lun,fpa
readf,lun,odd_even
readf,lun,header_size
readf,lun,list_size
Free_Lun,lun

;*****************************************************************************
;  Get the list of dump files.
;*****************************************************************************

; Get list of .dmp files from specified path. 
IF list_size EQ 0 THEN spawn,'ls -C1 '+inpath+'*.bin',list_files
IF list_size EQ 1 THEN BEGIN
   spawn,'ls -C1 '+inpath+uaidname+'_EV_C0*',list_files1
   spawn,'ls -C1 '+inpath+uaidname+'_EV_C1*',list_files2
   spawn,'ls -C1 '+inpath+uaidname+'_EV_C2*',list_files3
   
   list_files=strarr(n_elements(list_files1)+n_elements(list_files2)+n_elements(list_files3))
   list_files[0:n_elements(list_files1)-1]=list_files1[0:n_elements(list_files1)-1]
   list_files[n_elements(list_files1):n_elements(list_files1)+n_elements(list_files2)-1]=list_files2[0:n_elements(list_files2)-1]
   list_files[n_elements(list_files1)+n_elements(list_files2):n_elements(list_files1)+n_elements(list_files2)+n_elements(list_files3)-1] $
      =list_files3[0:n_elements(list_files3)-1]
ENDIF
IF n_elements(list_files) EQ 0 THEN goto,skipout
nfile=n_elements(list_files)

;*****************************************************************************
;  Select focal plane dependent parameters.
;*****************************************************************************

IF fpa EQ 'DNB' THEN restore,'general.DNB.2.sav'
IF fpa EQ 'VisNIR' THEN restore,'general.VisNIR.2.sav'
IF fpa EQ 'SMWIR' THEN restore,'general.SMWIR.2.sav'
IF fpa EQ 'LWIR' THEN restore,'general.LWIR.2.sav'

IF fpa EQ 'DNB' THEN maxgain=3

ncollects=nfile

;*****************************************************************************
;  Definitions
;*****************************************************************************

;get the maximum number of scans.
read_dump_bin,list_files[0],scan,ham,det,frame,gain_type,dns,UAID=uaid,COLLECT=collect,APID=apid,BAND=band,DNB_S=dnb_s,DSECT=dsect,OFFSET=offset,msamp_num=6400
max_scan=max(scan)

dn_temp=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,4)
dn_temp_scan=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan)
std_temp=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,4)
std_temp_scan=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan)

; Slope over frames.
slope1=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxgain,max_scan,2)

;*****************************************************************************
;  Main program. Read data from dump files.
;*****************************************************************************

a5=-1
FOR ifile=0, nfile-1 DO BEGIN

   ; Detemine collect number (a5).
   a5=a5+1

   ; Read the dump file.
   print,list_files[ifile],' ',a5
   nlines=File_Lines(list_files[ifile])
   IF nlines EQ 0 THEN continue
    
   read_dump_bin,list_files[ifile],scan,ham,det,frame,gain_type,dns,UAID=uaid,COLLECT=collect,APID=apid,BAND=band,DNB_S=dnb_s,DSECT=dsect,OFFSET=offset,msamp_num=6400
   nlines=n_elements(dns)
    
   max_scan=max(scan)                 ; Total # scans.
   max_frame=max(frame)-min(frame)+1  ; Total # frames.

   ; Translate the band name to a number.
   FOR iband=0, nbands-1 DO BEGIN
      IF band[0] EQ band_name[iband] THEN b=iband
   ENDFOR

;*****************************************************************************
;  Write DN values to an array.
;*****************************************************************************

   dn_frame=dblarr(nham,maxdet,maxsubsets,maxgain,max_scan,max_frame)
   iscan=0
   idet=0
   iframe=0
   subset=min(frame) MOD 2
   FOR i=0L, nlines-1 DO BEGIN
      IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
         dn_frame[ham[i],idet,subset,gain_type[i],iscan,iframe]=dns[i]
         iframe=iframe+1
         IF iframe EQ max_frame THEN BEGIN
            idet=idet+1
            iframe=0
            IF idet EQ detectors[b] THEN BEGIN
               idet=0
               iscan=iscan+1
            ENDIF
         ENDIF
      ENDIF
      IF band_type[b] EQ 1 THEN BEGIN
         IF subset EQ 0 THEN BEGIN
            subset=1
            goto,skip_over_1
         ENDIF
         IF subset EQ 1 THEN subset=0
         skip_over_1:
      ENDIF
   ENDFOR
 
;*****************************************************************************
;  Average over frames.
;*****************************************************************************

   dn_frame_avg=dblarr(nham,detectors[b],subsets[b],maxgain,max_scan)
   std_frame_avg=dblarr(nham,detectors[b],subsets[b],maxgain,max_scan)
   FOR iscan=0, max_scan-1 DO BEGIN
      FOR isubset=0, subsets[b]-1 DO BEGIN
         FOR idet=0, detectors[b]-1 DO BEGIN
            FOR igain=0, maxgain-1 DO BEGIN
               FOR iham=0, nham-1 DO BEGIN
                  dn_frame_avg[iham,idet,isubset,igain,iscan]=sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],-1,1)
                  std_frame_avg[iham,idet,isubset,igain,iscan]=sig_mean(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1],-1,2)
                  
                  IF dn_frame_avg[iham,idet,isubset,igain,iscan] EQ -999 THEN dn_frame_avg[iham,idet,isubset,igain,iscan]=0
                  IF std_frame_avg[iham,idet,isubset,igain,iscan] EQ -999 THEN std_frame_avg[iham,idet,isubset,igain,iscan]=0
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR

;*****************************************************************************
;  Calculate drifting of dn over frames using a straight line fit.
;*****************************************************************************
   
   f=findgen(max_frame)
   FOR iham=0, nham-1 DO BEGIN
      FOR idet=0, detectors[b]-1 DO BEGIN
         FOR isubset=0, subsets[b]-1 DO BEGIN
            FOR igain=0, maxgain-1 DO BEGIN
               FOR iscan=0, max_scan-1 DO BEGIN
                  x=WHERE(dn_frame[iham,idet,isubset,igain,iscan,0:max_frame-1] NE 0,num)
                  IF num LE 1 THEN continue
                  
                  result1=poly_fit(f[x],dn_frame[iham,idet,isubset,igain,iscan,x],1)
                  
                  slope1[a5,b,iham,idet,isubset,igain,iscan,*]=result1[*]
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR

;*****************************************************************************
;  Average every fourth scan.
;*****************************************************************************

   dn_scan_avg=dblarr(nham,detectors[b],subsets[b],maxgain,4)
   std_scan_avg=dblarr(nham,detectors[b],subsets[b],maxgain,4)
   FOR isubset=0, subsets[b]-1 DO BEGIN
      FOR idet=0, detectors[b]-1 DO BEGIN
         FOR iham=0, nham-1 DO BEGIN
            FOR igain=0, maxgain-1 DO BEGIN
               FOR iscan=0, max_scan-4, 4 DO BEGIN
                  dn_scan_avg[iham,idet,isubset,igain,0]=dn_scan_avg[iham,idet,isubset,igain,0]+dn_frame_avg[iham,idet,isubset,igain,iscan]
                  dn_scan_avg[iham,idet,isubset,igain,1]=dn_scan_avg[iham,idet,isubset,igain,1]+dn_frame_avg[iham,idet,isubset,igain,iscan+1]
                  dn_scan_avg[iham,idet,isubset,igain,2]=dn_scan_avg[iham,idet,isubset,igain,2]+dn_frame_avg[iham,idet,isubset,igain,iscan+2]
                  dn_scan_avg[iham,idet,isubset,igain,3]=dn_scan_avg[iham,idet,isubset,igain,3]+dn_frame_avg[iham,idet,isubset,igain,iscan+3]
                  
                  std_scan_avg[iham,idet,isubset,igain,0]=std_scan_avg[iham,idet,isubset,igain,0]+std_frame_avg[iham,idet,isubset,igain,iscan]
                  std_scan_avg[iham,idet,isubset,igain,1]=std_scan_avg[iham,idet,isubset,igain,1]+std_frame_avg[iham,idet,isubset,igain,iscan+1]
                  std_scan_avg[iham,idet,isubset,igain,2]=std_scan_avg[iham,idet,isubset,igain,2]+std_frame_avg[iham,idet,isubset,igain,iscan+2]
                  std_scan_avg[iham,idet,isubset,igain,3]=std_scan_avg[iham,idet,isubset,igain,3]+std_frame_avg[iham,idet,isubset,igain,iscan+3]
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
   dn_temp[a5,b,0:1,0:detectors[b]-1,0:subsets[b]-1,0:maxgain-1,0:3]=dn_scan_avg[0:1,0:detectors[b]-1,0:subsets[b]-1,0:maxgain-1,0:3]/(max_scan/4)
   dn_temp_scan[a5,b,0:1,0:detectors[b]-1,0:subsets[b]-1,0:maxgain-1,0:max_scan-1]=dn_frame_avg[0:1,0:detectors[b]-1,0:subsets[b]-1,0:maxgain-1,0:max_scan-1]
   std_temp[a5,b,0:1,0:detectors[b]-1,0:subsets[b]-1,0:maxgain-1,0:3]=std_scan_avg[0:1,0:detectors[b]-1,0:subsets[b]-1,0:maxgain-1,0:3]/(max_scan/4)
   std_temp_scan[a5,b,0:1,0:detectors[b]-1,0:subsets[b]-1,0:maxgain-1,0:max_scan-1]=std_frame_avg[0:1,0:detectors[b]-1,0:subsets[b]-1,0:maxgain-1,0:max_scan-1]
ENDFOR

;*****************************************************************************
;  Detemine the sender band.
;*****************************************************************************

FOR b=0, nbands-1 DO BEGIN
   IF sender EQ band_name[b] THEN senderband=b
ENDFOR

;*****************************************************************************
;  Output to sav file
;*****************************************************************************

save,/compress,ncollects,senderband,dn_temp,dn_temp_scan,std_temp,filename=outpath+uaidname+'.dn.1.sav'

;*****************************************************************************
;  Set DN Plot environment.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
device,/landscape,filename=outpath+uaidname+'.DN_scan.err_bars.ps',/color
!p.multi=[0,1,1]

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

m_det_even=[1,3,5,7,9,11,13,15]
i_det_even=[1,3,5,7,9,11,13,15,17,19,21,23,25,27,29,31]
m_det_odd=[0,2,4,6,8,10,12,14]
i_det_odd=[0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30]

name1=['2','4','6','8','10','12','14','16']
name2=['2','4','6','8','10','12','14','16','18','20','22','24','26','28','30','32']
name3=['1','3','5','7','9','11','13','15']
name4=['1','3','5','7','9','11','13','15','17','19','21','23','25','27','29','31']

IF maxgain EQ 2 THEN gain_state=['Low gain','High gain']
IF maxgain EQ 3 THEN gain_state=['HGS','MGS','LGS']

D=findgen(max_scan)

;*****************************************************************************
;  Redefine the dn such that no zeros in the arrays.
;*****************************************************************************

dn_x=fltarr(ncollects,nbands,maxdet,maxsubsets,maxgain,max_scan)
std_x=fltarr(ncollects,nbands,maxdet,maxsubsets,maxgain,max_scan)
slope=fltarr(ncollects,nbands,maxdet,maxsubsets,maxgain,max_scan,2)
FOR collects=0, ncollects-1 DO BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR isubset=0, subsets[b]-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
               iham=0
               IF dn_temp[collects,b,0,idet,isubset,igain,0] EQ 0 THEN iham=1
               
               IF mean(dn_temp_scan[collects,b,iham,idet,isubset,igain,0:max_scan-1]) EQ 0 THEN continue
               FOR iscan=0, max_scan-1 DO BEGIN
                  dn_x[collects,b,idet,isubset,igain,iscan]=dn_temp_scan[collects,b,iham,idet,isubset,igain,iscan]
                  std_x[collects,b,idet,isubset,igain,iscan]=std_temp_scan[collects,b,iham,idet,isubset,igain,iscan]
                  slope[collects,b,idet,isubset,igain,iscan,0:1]=slope1[collects,b,iham,idet,isubset,igain,iscan,0:1]
                  
                  iham=iham+1
                  IF iham EQ 2 THEN iham=0
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  For VisNIR plot all detectors on one page.
;*****************************************************************************

IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='DN vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+ ' '+gain_state[igain]
               
               ymax=max(dn_x[collects,b,0:detectors[b]-1,isubset,igain,*],min=ymin)
               ymax=ymax+max(std_x[collects,b,0:detectors[b]-1,isubset,igain,0:max_scan-1])
               ymin=ymin-max(std_x[collects,b,0:detectors[b]-1,isubset,igain,0:max_scan-1])
               
               plot,D[0:max_scan-1]+1,dn_x[collects,b,detectors[b]/2,isubset,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='DN', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               
               FOR idet=0, detectors[b]-1 DO BEGIN
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=3
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[idet],thick=3
                  
                  FOR iscan=0, max_scan-1 DO BEGIN
                     oplot,[D[iscan]+1,D[iscan]+1],[dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan], $
                                                    dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[idet],thick=2 
                     oplot,[D[iscan]+0.75,D[iscan]+1.25],[dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan], $
                                                          dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[idet],thick=2
                     oplot,[D[iscan]+0.75,D[iscan]+1.25],[dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan], $
                                                          dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[idet],thick=2
                  ENDFOR
               ENDFOR
               XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
               IF b LE 1 THEN set_legend,2,chname,lcol,symbol
               IF b GT 1 THEN set_legend,1,chname,lcol,symbol
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

;*****************************************************************************
;  For SMWIR and LWIR plot odd and even detectors on seperately.
;*****************************************************************************

IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='DN vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+', odd detectors'
               
               IF band_type[b] EQ 1 THEN BEGIN
                  ymax=max(dn_x[collects,b,i_det_odd,isubset,igain,0:max_scan-1],min=ymin)
                  ymax=ymax+max(std_x[collects,b,i_det_odd,isubset,igain,0:max_scan-1])
                  ymin=ymin-max(std_x[collects,b,i_det_odd,isubset,igain,0:max_scan-1])
               ENDIF
               IF band_type[b] EQ 0 THEN BEGIN
                  ymax=max(dn_x[collects,b,m_det_odd,isubset,igain,0:max_scan-1],min=ymin)
                  ymax=ymax+max(std_x[collects,b,m_det_odd,isubset,igain,0:max_scan-1])
                  ymin=ymin-max(std_x[collects,b,m_det_odd,isubset,igain,0:max_scan-1])
               ENDIF
               
               plot,D[0:max_scan-1]+1,dn_x[collects,b,8,isubset,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='DN', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               i=0
               FOR idet=0, detectors[b]-1 DO BEGIN
                  IF idet/2-idet/2. NE 0 THEN continue
                  
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[i],psym=symbol[i],thick=3
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[i],thick=3
                  
                  FOR iscan=0, max_scan-1 DO BEGIN
                     oplot,[D[iscan]+1,D[iscan]+1],[dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan], $
                                                    dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[i],thick=2 
                     oplot,[D[iscan]+0.75,D[iscan]+1.25],[dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan], $
                                                          dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[i],thick=2
                     oplot,[D[iscan]+0.75,D[iscan]+1.25],[dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan], $
                                                          dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[i],thick=2
                  ENDFOR
                  i=i+1
                  
                  XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
                  IF band_type[b] EQ 1 THEN set_legend,2,name4,lcol,symbol
                  IF band_type[b] EQ 0 THEN set_legend,1,name3,lcol,symbol
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
    
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='DN vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+', even detectors'
               
               IF band_type[b] EQ 1 THEN BEGIN
                  ymax=max(dn_x[collects,b,i_det_even,isubset,igain,0:max_scan-1],min=ymin)
                  ymax=ymax+max(std_x[collects,b,i_det_even,isubset,igain,0:max_scan-1])
                  ymin=ymin-max(std_x[collects,b,i_det_even,isubset,igain,0:max_scan-1])
               ENDIF
               IF band_type[b] EQ 0 THEN BEGIN
                  ymax=max(dn_x[collects,b,m_det_even,isubset,igain,0:max_scan-1],min=ymin)
                  ymax=ymax+max(std_x[collects,b,m_det_even,isubset,igain,0:max_scan-1])
                  ymin=ymin-max(std_x[collects,b,m_det_even,isubset,igain,0:max_scan-1])
               ENDIF
               
               plot,D[0:max_scan-1]+1,dn_x[collects,b,8,0,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='DN', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               
               i=0
               FOR idet=0, detectors[b]-1 DO BEGIN
                  IF idet/2-idet/2. EQ 0 THEN continue
                  
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[i],psym=symbol[i],thick=3
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[i],thick=3
                  
                  FOR iscan=0, max_scan-1 DO BEGIN
                     oplot,[D[iscan]+1,D[iscan]+1],[dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan], $
                                                    dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[i],thick=2 
                     oplot,[D[iscan]+0.75,D[iscan]+1.25],[dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan], $
                                                          dn_x[collects,b,idet,isubset,igain,iscan]-std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[i],thick=2
                     oplot,[D[iscan]+0.75,D[iscan]+1.25],[dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan], $
                                                          dn_x[collects,b,idet,isubset,igain,iscan]+std_x[collects,b,idet,isubset,igain,iscan]],color=lcol[i],thick=2
                  ENDFOR
                  i=i+1
                  
                  XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
                  IF band_type[b] EQ 1 THEN set_legend,2,name2,lcol,symbol
                  IF band_type[b] EQ 0 THEN set_legend,1,name1,lcol,symbol
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

device,/close

;*****************************************************************************
;  Set DN Plot environment.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
device,/landscape,filename=outpath+uaidname+'.DN_scan.ps',/color
!p.multi=[0,1,1]

;*****************************************************************************
;  For VisNIR plot all detectors on one page.
;*****************************************************************************

IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='DN vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+ ' '+gain_state[igain]
               
               ymax=max(dn_x[collects,b,0:detectors[b]-1,isubset,igain,*],min=ymin)
               
               plot,D[0:max_scan-1]+1,dn_x[collects,b,detectors[b]/2,isubset,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='DN', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               
               FOR idet=0, detectors[b]-1 DO BEGIN
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=3
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[idet],thick=3
               ENDFOR
               XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
               IF b LE 1 THEN set_legend,2,chname,lcol,symbol
               IF b GT 1 THEN set_legend,1,chname,lcol,symbol
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

;*****************************************************************************
;  For SMWIR and LWIR plot odd and even detectors on seperately.
;*****************************************************************************

IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='DN vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+', odd detectors'
               
               IF band_type[b] EQ 1 THEN ymax=max(dn_x[collects,b,i_det_odd,isubset,igain,0:max_scan-1],min=ymin)
               IF band_type[b] EQ 0 THEN ymax=max(dn_x[collects,b,m_det_odd,isubset,igain,0:max_scan-1],min=ymin)
               
               plot,D[0:max_scan-1]+1,dn_x[collects,b,8,isubset,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='DN', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               i=0
               FOR idet=0, detectors[b]-1 DO BEGIN
                  IF idet/2-idet/2. NE 0 THEN continue
                  
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[i],psym=symbol[i],thick=3
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[i],thick=3
                  i=i+1
                  
                  XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
                  IF band_type[b] EQ 1 THEN set_legend,2,name4,lcol,symbol
                  IF band_type[b] EQ 0 THEN set_legend,1,name3,lcol,symbol
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
   
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='DN vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+', even detectors'
               
               IF band_type[b] EQ 1 THEN ymax=max(dn_x[collects,b,i_det_even,isubset,igain,0:max_scan-1],min=ymin)
               IF band_type[b] EQ 0 THEN ymax=max(dn_x[collects,b,m_det_even,isubset,igain,0:max_scan-1],min=ymin)
               
               plot,D[0:max_scan-1]+1,dn_x[collects,b,8,0,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='DN', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               
               i=0
               FOR idet=0, detectors[b]-1 DO BEGIN
                  IF idet/2-idet/2. EQ 0 THEN continue
                  
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[i],psym=symbol[i],thick=3
                  oplot,D[0:max_scan-1]+1,dn_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[i],thick=3
                  i=i+1
                  
                  XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
                  IF band_type[b] EQ 1 THEN set_legend,2,name2,lcol,symbol
                  IF band_type[b] EQ 0 THEN set_legend,1,name1,lcol,symbol
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

device,/close

;*****************************************************************************
;  Set Standard Deviation Plot environment.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
device,/landscape,filename=outpath+uaidname+'.std_scan.ps',/color
!p.multi=[0,1,1]

;*****************************************************************************
;  For VisNIR plot all detectors on one page.
;*****************************************************************************

IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(std_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='Standard Deviation vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+ ' '+gain_state[igain]
               
               ymax=max(std_x[collects,b,0:detectors[b]-1,isubset,igain,*],min=ymin)
               
               plot,D[0:max_scan-1]+1,std_x[collects,b,detectors[b]/2,isubset,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan', $
                    ytit='Standard Deviation',title=title2,charsize=1.5,charthick=5,/nodata
               
               FOR idet=0, detectors[b]-1 DO BEGIN
                  oplot,D[0:max_scan-1]+1,std_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=3
                  oplot,D[0:max_scan-1]+1,std_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[idet],thick=3
               ENDFOR
               XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
               IF b LE 1 THEN set_legend,2,chname,lcol,symbol
               IF b GT 1 THEN set_legend,1,chname,lcol,symbol
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

;*****************************************************************************
;  For SMWIR and LWIR plot odd and even detectors on seperately.
;*****************************************************************************

IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='Standard Deviation vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+', odd detectors'
               
               IF band_type[b] EQ 1 THEN ymax=max(std_x[collects,b,i_det_odd,isubset,igain,0:max_scan-1],min=ymin)
               IF band_type[b] EQ 0 THEN ymax=max(std_x[collects,b,m_det_odd,isubset,igain,0:max_scan-1],min=ymin)
               plot,D[0:max_scan-1]+1,std_x[collects,b,8,isubset,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='Standard Deviation', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               
               i=0
               FOR idet=0, detectors[b]-1 DO BEGIN
                  IF idet/2-idet/2. EQ 0 THEN continue
                  
                  oplot,D[0:max_scan-1]+1,std_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[i],psym=symbol[i],thick=3
                  oplot,D[0:max_scan-1]+1,std_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[i],thick=3
                  i=i+1
                  
                  XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
                  IF band_type[b] EQ 1 THEN set_legend,2,name4,lcol,symbol
                  IF band_type[b] EQ 0 THEN set_legend,1,name3,lcol,symbol
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
    
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='Standard Deviation vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+', even detectors'
               
               IF band_type[b] EQ 1 THEN ymax=max(std_x[collects,b,i_det_even,isubset,igain,0:max_scan-1],min=ymin)
               IF band_type[b] EQ 0 THEN ymax=max(std_x[collects,b,m_det_even,isubset,igain,0:max_scan-1],min=ymin)
               plot,D[0:max_scan-1]+1,std_x[collects,b,8,0,igain,0:max_scan-1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='Standard Deviation', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               
               i=0
               FOR idet=0, detectors[b]-1 DO BEGIN
                  IF idet/2-idet/2. NE 0 THEN continue
                  
                  oplot,D[0:max_scan-1]+1,std_x[collects,b,idet,isubset,igain,0:max_scan-1],symsize=1,color=lcol[i],psym=symbol[i],thick=3
                  oplot,D[0:max_scan-1]+1,std_x[collects,b,idet,isubset,igain,0:max_scan-1],color=lcol[i],thick=3
                  i=i+1
                  
                  XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
                  IF band_type[b] EQ 1 THEN set_legend,2,name2,lcol,symbol
                  IF band_type[b] EQ 0 THEN set_legend,1,name1,lcol,symbol
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

device,/close

;*****************************************************************************
;  Set Slope Plot environment.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
device,/landscape,filename=outpath+uaidname+'.slope_scan.ps',/color
!p.multi=[0,1,1]

;*****************************************************************************
;  For VisNIR plot all detectors on one page.
;*****************************************************************************

IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(slope[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='Slope vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+ ' '+gain_state[igain]
               
               ymax=max(slope[collects,b,0:detectors[b]-1,isubset,igain,*,1],min=ymin)
               
               plot,D[0:max_scan-1]+1,slope[collects,b,detectors[b]/2,isubset,igain,0:max_scan-1,1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan', $
                    ytit='Slope',title=title2,charsize=1.5,charthick=5,/nodata
               
               FOR idet=0, detectors[b]-1 DO BEGIN
                  oplot,D[0:max_scan-1]+1,slope[collects,b,idet,isubset,igain,0:max_scan-1,1],symsize=1,color=lcol[idet],psym=symbol[idet],thick=3
                  oplot,D[0:max_scan-1]+1,slope[collects,b,idet,isubset,igain,0:max_scan-1,1],color=lcol[idet],thick=3
               ENDFOR
               XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
               IF b LE 1 THEN set_legend,2,chname,lcol,symbol
               IF b GT 1 THEN set_legend,1,chname,lcol,symbol
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

;*****************************************************************************
;  For SMWIR and LWIR plot odd and even detectors on seperately.
;*****************************************************************************

IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='Slope vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+', odd detectors'
               
               IF band_type[b] EQ 1 THEN ymax=max(slope[collects,b,i_det_odd,isubset,igain,0:max_scan-1,1],min=ymin)
               IF band_type[b] EQ 0 THEN ymax=max(slope[collects,b,m_det_odd,isubset,igain,0:max_scan-1,1],min=ymin)
               plot,D[0:max_scan-1]+1,slope[collects,b,8,isubset,igain,0:max_scan-1,1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='Slope', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               
               i=0
               FOR idet=0, detectors[b]-1 DO BEGIN
                  IF idet/2-idet/2. EQ 0 THEN continue
                  
                  oplot,D[0:max_scan-1]+1,slope[collects,b,idet,isubset,igain,0:max_scan-1,1],symsize=1,color=lcol[i],psym=symbol[i],thick=3
                  oplot,D[0:max_scan-1]+1,slope[collects,b,idet,isubset,igain,0:max_scan-1,1],color=lcol[i],thick=3
                  i=i+1
                  
                  XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
                  IF band_type[b] EQ 1 THEN set_legend,2,name4,lcol,symbol
                  IF band_type[b] EQ 0 THEN set_legend,1,name3,lcol,symbol
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
    ENDFOR
    
   FOR b=0, nbands-1 DO BEGIN
      FOR collects=0, ncollects-1 DO BEGIN
         FOR igain=0, maxgain-1 DO BEGIN
            IF mean(dn_x[collects,b,*,0,igain,0:max_scan-1]) EQ 0 THEN continue
            FOR isubset=0, subsets[b]-1 DO BEGIN
               title1='Slope vs scan'
               title2=band_name[b]+' Collect='+strtrim(string(collects+1),2)+' SS'+strtrim(string(isubset+1),2)+', even detectors'
               
               IF band_type[b] EQ 1 THEN ymax=max(slope[collects,b,i_det_even,isubset,igain,0:max_scan-1,1],min=ymin)
               IF band_type[b] EQ 0 THEN ymax=max(slope[collects,b,m_det_even,isubset,igain,0:max_scan-1,1],min=ymin)
               plot,D[0:max_scan-1]+1,slope[collects,b,8,0,igain,0:max_scan-1,1],yrange=[ymin,ymax],xrange=[0,max_scan+1],xtit='Scan',ytit='Slope', $
                    title=title2,charsize=1.5,charthick=5,/nodata
               
               i=0
               FOR idet=0, detectors[b]-1 DO BEGIN
                  IF idet/2-idet/2. NE 0 THEN continue
                  
                  oplot,D[0:max_scan-1]+1,slope[collects,b,idet,isubset,igain,0:max_scan-1,1],symsize=1,color=lcol[i],psym=symbol[i],thick=3
                  oplot,D[0:max_scan-1]+1,slope[collects,b,idet,isubset,igain,0:max_scan-1,1],color=lcol[i],thick=3
                  i=i+1
                  
                  XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
                  IF band_type[b] EQ 1 THEN set_legend,2,name2,lcol,symbol
                  IF band_type[b] EQ 0 THEN set_legend,1,name1,lcol,symbol
               ENDFOR
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

device,/close

skipout:

END
