;*****************************************************************************
;  FP-15 and FP-16 RSR test. Reads and formats the wavelength from the gse files.
;
;  INPUTS:
;  1) infile - file with input, output paths, sender and uaid name, and fpa

;  OUTPUTS:
;  1) lambda[collects] -- wavelength
;  
;  v1.0 - 11/24/2008 - Jeff McIntire
;  v2.0 - 05/30/2014 - Jeff McIntire
;       - rewritten for F2
;*****************************************************************************

PRO fp15_16_read_gse, infile

;*****************************************************************************
;  Read in input file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
inpath2=strarr(1)		; Input path 2.
outpath=strarr(1)		; Output path.
uaidname=strarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,inpath2
readf,lun,outpath
readf,lun,uaidname
Free_Lun,lun

;*****************************************************************************
;  Get list of gse files.
;*****************************************************************************

; Get list of .txt files from specified path. 
spawn,'ls -C1 '+inpath2+'*gse',list_files

IF n_elements(list_files) EQ 0 THEN goto,skipout
nfile=n_elements(list_files)

;*****************************************************************************
;  Read in input file.
;*****************************************************************************

; Definitions.
spma_wavelength_a=dblarr(nfile)
spma_wavelength_b=dblarr(nfile)
lambda=dblarr(nfile)

FOR j=0, nfile-1 DO BEGIN
   nlines=File_Lines(list_files[j])
   IF nlines EQ 0 THEN continue
   
   awa=''
   name=strarr(nlines)
   wavelength=dblarr(nlines)    ; wavelength.
   
   openr,lun,list_files[j],/Get_Lun
   
   FOR ix=0L, nlines-1 DO BEGIN
      readf,lun,awa
      a1=strsplit(awa,String(9B),/Extract)
      name(ix)=a1(1)
      wavelength(ix)=DOUBLE(a1(5))
   ENDFOR
   Free_Lun,lun
   
;*****************************************************************************
;  Pick out the SPMA wavelengths.
;*****************************************************************************

   FOR ix=0L, nlines-1 DO BEGIN
      IF name[ix] EQ 'SPMA_MONO_A_WAVE' THEN BEGIN
         spma_wavelength_a[j]=wavelength[ix]
      ENDIF
      IF name[ix] EQ 'SPMA_MONO_B_WAVE' THEN BEGIN
         spma_wavelength_b[j]=wavelength[ix]
      ENDIF
   ENDFOR
    
;*****************************************************************************
;  Average over available data.
;*****************************************************************************

   lambda[j]=(spma_wavelength_a[j]+spma_wavelength_b[j])/2
ENDFOR

save,/compress,lambda,filename=outpath+uaidname+'.lambda.sav'

skipout:

END
