;*****************************************************************************
;  Format the SPMA RSO and then get the value at each wavelength for
;  FP 15 and FP 16. The value is taken as the interpolation between to the
;  two nearest points.
;  
;  INPUTS:
;  1) uaid_name -- specific UAID to be considered.
;  2) inpath -- path to input data.
;
;  OUTPUTS:
;  1) r_spma -- SpMA radiance.
;
;  v1.0 - 7/03/2009 - Jeff McIntire
;******************************************************************************

PRO rsr_spma_cfpa_2, inpath,uaid_name,grat,filt,r_spma

;*****************************************************************************
;  Read in the SpMA correction.
;*****************************************************************************

infile='rso/rsr_spma_cfpa_1.dat'

openr,lun,infile,/get_lun

n_lines=File_Lines(infile)
IF n_lines EQ 0 THEN goto,skipout

awa=''
header=strarr(1)			; header.
wavelength=fltarr(n_lines)		; wavelength.
grating=fltarr(n_lines)	        	; grating.
filter=fltarr(n_lines)	        	; filter.
nspma=fltarr(n_lines)			; spma.

FOR ix=0L, n_lines-1 DO BEGIN
   readf,lun,awa
   a1=strsplit(awa,String(9B),/Extract)
   wavelength(ix)=a1(0)*1000.
   grating(ix)=a1(1)
   filter(ix)=a1(2)
   nspma(ix)=a1(3)
ENDFOR
close,lun,/file,/all
free_lun,lun

;*****************************************************************************
;  Get the SpMA RSO at the wavelengths of FP 15 and FP 16.
;*****************************************************************************

restore,inpath+uaid_name+'.lambda.sav'

ncollects=n_elements(lambda)

r_spma=fltarr(ncollects)
FOR i=0, ncollects-1 DO BEGIN
   FOR j=0, n_lines-2 DO BEGIN
      IF grat[i] NE grating[j] THEN continue
      IF filt[i] NE filter[j] THEN continue
      
      x=WHERE(grat[i] EQ grating[*] AND filt[i] EQ filter[*])
      IF lambda[i] LT min(wavelength[x]) THEN r_spma[i]=nspma[x[0]]
      IF lambda[i] GT max(wavelength[x]) THEN r_spma[i]=nspma[x[n_elements(x)-1]]
      IF lambda[i] GT max(wavelength[x]) THEN continue
      
      IF (lambda[i]-wavelength[j]) GT 0 AND (lambda[i]-wavelength[j+1]) LT 0 THEN BEGIN
         result=poly_fit(wavelength[j:j+1],nspma[j:j+1],1)
         r_spma[i]=result[0]+result[1]*lambda[i]
      ENDIF
   ENDFOR
ENDFOR

skipout:

END
