;*****************************************************************************
;  Format the SPMA RSO and then get the value at each wavelength for
;  FP 15 and FP 16. The value is taken from a linear interpolation between to the
;  two nearest points.
;  
;  INPUTS:
;  1) uaid_name -- specific UAID to be considered.
;
;  OUTPUTS:
;  1) R_spma -- SpMA radiance.
;
;  v1.0 - 8/28/2008 - Jeff McIntire
;*****************************************************************************

PRO rsr_spma_dnb, inpath,uaid_name,grat,filt,j,type,R_spma

;*****************************************************************************
;  Read in the SpMA correction.
;*****************************************************************************

IF j EQ 0 THEN infile='rso/rsr_spma_VisNIR_f1_4.dat'
IF j EQ 1 THEN infile='rso/rsr_spma_VisNIR_f1_3.dat'

openr,lun,infile,/get_lun

n_lines=File_Lines(infile)
IF n_lines EQ 0 THEN goto,skipout

awa=''
header=strarr(1)			; header.
wavelength=fltarr(n_lines)		; wavelength.
grating=fltarr(n_lines)	        	; grating.
filter=fltarr(n_lines)	        	; filter.
nspma=fltarr(n_lines)			; spma.

FOR ix=0L, n_lines-1 DO BEGIN
   readf,lun,awa
   a1=strsplit(awa,String(9B),/Extract)
   wavelength(ix)=a1(2)*1000.
   grating(ix)=a1(0)
   filter(ix)=a1(1)
   nspma(ix)=a1(3)
ENDFOR
close,lun,/file,/all
free_lun,lun

;*****************************************************************************
;  Get the SpMA RSO at the wavelengths of FP 15 and FP 16.
;*****************************************************************************

restore,inpath+uaid_name+'.lambda.sav'

ncollects=n_elements(lambda)

r_spma=fltarr(ncollects)
FOR i=0, ncollects-1 DO BEGIN
   FOR jj=0, n_lines-2 DO BEGIN
      IF grat[i] NE grating[jj] THEN continue
      IF filt[i] NE filter[jj] THEN continue
      
      x=WHERE(grat[i] EQ grating[*] AND filt[i] EQ filter[*])
      IF lambda[i] LT min(wavelength[x]) THEN r_spma[i]=nspma[x[0]]
      IF lambda[i] GT max(wavelength[x]) THEN r_spma[i]=nspma[x[n_elements(x)-1]]
      IF lambda[i] GT max(wavelength[x]) THEN continue
      
      IF (lambda[i]-wavelength[jj]) GT 0 AND (lambda[i]-wavelength[jj+1]) LT 0 THEN BEGIN
         result=poly_fit(wavelength[jj:jj+1],nspma[jj:jj+1],1)
         r_spma[i]=result[0]+result[1]*lambda[i]
      ENDIF
   ENDFOR
ENDFOR

skipout:

END
