;*****************************************************************************
;  FP 15 and 16. Calculates the spectral metrics and stitches FP-15
;  and FP-16 data.
;
;  Output:
;  1) postscript file containing graphs.
;  
;  v1.0 - 10/27/2008 - Jeff McIntire
;  v6.0 - 05/30/2014 - Jeff McIntire
;       - rewritten for F2
;  v7.0 - 06/10/2014 - Jeff McIntire
;       - investigated ND filter transmission and FP-15 / FP-16 stitching.
;*****************************************************************************

;*****************************************************************************
;  Function for curvefit. (Atmosphere correction for M13)
;*****************************************************************************

PRO curfunc1, X,A, F, pder

F=(A[0]+A[1]*X)
;F=exp(A[0]+A[1]*X)

IF N_PARAMS() GE 4 THEN pder=[[replicate(1.0, N_ELEMENTS(X))],[replicate(A[1], N_ELEMENTS(X))]]
;IF N_PARAMS() GE 4 THEN pder=[[exp(A[0]+A[1]*X)],[X*exp(A[0]+A[1]*X)]]

END

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO fp15_16_stitching, infile

close,/all

;*****************************************************************************
;  Read in input file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
inpath2=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
uaidname=strarr(1)
sender=strarr(1)
fpa=strarr(1)
odd_even=strarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,inpath2
readf,lun,outpath
readf,lun,uaidname
readf,lun,sender
readf,lun,fpa
readf,lun,odd_even
Free_Lun,lun

;*****************************************************************************
;  Select focal plane dependent parameters and restore data from
;  previous routines.
;*****************************************************************************

IF fpa EQ 'DNB' THEN restore,'general.DNB.2.sav'
IF fpa EQ 'VisNIR' THEN restore,'general.VisNIR.2.sav'
IF fpa EQ 'SMWIR' THEN restore,'general.SMWIR.2.sav'
IF fpa EQ 'LWIR' THEN restore,'general.LWIR.2.sav'

oddeven=['odd','even']

;*****************************************************************************
;  Restore absorption
;*****************************************************************************

restore,'absorption.sav'

;*****************************************************************************
;  definitions.
;*****************************************************************************

; dn for OOB (0), IB (1), and FP-15 (2) sender band only.
dn_col=fltarr(nbands,300,maxdet,maxsubsets,3)

; RSO Corrected dn for OOB (0), IB (1), and FP-15 (2) sender band only.
dn_col_corr=fltarr(nbands,300,maxdet,maxsubsets,3)

; std and SNR for OOB (0), IB (1), and FP-15 (2) sender band only.
std_col=fltarr(nbands,300,maxdet,maxsubsets,3)
snr_col=fltarr(nbands,300,maxdet,maxsubsets,3)
snr_col_avg=fltarr(nbands,300,maxdet,maxsubsets,3)

; Number of collects for OOB (0), IB (1), and FP-15 (2).
ncol=fltarr(nbands,3)

; Wavelength for OOB (0), IB (1), and FP-15 (2).
lambda_uaid=fltarr(nbands,maxdet,300,3)

; SpMA response for OOB (0), IB (1), and FP-15 (2).
rso_spma=fltarr(nbands,300,3)

; Selector for overlap collects.
overlap=intarr(nbands,300)

;*****************************************************************************
;  Read in data.
;*****************************************************************************

IF fpa EQ 'DNB' THEN n_array=1
IF fpa EQ 'VisNIR' THEN n_array=1
IF fpa EQ 'SMWIR' THEN n_array=2
IF fpa EQ 'LWIR' THEN n_array=2

infname=strarr(2)
FOR m=0, n_array-1 DO BEGIN

    ; FP-15 IB overlap UAIDs.
   IF fpa EQ 'DNB' THEN uaidname1=['U3104215','U3104190','A','A']
   IF fpa EQ 'VisNIR' THEN uaidname1=['U3104150','U3103915','U3104244','U3104252','U3104229','U3104126','U3104279','U3104265','U3104114']
   IF fpa EQ 'SMWIR' AND m EQ 0 THEN uaidname1=['U3104075','U3103735','U3103940','U3104061','U3103989','U3104036','U3103680','U3103562']
   IF fpa EQ 'SMWIR' AND m EQ 1 THEN uaidname1=['U3104092','U3103926','U3103963','U3104048','U3104002','U3104015','U3103721','U3103579']
   IF fpa EQ 'LWIR' AND m EQ 0 THEN uaidname1=['U3103595','U3103693','U3103655','U3103748','U3103627']
   IF fpa EQ 'LWIR' AND m EQ 1 THEN uaidname1=['U3103606','U3103707','U3103667','U3103761','U3103641']
   
    ; FP-16 IB overlap UAIDs.
   IF fpa EQ 'DNB' THEN uaidname2=['A','U3104195','A','A']
   IF fpa EQ 'VisNIR' THEN uaidname2=['A','A','U3104246','U3104258','U3104235','U3104498','U3104294','U3104269','U3104120']
   IF fpa EQ 'SMWIR' AND m EQ 0 THEN uaidname2=['U3104079','U3103739','U3103980','U3104065','U3103993','U3104040','U3103684','U3103568']
   IF fpa EQ 'SMWIR' AND m EQ 1 THEN uaidname2=['U3104097','U3103930','U3103969','U3104052','U3104006','U3104028','U3103725','U3103585']
   IF fpa EQ 'LWIR' AND m EQ 0 THEN uaidname2=['A','U3103697','A','A','A']
   IF fpa EQ 'LWIR' AND m EQ 1 THEN uaidname2=['A','U3103711','A','A','A']

    ; FP-16 OOB UAIDs.
   IF fpa EQ 'DNB' THEN uaidname3=['U3104219','U3104197','A','A']
   IF fpa EQ 'VisNIR' THEN uaidname3=['U3104156','U3103920','U3104247','U3104259','U3104236','U3104499','U3104295','U3104270','U3104121']
   IF fpa EQ 'SMWIR' AND m EQ 0 THEN uaidname3=['U3104080','U3103740','U3103981','U3104066','U3103994','U3104041','U3103685','U3103569']
   IF fpa EQ 'SMWIR' AND m EQ 1 THEN uaidname3=['U3104098','U3103931','U3103970','U3104053','U3104007','U3104029','U3103726','U3103586']
   IF fpa EQ 'LWIR' AND m EQ 0 THEN uaidname3=['U3103598','U3103698','U3103658','U3103751','U3103631']
   IF fpa EQ 'LWIR' AND m EQ 1 THEN uaidname3=['U3103609','U3103712','U3103670','U3103764','U3103644']

    ; Read in Configuration file.
   IF fpa EQ 'DNB' THEN infile='config/DNB_config.dat'
   IF fpa EQ 'VisNIR' THEN infile='config/VisNIR_config.dat'
   IF fpa EQ 'SMWIR' THEN infile='config/SMWIR_config.dat'
   IF fpa EQ 'LWIR' THEN infile='config/LWIR_config.dat'

   openr,lun,infile,/get_lun
   n_lines=File_Lines(infile)

   awa=''
   header=strarr(1) 
   band=strarr(n_lines)
   test=strarr(n_lines)
   uaid=strarr(n_lines)
   collect=fltarr(n_lines)
   oe=fltarr(n_lines)
   g=fltarr(n_lines)
   f=fltarr(n_lines)

   readf,lun,header
   FOR ix=0L, n_lines-2 DO BEGIN
      readf,lun,awa
      a1=strsplit(awa,String(9B),/Extract)
      band(ix)=a1(0)
      test(ix)=a1(1)
      uaid(ix)=a1(2)
      collect(ix)=a1(3)
      oe(ix)=a1(4)
      g(ix)=a1(5)
      f(ix)=a1(6)
   ENDFOR
   free_lun,lun

   ; Read in FP-16 OOB data.
   FOR j=0, n_elements(uaidname3)-1 DO BEGIN
      grat=fltarr(300)
      filt=fltarr(300)
      IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
         FOR i=0L, n_lines-2 DO BEGIN
            IF band[i] EQ band_name[j] AND uaid[i] EQ uaidname3[j] THEN BEGIN
               FOR col=collect[i]-1, 299 DO grat[col]=g[i]
               FOR col=collect[i]-1, 299 DO filt[col]=f[i]
            ENDIF
         ENDFOR
      ENDIF
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
         FOR i=0L, n_lines-2 DO BEGIN
            IF band[i] EQ band_name[j] AND uaid[i] EQ uaidname3[j] AND oe[i] EQ m THEN BEGIN
               FOR col=collect[i]-1, 299 DO grat[col]=g[i]
               FOR col=collect[i]-1, 299 DO filt[col]=f[i]
            ENDIF
         ENDFOR
      ENDIF
      IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
         infname[0]=inpath2+'/'+band_name[j]+'/fp16_oob/'+uaidname3[j]+'.oob.sav'
         infname[1]=inpath2+'/'+band_name[j]+'/fp16_oob/'+uaidname3[j]+'.lambda.sav'
      ENDIF
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
         infname[0]=inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp16_oob/'+uaidname3[j]+'.oob.sav'
         infname[1]=inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp16_oob/'+uaidname3[j]+'.lambda.sav'
      ENDIF
      print,'Processing file:',uaidname3[j]
      IF uaidname3[j] EQ 'A' THEN continue
      restore,infname[0]
      restore,infname[1]
        
      IF fpa EQ 'DNB' THEN rsr_spma_DNB,inpath2+'/'+band_name[j]+'/fp16_oob/',uaidname3[j],grat,filt,j,0,L_spma
      IF fpa EQ 'VisNIR' THEN rsr_spma_VisNIR,inpath2+'/'+band_name[j]+'/fp16_oob/',uaidname3[j],grat,filt,j,0,L_spma
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN rsr_spma_cfpa_2,inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp16_oob/',uaidname3[j],grat,filt,L_spma
      
      ncol[j,0]=ncollects
      rso_spma[j,0:ncollects-1,0]=L_spma[0:ncollects-1]
      FOR id=0, detectors[j]-1 DO BEGIN
         IF fpa EQ 'SMWIR' AND id/2-id/2.+.5*m NE 0 THEN continue
         IF fpa EQ 'LWIR' AND id/2-id/2.+.5*m NE 0 THEN continue
         
         lambda_uaid[j,id,0:ncollects-1,0]=lambda[0:ncollects-1]
         FOR is=0, subsets[j]-1 DO BEGIN
            dn_col[j,0:ncollects-1,id,is,0]=dn[0:ncollects-1,j,id,is]
            dn_col_corr[j,0:ncollects-1,id,is,0]=dn_col[j,0:ncollects-1,id,is,0]/L_spma[0:ncollects-1]
            std_col[j,0:ncollects-1,id,is,0]=sqrt(std[0:ncollects-1,senderband,id,is,0]^2+std[0:ncollects-1,senderband,id,is,1]^2)
            snr_col[j,0:ncollects-1,id,is,0]=dn[0:ncollects-1,senderband,id,is]/std_col[j,0:ncollects-1,id,is,0]
         ENDFOR
      ENDFOR

      ; Find overlap collects for different sweeps, if any.
      IF m EQ 0 THEN iik=0 ELSE iik=1
      FOR iii=0, ncol[j,0]-1 DO BEGIN
         IF lambda_uaid[j,iik,iii,0]-lambda_uaid[j,iik,iii+1,0] GE 0 THEN BEGIN
            FOR iij=iii+1, ncol[j,0]-1 DO BEGIN
               IF lambda_uaid[j,iik,iii,0]-lambda_uaid[j,iik,iij,0] GE 0 THEN overlap[j,iij]=1
            ENDFOR
         ENDIF
      ENDFOR
      x=WHERE(overlap[j,0:ncol[j,0]-1] EQ 0)

      ncol[j,0]=n_elements(x)

      rso_spma[j,0:ncol[j,0]-1,0]=rso_spma[j,x,0]
      rso_spma[j,ncol[j,0]:299,0]=0
      FOR id=0, detectors[j]-1 DO BEGIN
         IF fpa EQ 'SMWIR' AND id/2-id/2.+.5*m NE 0 THEN continue
         IF fpa EQ 'LWIR' AND id/2-id/2.+.5*m NE 0 THEN continue
         
         lambda_uaid[j,id,0:ncol[j,0]-1,0]=lambda_uaid[j,id,x,0]
         
         dn_col[j,0:ncol[j,0]-1,id,0:subsets[j]-1,0]=dn_col[j,x,id,0:subsets[j]-1,0]
         dn_col_corr[j,0:ncol[j,0]-1,id,0:subsets[j]-1,0]=dn_col_corr[j,x,id,0:subsets[j]-1,0]
         std_col[j,0:ncol[j,0]-1,id,0:subsets[j]-1,0]=std_col[j,x,id,0:subsets[j]-1,0]
         snr_col[j,0:ncol[j,0]-1,id,0:subsets[j]-1,0]=snr_col[j,x,id,0:subsets[j]-1,0]
         
         lambda_uaid[j,id,ncol[j,0]:299,0]=0
         
         dn_col[j,ncol[j,0]:299,id,0:subsets[j]-1,0]=0
         dn_col_corr[j,ncol[j,0]:299,id,0:subsets[j]-1,0]=0
         std_col[j,ncol[j,0]:299,id,0:subsets[j]-1,0]=0
         snr_col[j,ncol[j,0]:299,id,0:subsets[j]-1,0]=0
      ENDFOR
   ENDFOR

   ; Read in FP-16 IB data.
   FOR j=0, n_elements(uaidname2)-1 DO BEGIN
      grat=fltarr(300)
      filt=fltarr(300)
      IF fpa  EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
         FOR i=0L, n_lines-2 DO BEGIN
            IF band[i] EQ band_name[j] AND uaid[i] EQ uaidname2[j] THEN BEGIN
               FOR col=collect[i]-1, 299 DO grat[col]=g[i]
               FOR col=collect[i]-1, 299 DO filt[col]=f[i]
            ENDIF
         ENDFOR
      ENDIF
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
         FOR i=0L, n_lines-2 DO BEGIN
            IF band[i] EQ band_name[j] AND uaid[i] EQ uaidname2[j] AND oe[i] EQ m THEN BEGIN
               FOR col=collect[i]-1, 299 DO grat[col]=g[i]
               FOR col=collect[i]-1, 299 DO filt[col]=f[i]
            ENDIF
         ENDFOR
      ENDIF
      IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
         infname[0]=inpath2+'/'+band_name[j]+'/fp16_ib/'+uaidname2[j]+'.oob.sav'
         infname[1]=inpath2+'/'+band_name[j]+'/fp16_ib/'+uaidname2[j]+'.lambda.sav'
      ENDIF
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
         infname[0]=inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp16_ib/'+uaidname2[j]+'.oob.sav'
         infname[1]=inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp16_ib/'+uaidname2[j]+'.lambda.sav'
      ENDIF
      print,'Processing file:',uaidname2[j]
      IF uaidname2[j] EQ 'A' THEN continue
      restore,infname[0]
      restore,infname[1]
      
      IF fpa EQ 'DNB' THEN rsr_spma_DNB,inpath2+'/'+band_name[j]+'/fp16_ib/',uaidname2[j],grat,filt,j,1,L_spma
      IF fpa EQ 'VisNIR' THEN rsr_spma_VisNIR,inpath2+'/'+band_name[j]+'/fp16_ib/',uaidname2[j],grat,filt,j,1,L_spma
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN rsr_spma_cfpa_2,inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp16_ib/',uaidname2[j],grat,filt,L_spma
      
      ncol[j,1]=ncollects
      rso_spma[j,0:ncollects-1,1]=L_spma[0:ncollects-1]
      FOR id=0, detectors[j]-1 DO BEGIN
         IF fpa EQ 'SMWIR' AND id/2-id/2.+.5*m NE 0 THEN continue
         IF fpa EQ 'LWIR' AND id/2-id/2.+.5*m NE 0 THEN continue
         
         lambda_uaid[j,id,0:ncollects-1,1]=lambda[0:ncollects-1]
         FOR is=0, subsets[j]-1 DO BEGIN
            dn_col[j,0:ncollects-1,id,is,1]=dn[0:ncollects-1,j,id,is]
            dn_col_corr[j,0:ncollects-1,id,is,1]=dn_col[j,0:ncollects-1,id,is,1]/L_spma[0:ncollects-1]
            std_col[j,0:ncollects-1,id,is,1]=sqrt(std[0:ncollects-1,senderband,id,is,0]^2+std[0:ncollects-1,senderband,id,is,1]^2)
            snr_col[j,0:ncollects-1,id,is,1]=dn[0:ncollects-1,senderband,id,is]/std_col[j,0:ncollects-1,id,is,1]
         ENDFOR
      ENDFOR
      overlap[*,*]=0

      ; Find overlap collects for different sweeps, if any.
      IF m EQ 0 THEN iik=0 ELSE iik=1
      FOR iii=0, ncol[j,1]-1 DO BEGIN
         IF lambda_uaid[j,iik,iii,1]-lambda_uaid[j,iik,iii+1,1] GT 0 THEN BEGIN
            FOR iij=iii+1, ncol[j,1]-1 DO BEGIN
               IF lambda_uaid[j,iik,iii,1]-lambda_uaid[j,iik,iij,1] GE 0 THEN overlap[j,iij]=1
            ENDFOR
         ENDIF
      ENDFOR
      x=WHERE(overlap[j,0:ncol[j,1]-1] EQ 0)
      ncol[j,1]=n_elements(x)
      
      rso_spma[j,0:ncol[j,1]-1,1]=rso_spma[j,x,1]
      rso_spma[j,ncol[j,1]:299,1]=0
      FOR id=0, detectors[j]-1 DO BEGIN
         
         lambda_uaid[j,id,0:ncol[j,1]-1,1]=lambda_uaid[j,id,x,1]
         
         dn_col[j,0:ncol[j,1]-1,id,0:subsets[j]-1,1]=dn_col[j,x,id,0:subsets[j]-1,1]
         dn_col_corr[j,0:ncol[j,1]-1,id,0:subsets[j]-1,1]=dn_col_corr[j,x,id,0:subsets[j]-1,1]
         std_col[j,0:ncol[j,1]-1,id,0:subsets[j]-1,1]=std_col[j,x,id,0:subsets[j]-1,1]
         snr_col[j,0:ncol[j,1]-1,id,0:subsets[j]-1,1]=snr_col[j,x,id,0:subsets[j]-1,1]
         
         lambda_uaid[j,id,ncol[j,1]:299,1]=0
         
         dn_col[j,ncol[j,1]:299,id,0:subsets[j]-1,1]=0
         dn_col_corr[j,ncol[j,1]:299,id,0:subsets[j]-1,1]=0
         std_col[j,ncol[j,1]:299,id,0:subsets[j]-1,1]=0
         snr_col[j,ncol[j,1]:299,id,0:subsets[j]-1,1]=0
      ENDFOR
   ENDFOR

   ; Read in FP-15 data.
   FOR j=0, n_elements(uaidname1)-1 DO BEGIN
      grat=fltarr(300)
      filt=fltarr(300)
      IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
         FOR i=0L, n_lines-2 DO BEGIN
            IF band[i] EQ band_name[j] AND uaid[i] EQ uaidname1[j] THEN BEGIN
               FOR col=collect[i]-1, 299 DO grat[col]=g[i]
               FOR col=collect[i]-1, 299 DO filt[col]=f[i]
            ENDIF
         ENDFOR
      ENDIF
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
         FOR i=0L, n_lines-2 DO BEGIN
            IF band[i] EQ band_name[j] AND uaid[i] EQ uaidname1[j] AND oe[i] EQ m THEN BEGIN
               FOR col=collect[i]-1, 299 DO grat[col]=g[i]
               FOR col=collect[i]-1, 299 DO filt[col]=f[i]
            ENDIF
         ENDFOR
      ENDIF
      IF fpa EQ 'DNB' OR fpa EQ 'VisNIR' THEN BEGIN
         infname[0]=inpath2+'/'+band_name[j]+'/fp15/'+uaidname1[j]+'.oob.sav'
         infname[1]=inpath2+'/'+band_name[j]+'/fp15/'+uaidname1[j]+'.lambda.sav'
      ENDIF
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN BEGIN
         infname[0]=inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp15/'+uaidname1[j]+'.oob.sav'
         infname[1]=inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp15/'+uaidname1[j]+'.lambda.sav'
      ENDIF
      print,'Processing file:',uaidname1[j]
      IF uaidname1[j] EQ 'A' THEN continue
      restore,infname[0]
      restore,infname[1]
        
      IF fpa EQ 'DNB' THEN rsr_spma_DNB,inpath2+'/'+band_name[j]+'/fp15/',uaidname1[j],grat,filt,j,2,L_spma
      IF fpa EQ 'VisNIR' THEN rsr_spma_VisNIR,inpath2+'/'+band_name[j]+'/fp15/',uaidname1[j],grat,filt,j,2,L_spma
      IF fpa EQ 'SMWIR' OR fpa EQ 'LWIR' THEN rsr_spma_cfpa_2,inpath2+'/'+band_name[j]+'/'+oddeven[m]+'/fp15/',uaidname1[j],grat,filt,L_spma
      
      ncol[j,2]=ncollects
      rso_spma[j,0:ncollects-1,2]=L_spma[0:ncollects-1]
      FOR id=0, detectors[j]-1 DO BEGIN
         IF fpa EQ 'SMWIR' AND id/2-id/2.+.5*m NE 0 THEN continue
         IF fpa EQ 'LWIR' AND id/2-id/2.+.5*m NE 0 THEN continue
         
         lambda_uaid[j,id,0:ncollects-1,2]=lambda[0:ncollects-1]
         FOR is=0, subsets[j]-1 DO BEGIN
            dn_col[j,0:ncollects-1,id,is,2]=dn[0:ncollects-1,j,id,is]
            dn_col_corr[j,0:ncollects-1,id,is,2]=dn_col[j,0:ncollects-1,id,is,2]/L_spma[0:ncollects-1]
            std_col[j,0:ncollects-1,id,is,2]=sqrt(std[0:ncollects-1,senderband,id,is,0]^2+std[0:ncollects-1,senderband,id,is,1]^2)
            snr_col[j,0:ncollects-1,id,is,2]=dn[0:ncollects-1,senderband,id,is]/std_col[j,0:ncollects-1,id,is,2]
            ; snr_col[j,0:ncollects-1,id,is,2]=dn[0:ncollects-1,senderband,id,is]/std[0:ncollects-1,senderband,id,is,0]
         ENDFOR
      ENDFOR

      IF fpa EQ 'DNB' THEN BEGIN
         overlap[*,*]=0

         ; Find overlap collects for different sweeps, if any.
         IF m EQ 0 THEN iik=0 ELSE iik=1
         FOR iii=ncol[j,2]-1, 0, -1 DO BEGIN
            IF lambda_uaid[j,iik,iii,2]-lambda_uaid[j,iik,iii-1,2] LT 0 THEN BEGIN
               FOR iij=iii-1, 0, -1 DO BEGIN
                  IF lambda_uaid[j,iik,iii,2]-lambda_uaid[j,iik,iij,2] LE 0 THEN overlap[j,iij]=1
               ENDFOR
            ENDIF
         ENDFOR
         x=WHERE(overlap[j,0:ncol[j,2]-1] EQ 0)
         ncol[j,2]=n_elements(x)
         
         rso_spma[j,0:ncol[j,2]-1,2]=rso_spma[j,x,2]
         rso_spma[j,ncol[j,2]:299,2]=0
         FOR id=0, detectors[j]-1 DO BEGIN

            lambda_uaid[j,id,0:ncol[j,2]-1,2]=lambda_uaid[j,id,x,2]
            
            dn_col[j,0:ncol[j,2]-1,id,0:subsets[j]-1,2]=dn_col[j,x,id,0:subsets[j]-1,2]
            dn_col_corr[j,0:ncol[j,2]-1,id,0:subsets[j]-1,2]=dn_col_corr[j,x,id,0:subsets[j]-1,2]
            std_col[j,0:ncol[j,2]-1,id,0:subsets[j]-1,2]=std_col[j,x,id,0:subsets[j]-1,2]
            snr_col[j,0:ncol[j,2]-1,id,0:subsets[j]-1,2]=snr_col[j,x,id,0:subsets[j]-1,2]
            
            lambda_uaid[j,id,ncol[j,2]:299,2]=0
            
            dn_col[j,ncol[j,2]:299,id,0:subsets[j]-1,2]=0
            dn_col_corr[j,ncol[j,2]:299,id,0:subsets[j]-1,2]=0
            std_col[j,ncol[j,2]:299,id,0:subsets[j]-1,2]=0
            snr_col[j,ncol[j,2]:299,id,0:subsets[j]-1,2]=0
         ENDFOR
      ENDIF
   ENDFOR
ENDFOR

;*****************************************************************************
;  Read in spectral smile of SpMA.
;*****************************************************************************

infile='smile/smile_f1.dat'
openr,lun,infile,/get_lun
n_lines=File_Lines(infile)

awa=''
band=strarr(n_lines)
odd__even=strarr(n_lines)
uaid=strarr(n_lines)
smile=fltarr(16,n_lines)

FOR ix=0L, n_lines-1 DO BEGIN
   readf,lun,awa
   a1=strsplit(awa,String(9B),/Extract)
   band(ix)=a1(0)
   odd__even(ix)=a1(1)
   uaid(ix)=a1(2)
   FOR iy=0, 15 DO smile(iy,ix)=a1(3+iy)
ENDFOR
free_lun,lun

;*****************************************************************************
;  Apply the smile correction. Correct only FP-15.
;*****************************************************************************

IF fpa EQ 'VisNIR' THEN BEGIN
   FOR j=0, n_elements(uaidname1)-1 DO BEGIN
      FOR k=0, 2 DO BEGIN
         IF k EQ 0 THEN continue
         IF k EQ 1 THEN continue
         
         FOR id=0, detectors[j]-1 DO BEGIN
            x=WHERE(band[*] EQ band_name[j])
            
            IF band_type[j] EQ 0 THEN BEGIN
               FOR l=0, ncol[j,2]-1 DO BEGIN
                  lambda_uaid[j,id,l,k]=lambda_uaid[j,id,l,k]+smile[id,x]
               ENDFOR
            ENDIF ELSE BEGIN
               FOR l=0, ncol[j,0]-1 DO BEGIN
                  lambda_uaid[j,id,l,k]=lambda_uaid[j,id,l,k]+smile[fix(id/2),x]
               ENDFOR
            ENDELSE
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF
IF fpa EQ 'SMWIR' THEN BEGIN
   FOR j=0, n_elements(uaidname1)-1 DO BEGIN
      FOR k=0, 2 DO BEGIN
         IF k EQ 0 THEN continue
         IF k EQ 1 THEN continue
         
         FOR id=0, detectors[j]-1 DO BEGIN
            x=WHERE(band[*] EQ band_name[j] AND odd__even[*] EQ 0)
            y=WHERE(band[*] EQ band_name[j] AND odd__even[*] EQ 1)
            
            IF j EQ 1 AND id/2-id/2.+0.5 NE 0 THEN continue
            IF j EQ 4 OR j EQ 5 THEN continue
            
            IF band_type[j] EQ 0 THEN BEGIN
               FOR l=0, ncol[j,2]-1 DO BEGIN
                  lambda_uaid[j,id,l,k]=lambda_uaid[j,id,l,k]+smile[id,x]+smile[id,y]
               ENDFOR
            ENDIF ELSE BEGIN
               FOR l=0, ncol[j,0]-1 DO BEGIN
                  lambda_uaid[j,id,l,k]=lambda_uaid[j,id,l,k]+smile[fix(id/2),x]+smile[fix(id/2),y]
               ENDFOR
            ENDELSE
         ENDFOR
      ENDFOR
   ENDFOR
ENDIF

;*****************************************************************************
;  Average SNR for all FP-16 wavelengths.
;*****************************************************************************

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k EQ 1 AND uaidname2[j] EQ 'A' THEN continue
      IF k EQ 2 THEN continue
      
      FOR is=0, subsets[j]-1 DO BEGIN
         FOR l=0, ncol[j,k]-1 DO BEGIN
            IF fpa EQ 'SMWIR' AND j EQ 2 THEN BEGIN
               x=[0,1,2,3,4,5,6,7,8,9,11,12,13,14,15]
               snr_col_avg[j,l,0,is,k]=sig_mean(snr_col[j,l,x,is,k],-1,1)
            ENDIF ELSE BEGIN
               snr_col_avg[j,l,0,is,k]=sig_mean(snr_col[j,l,0:detectors[j]-1,is,k],-1,1)
            ENDELSE
            
            snr_col_avg[j,l,0:detectors[j]-1,is,k]=snr_col_avg[j,l,0,is,k]
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Make RSO correction for signal with SNR GT 3
;*****************************************************************************

;IF fpa EQ 'SMWIR' THEN snr_cutoff=[0.1,0.25,0.4,0.2,0.2,0.2,0.2,0.2]
;IF fpa EQ 'LWIR' THEN snr_cutoff=[1.1,1.8,0.6,1.0,0.6]
IF fpa EQ 'DNB' THEN snr_cutoff=[0.05,0.0,0.0,0.0]
IF fpa EQ 'VisNIR' THEN snr_cutoff=[0.05,0.04,0.02,0.02,0.03,0.04,0.02,0.015,0.022]
IF fpa EQ 'SMWIR' THEN snr_cutoff=[0.05,0.05,0.04,0.03,0.07,0.03,0.03,0.025]
IF fpa EQ 'LWIR' THEN snr_cutoff=[0.20,0.3,0.17,0.17,0.15]

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k EQ 1 AND uaidname2[j] EQ 'A' THEN continue
      IF k EQ 2 THEN continue
      
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            x=WHERE(dn_col_corr[j,0:ncol[j,k]-1,id,is,k] EQ max(dn_col_corr[j,0:ncol[j,k]-1,id,is,k]))
            
            FOR l=0, ncol[j,k]-1 DO BEGIN
               IF fpa EQ 'DNB' THEN BEGIN
                  IF j EQ 1 THEN BEGIN
                     dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,l,k]
                  ENDIF
                  IF j EQ 0 THEN BEGIN
                     IF snr_col_avg[j,l,id,is,k] GE snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,l,k] 
                     IF snr_col_avg[j,l,id,is,k] LT snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,x[0],k]
                  ENDIF
               ENDIF
               IF fpa EQ 'VisNIR' THEN BEGIN
                  ;IF snr_col[j,l,id,is,k] GE 0.1 THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,l,k] 
                  ;IF snr_col[j,l,id,is,k] LT 0.1 THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,x[0],k]
                  dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,l,k]
               ENDIF
               IF fpa EQ 'SMWIR' THEN BEGIN
                  IF snr_col_avg[j,l,id,is,k] GE snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,l,k] 
                  IF snr_col_avg[j,l,id,is,k] LT snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,x[0],k]
                  ;IF snr_col[j,l,id,is,k] GE snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,l,k] 
                  ;IF snr_col[j,l,id,is,k] LT snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,x[0],k]
                  ;IF snr_col_avg[j,l,id,is,k] LT snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=0.
               ENDIF
               IF fpa EQ 'LWIR' THEN BEGIN
                  IF snr_col_avg[j,l,id,is,k] GE snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,l,k] 
                  IF snr_col_avg[j,l,id,is,k] LT snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,x[0],k]
                  ;IF snr_col[j,l,id,is,k] GE snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,l,k] 
                  ;IF snr_col[j,l,id,is,k] LT snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=dn_col[j,l,id,is,k]/rso_spma[j,x[0],k]
                  ;IF snr_col_avg[j,l,id,is,k] LT snr_cutoff[j] THEN dn_col_corr[j,l,id,is,k]=0.
               ENDIF
            ENDFOR
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  SpMA response at center wavelength.
;*****************************************************************************

result1=fltarr(2)
cent_col=intarr(n_elements(uaidname1),maxdet,3)
rso_spma_center=fltarr(n_elements(uaidname1),maxdet,3)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k EQ 0 AND uaidname3[j] EQ 'A' THEN continue
      IF k EQ 1 AND uaidname2[j] EQ 'A' THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue
      
      FOR id=0, detectors[j]-1 DO BEGIN
         cent_col[j,id,k]=WHERE(abs(lambda_uaid[j,id,0:ncol[j,k]-1,k]-lambda_center[j]*1000) EQ min(abs(lambda_uaid[j,id,0:ncol[j,k]-1,k]-lambda_center[j]*1000)))
         result1[0:1]=poly_fit(lambda_uaid[j,id,cent_col[j,id,k]:cent_col[j,id,k]+1,k],rso_spma[j,cent_col[j,id,k]:cent_col[j,id,k]+1,k],1)
         rso_spma_center[j,id,k]=result1[0]+result1[1]*lambda_center[j]*1000
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate RSR.
;*****************************************************************************

RSR_col=fltarr(n_elements(uaidname1),250,maxdet,maxsubsets,3)
RSR_col_corr=fltarr(n_elements(uaidname1),250,maxdet,maxsubsets,3)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k EQ 0 AND uaidname3[j] EQ 'A' THEN continue
      IF k EQ 1 AND uaidname2[j] EQ 'A' THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue
      
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            RSR_col[j,0:ncol[j,k]-1,id,is,k]=dn_col[j,0:ncol[j,k]-1,id,is,k]/max(dn_col[j,0:ncol[j,k]-1,id,is,k])
            RSR_col_corr[j,0:ncol[j,k]-1,id,is,k]=dn_col_corr[j,0:ncol[j,k]-1,id,is,k]/max(dn_col_corr[j,0:ncol[j,k]-1,id,is,k])
         ENDFOR
      ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate full width half max, bandwith, and center wavelength..
;*****************************************************************************

result=fltarr(2,4)
result2=fltarr(3,4)
lambda_hm=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3,2)
lambda_hm_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3,2)
bandwidth=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)
bandwidth_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)
center_lambda=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)
center_lambda_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)

hm=0.5

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k NE 2 THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue
      
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            zx=WHERE(RSR_col[j,*,id,is,k] EQ 1.0)
            zy=WHERE(RSR_col_corr[j,*,id,is,k] EQ 1.0)
            
            leftcol=0
            rightcol=0
            leftcol_corr=0
            rightcol_corr=0
            FOR l=1, ncol[j,2]-2 DO BEGIN
               IF hm GT RSR_col[j,l-1,id,is,k] AND hm LT RSR_col[j,l,id,is,k] AND l LE zx[0] THEN leftcol=l
               IF hm LT RSR_col[j,l,id,is,k] AND hm GT RSR_col[j,l+1,id,is,k] AND l GE zx[0] THEN rightcol=l+1
               
               IF hm GT RSR_col_corr[j,l-1,id,is,k] AND hm LT RSR_col_corr[j,l,id,is,k] AND l LE zy[0] THEN leftcol_corr=l
               IF hm LT RSR_col_corr[j,l,id,is,k] AND hm GT RSR_col_corr[j,l+1,id,is,k] AND l GE zy[0] THEN rightcol_corr=l+1
            ENDFOR
            
            result[0:1,0]=poly_fit(RSR_col[j,leftcol-1:leftcol,id,is,k],lambda_uaid[j,id,leftcol-1:leftcol,k],1)
            result[0:1,1]=poly_fit(RSR_col[j,rightcol-1:rightcol,id,is,k],lambda_uaid[j,id,rightcol-1:rightcol,k],1)
            result[0:1,2]=poly_fit(RSR_col_corr[j,leftcol_corr-1:leftcol_corr,id,is,k],lambda_uaid[j,id,leftcol_corr-1:leftcol_corr,k],1)
            result[0:1,3]=poly_fit(RSR_col_corr[j,rightcol_corr-1:rightcol_corr,id,is,k],lambda_uaid[j,id,rightcol_corr-1:rightcol_corr,k],1)

            lambda_hm[j,id,is,k,0]=result[0,0]+result[1,0]*0.5
            lambda_hm[j,id,is,k,1]=result[0,1]+result[1,1]*0.5
            lambda_hm_corr[j,id,is,k,0]=result[0,2]+result[1,2]*0.5
            lambda_hm_corr[j,id,is,k,1]=result[0,3]+result[1,3]*0.5
            
            bandwidth[j,id,is,k]=lambda_hm[j,id,is,k,1]-lambda_hm[j,id,is,k,0]
            bandwidth_corr[j,id,is,k]=lambda_hm_corr[j,id,is,k,1]-lambda_hm_corr[j,id,is,k,0]
            
            center_lambda[j,id,is,k]=lambda_hm[j,id,is,k,1]-bandwidth[j,id,is,k]/2
            center_lambda_corr[j,id,is,k]=lambda_hm_corr[j,id,is,k,1]-bandwidth_corr[j,id,is,k]/2
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the characterization uncertainty. Defined as the
;  deviation for detector averaged center wavelength or bandwidth.
;*****************************************************************************

char_uncert_center=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)
char_uncert_center_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)
char_uncert_bandwidth=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)
char_uncert_bandwidth_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k NE 2 THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue

      temp=mean(center_lambda[j,0:detectors[j]-1,0:subsets[j]-1,k])
      temp_corr=mean(center_lambda_corr[j,0:detectors[j]-1,0:subsets[j]-1,k])
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            char_uncert_center[j,id,is,k]=abs(center_lambda[j,id,is,k]-temp)
            char_uncert_center_corr[j,id,is,k]=abs(center_lambda_corr[j,id,is,k]-temp_corr)
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k NE 2 THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue
      
      temp=mean(bandwidth[j,0:detectors[j]-1,0:subsets[j]-1,k])
      temp_corr=mean(bandwidth_corr[j,0:detectors[j]-1,0:subsets[j]-1,k])
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            char_uncert_bandwidth[j,id,is,k]=abs(bandwidth[j,id,is,k]-temp)
            char_uncert_bandwidth_corr[j,id,is,k]=abs(bandwidth_corr[j,id,is,k]-temp_corr)
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the centroid wavelength..
;*****************************************************************************

lambda_centroid=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)
lambda_centroid_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets,3)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k NE 2 THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue
      
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            lambda_centroid[j,id,is,k]=total(lambda_uaid[j,id,0:ncol[j,k]-1,k]*RSR_col[j,0:ncol[j,k]-1,id,is,k])/total(RSR_col[j,0:ncol[j,k]-1,id,is,k])
            lambda_centroid_corr[j,id,is,k]=total(lambda_uaid[j,id,0:ncol[j,k]-1,k]*RSR_col_corr[j,0:ncol[j,k]-1,id,is,k])/total(RSR_col_corr[j,0:ncol[j,k]-1,id,is,k])
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the 1% limits.
;*****************************************************************************

result=fltarr(2,4)
result2=fltarr(3,4)
lambda_limit=fltarr(n_elements(uaidname1),maxdet,maxsubsets,2,3)
lambda_limit_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets,2,3)

limit=0.01

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k NE 2 THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue
      
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            zx=WHERE(RSR_col[j,*,id,is,k] EQ 1.0)
            zy=WHERE(RSR_col_corr[j,*,id,is,k] EQ 1.0)
            
            leftcol=0
            rightcol=0
            leftcol_corr=0
            rightcol_corr=0
            FOR l=zx[0], 1, -1 DO BEGIN
               IF limit GT RSR_col[j,l-1,id,is,k] AND limit LT RSR_col[j,l,id,is,k] THEN leftcol=l
               IF leftcol NE 0 THEN continue
            ENDFOR
            FOR l=zy[0], 1, -1 DO BEGIN
               IF leftcol_corr NE 0 THEN continue
               IF limit GT RSR_col_corr[j,l-1,id,is,k] AND limit LT RSR_col_corr[j,l,id,is,k] THEN leftcol_corr=l
            ENDFOR
            FOR l=zx[0], ncol[j,k]-2 DO BEGIN
               IF limit LT RSR_col[j,l,id,is,k] AND limit GT RSR_col[j,l+1,id,is,k] THEN rightcol=l+1
               IF rightcol NE 0 THEN continue
            ENDFOR
            FOR l=zy[0], ncol[j,k]-2 DO BEGIN
               IF rightcol_corr NE 0 THEN continue
               IF limit LT RSR_col_corr[j,l,id,is,k] AND limit GT RSR_col_corr[j,l+1,id,is,k] THEN rightcol_corr=l+1
            ENDFOR
            IF leftcol EQ 0 THEN continue
            
            result[0:1,0]=poly_fit(RSR_col[j,leftcol-1:leftcol,id,is,k],lambda_uaid[j,id,leftcol-1:leftcol,k],1)
            result[0:1,1]=poly_fit(RSR_col[j,rightcol-1:rightcol,id,is,k],lambda_uaid[j,id,rightcol-1:rightcol,k],1)
            result[0:1,2]=poly_fit(RSR_col_corr[j,leftcol_corr-1:leftcol_corr,id,is,k],lambda_uaid[j,id,leftcol_corr-1:leftcol_corr,k],1)
            result[0:1,3]=poly_fit(RSR_col_corr[j,rightcol_corr-1:rightcol_corr,id,is,k],lambda_uaid[j,id,rightcol_corr-1:rightcol_corr,k],1)
            
            lambda_limit[j,id,is,0,k]=result[0,0]+result[1,0]*limit
            lambda_limit[j,id,is,1,k]=result[0,1]+result[1,1]*limit
            lambda_limit_corr[j,id,is,0,k]=result[0,2]+result[1,2]*limit
            lambda_limit_corr[j,id,is,1,k]=result[0,3]+result[1,3]*limit
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Transimission of ND filter
;*****************************************************************************

IF fpa EQ 'DNB' THEN BEGIN
    lower_bound=[0.01,0.01,0.01,0.01]
    upper_bound=[0.6,0.6,0.6,0.6]
ENDIF
IF fpa EQ 'VisNIR' THEN BEGIN
   lower_bound=[0,0,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01]
   upper_bound=[0,0,0.6,0.6,0.6,0.6,0.5,0.6,0.6]
ENDIF
IF fpa EQ 'SMWIR' THEN BEGIN
   lower_bound=[0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01]
   upper_bound=[0.6,0.2,0.6,0.6,0.6,0.6,0.6,0.6]
ENDIF
IF fpa EQ 'LWIR' THEN BEGIN
   lower_bound=[0,0.01,0,0,0]
   upper_bound=[0,0.6,0,0,0]
ENDIF

trans_col_corr=fltarr(n_elements(uaidname1),300,maxdet,maxsubsets)
trans_avg_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
trans_std_corr=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
std_trans_corr=fltarr(n_elements(uaidname1))
trans_corr=fltarr(n_elements(uaidname1))

lambda_trans=fltarr(n_elements(uaidname1),20,maxdet,maxsubsets)
trans_wave=fltarr(n_elements(uaidname1),20,maxdet,maxsubsets)

outfile=outpath+'RSR.trans.txt'
outfile2=outpath+'RSR.trans.wave.txt'
fmt='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, i3)'
fmt2='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, i3, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile,/get_lun
printf,lun,'Transimission coeff'
printf,lun,'Band    Detector   Subsample    Transimission of ND   STDDEV of Transimission of ND     # of data points'
openw,lon,outfile2,/get_lun
printf,lon,'Transimission coeff'
printf,lon,'Band    Detector   Subsample    Transimission of ND   STDDEV of Transimission of ND     # of data points    Wavelengths used for ND transmission deteremination'

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname2[j] EQ 'A' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         yx=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,0] LT lambda_limit_corr[j,id,is,0,2])
         yy=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,0] GT lambda_limit_corr[j,id,is,1,2])
         
         count=0
         leftcol_corr_1=0
         leftcol_corr_2=0
         FOR l=max(yx)-5, ncol[j,0]-1 DO BEGIN
            IF RSR_col_corr[j,l,id,is,0] GT lower_bound[j] THEN leftcol_corr_1=l
            IF leftcol_corr_1 NE 0 THEN break
         ENDFOR
         FOR l=max(yx)-5, ncol[j,0]-1 DO BEGIN
            IF RSR_col_corr[j,l,id,is,0] GT upper_bound[j] THEN leftcol_corr_2=l-1
            
            IF leftcol_corr_2 NE 0 THEN break
         ENDFOR
         FOR l=cent_col[j,0], 1, -1 DO BEGIN
            IF l GT leftcol_corr_2 OR l LT leftcol_corr_1 THEN continue
            
            x=WHERE(lambda_uaid[j,id,l,0] EQ lambda_uaid[j,id,*,1])
            IF x[0] EQ -1 Then continue
            count=count+1
            
            trans_col_corr[j,l,id,is]=dn_col_corr[j,x,id,is,1]/dn_col_corr[j,l,id,is,0]

            lambda_trans[j,count-1,id,is]=lambda_uaid[j,id,l,0]
            trans_wave[j,count-1,id,is]=trans_col_corr[j,l,id,is]
         ENDFOR
         
         rightcol_corr_1=0
         rightcol_corr_2=0
         FOR l=max(yy)-5, 1, -1 DO BEGIN
            IF RSR_col_corr[j,l,id,is,0] GT lower_bound[j] THEN rightcol_corr_1=l
            IF band_name[j] EQ 'M1' AND l GT 50 THEN rightcol_corr_1=0
            
            IF rightcol_corr_1 NE 0 THEN break
         ENDFOR
         FOR l=max(yy)-5, 1, -1 DO BEGIN
            IF RSR_col_corr[j,l,id,is,0] GT upper_bound[j] THEN rightcol_corr_2=l+1
            
            IF rightcol_corr_2 NE 0 THEN break
         ENDFOR
         FOR l=cent_col[j,0], ncol[j,0]-1 DO BEGIN
            IF l LT rightcol_corr_2 OR l GT rightcol_corr_1 THEN continue
            
            x=WHERE(lambda_uaid[j,id,l,0] EQ lambda_uaid[j,id,*,1])
            IF x[0] EQ -1 Then continue
            count=count+1
            
            trans_col_corr[j,l,id,is]=dn_col_corr[j,x,id,is,1]/dn_col_corr[j,l,id,is,0]

            lambda_trans[j,count-1,id,is]=lambda_uaid[j,id,l,0]
            trans_wave[j,count-1,id,is]=trans_col_corr[j,l,id,is]
         ENDFOR
         trans_avg_corr[j,id,is]=sig_mean(trans_col_corr[j,*,id,is],3,1)
         trans_std_corr[j,id,is]=sig_mean(trans_col_corr[j,*,id,is],3,2)
         
         printf,lun,format=fmt,band_name[j],id+1,is,trans_avg_corr[j,id,is],trans_std_corr[j,id,is],count
         printf,lon,format=fmt2,band_name[j],id+1,is,trans_avg_corr[j,id,is],trans_std_corr[j,id,is],count,lambda_trans[j,0:count-1,id,is]
      ENDFOR
   ENDFOR
   trans_corr[j]=sig_mean(trans_avg_corr[j,*,*],3,1)
   std_trans_corr[j]=sig_mean(trans_std_corr[j,*,*],3,1)
ENDFOR
free_lun,lun
free_lun,lon

;*****************************************************************************
;  Investigate transimission of ND filter calculation.
;*****************************************************************************

trans_col_corr_2=fltarr(n_elements(uaidname1),300,maxdet,maxsubsets)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname2[j] EQ 'A' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         FOR l=0, ncol[j,1]-1 DO BEGIN
            IF lambda_uaid[j,id,l,1] EQ 0 THEN continue

            x=where(lambda_uaid[j,id,l,1] EQ lambda_uaid[j,id,*,0])

            trans_col_corr_2[j,l,id,is]=dn_col_corr[j,l,id,is,1]/dn_col_corr[j,x[0],id,is,0]
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR
free_lun,lun

;*****************************************************************************
;  Stich FP-16 IB onto FP-16 OOB
;*****************************************************************************

dn_col_stitch_ND=fltarr(n_elements(uaidname1),300,maxdet,maxsubsets)
RSR_col_stitch_ND=fltarr(n_elements(uaidname1),300,maxdet,maxsubsets)
snr_col_stitch_ND=fltarr(n_elements(uaidname1),300,maxdet,maxsubsets)
snr_col_stitch_ND_avg=fltarr(n_elements(uaidname1),300,maxdet,maxsubsets)

dn_col_stitch_ND_2=fltarr(n_elements(uaidname1),300,maxdet,maxsubsets)
RSR_col_stitch_ND_2=fltarr(n_elements(uaidname1),300,maxdet,maxsubsets)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname2[j] EQ 'A' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         x=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,0] EQ min(lambda_uaid[j,id,0:ncol[j,1]-1,1]))
         y=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,0] EQ max(lambda_uaid[j,id,0:ncol[j,1]-1,1]))
         
         dn_col_stitch_ND[j,0:x[0]-1,id,is]=dn_col_corr[j,0:x[0]-1,id,is,0]
         dn_col_stitch_ND[j,x[0]:y[0],id,is]=dn_col_corr[j,0:ncol[j,1]-1,id,is,1]/trans_avg_corr[j,id,is]
         dn_col_stitch_ND[j,y[0]+1:ncol[j,0]-1,id,is]=dn_col_corr[j,y[0]+1:ncol[j,0]-1,id,is,0]

         dn_col_stitch_ND_2[j,0:x[0]-1,id,is]=dn_col_corr[j,0:x[0]-1,id,is,0]
         dn_col_stitch_ND_2[j,x[0]:y[0],id,is]=dn_col_corr[j,0:ncol[j,1]-1,id,is,1]/trans_corr[j]
         dn_col_stitch_ND_2[j,y[0]+1:ncol[j,0]-1,id,is]=dn_col_corr[j,y[0]+1:ncol[j,0]-1,id,is,0]
         
         snr_col_stitch_ND[j,0:x[0]-1,id,is]=snr_col[j,0:x[0]-1,id,is,0]
         snr_col_stitch_ND[j,x[0]:y[0],id,is]=snr_col[j,0:ncol[j,1]-1,id,is,1]
         snr_col_stitch_ND[j,y[0]+1:ncol[j,0]-1,id,is]=snr_col[j,y[0]+1:ncol[j,0]-1,id,is,0]
         
         snr_col_stitch_ND_avg[j,0:x[0]-1,id,is]=snr_col_avg[j,0:x[0]-1,id,is,0]
         snr_col_stitch_ND_avg[j,x[0]:y[0],id,is]=snr_col_avg[j,0:ncol[j,1]-1,id,is,1]
         snr_col_stitch_ND_avg[j,y[0]+1:ncol[j,0]-1,id,is]=snr_col_avg[j,y[0]+1:ncol[j,0]-1,id,is,0]
      ENDFOR
   ENDFOR
ENDFOR

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname2[j] EQ 'A' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         RSR_col_stitch_ND[j,0:ncol[j,0]-1,id,is]=dn_col_stitch_ND[j,0:ncol[j,0]-1,id,is]/max(dn_col_stitch_ND[j,0:ncol[j,0]-1,id,is])

         RSR_col_stitch_ND_2[j,0:ncol[j,0]-1,id,is]=dn_col_stitch_ND_2[j,0:ncol[j,0]-1,id,is]/max(dn_col_stitch_ND_2[j,0:ncol[j,0]-1,id,is])
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the 0.1% limits
;*****************************************************************************

result=fltarr(2,4)
lambda_limit_stitch_2=fltarr(n_elements(uaidname1),maxdet,maxsubsets,2,3)

limit=0.001
;limit=0.0005

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k NE 2 THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue
      
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            zy=WHERE(RSR_col_corr[j,*,id,is,k] EQ 1.0)
            
            leftcol_corr=0
            rightcol_corr=0
            FOR l=zy[0], 1, -1 DO BEGIN
               IF limit GT RSR_col_corr[j,l-1,id,is,k] AND limit LT RSR_col_corr[j,l,id,is,k] THEN leftcol_corr=l
               IF leftcol_corr NE 0 THEN continue
            ENDFOR
            FOR l=zy[0], ncol[j,k]-2 DO BEGIN
               IF limit LT RSR_col_corr[j,l,id,is,k] AND limit GT RSR_col_corr[j,l+1,id,is,k] THEN rightcol_corr=l+1
               IF rightcol_corr NE 0 THEN continue
            ENDFOR
            IF leftcol_corr NE 0 THEN BEGIN
               result[0:1,2]=poly_fit(RSR_col_corr[j,leftcol_corr-1:leftcol_corr,id,is,k],lambda_uaid[j,id,leftcol_corr-1:leftcol_corr,k],1)
               
               lambda_limit_stitch_2[j,id,is,0,k]=result[0,2]+result[1,2]*limit
            ENDIF ELSE BEGIN
               lambda_limit_stitch_2[j,id,is,0,k]=min(lambda_uaid[j,id,0:ncol[j,2]-1,2])
            ENDELSE
            IF rightcol_corr NE 0 THEN BEGIN
               result[0:1,3]=poly_fit(RSR_col_corr[j,rightcol_corr-1:rightcol_corr,id,is,k],lambda_uaid[j,id,rightcol_corr-1:rightcol_corr,k],1)
               
               lambda_limit_stitch_2[j,id,is,1,k]=result[0,3]+result[1,3]*limit
            ENDIF ELSE BEGIN
               lambda_limit_stitch_2[j,id,is,1,k]=max(lambda_uaid[j,id,0:ncol[j,2]-1,2])
            ENDELSE
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate where FP-15 reaches an SNR of 3.
;*****************************************************************************

result=fltarr(2,4)
lambda_limit_stitch=fltarr(n_elements(uaidname1),maxdet,maxsubsets,2,3)

limit=3.0
;limit=1.0

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR k=0, 2 DO BEGIN
      IF k NE 2 THEN continue
      IF k EQ 2 AND uaidname1[j] EQ 'A' THEN continue
      
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            zy=WHERE(RSR_col_corr[j,*,id,is,k] EQ 1.0)
            
            leftcol_corr=0
            rightcol_corr=0
            FOR l=zy[0], 1, -1 DO BEGIN
               IF limit GT snr_col[j,l-1,id,is,k] AND limit LT snr_col[j,l,id,is,k] THEN leftcol_corr=l
               IF leftcol_corr NE 0 THEN continue
            ENDFOR
            FOR l=zy[0], ncol[j,k]-2 DO BEGIN
               IF limit LT snr_col[j,l,id,is,k] AND limit GT snr_col[j,l+1,id,is,k] THEN rightcol_corr=l+1
               IF rightcol_corr NE 0 THEN continue
            ENDFOR
            IF leftcol_corr NE 0 THEN BEGIN
               result[0:1,2]=poly_fit(snr_col[j,leftcol_corr-1:leftcol_corr,id,is,k],lambda_uaid[j,id,leftcol_corr-1:leftcol_corr,k],1)
               
               lambda_limit_stitch[j,id,is,0,k]=result[0,2]+result[1,2]*limit
            ENDIF ELSE BEGIN
               lambda_limit_stitch[j,id,is,0,k]=min(lambda_uaid[j,id,0:ncol[j,2]-1,2])
            ENDELSE
            IF rightcol_corr NE 0 THEN BEGIN
               result[0:1,3]=poly_fit(snr_col[j,rightcol_corr-1:rightcol_corr,id,is,k],lambda_uaid[j,id,rightcol_corr-1:rightcol_corr,k],1)
               
               lambda_limit_stitch[j,id,is,1,k]=result[0,3]+result[1,3]*limit
            ENDIF ELSE BEGIN
               lambda_limit_stitch[j,id,is,1,k]=max(lambda_uaid[j,id,0:ncol[j,2]-1,2])
            ENDELSE
            IF band_name[j] EQ 'M13' THEN lambda_limit_stitch[j,id,is,1,k]=max(lambda_uaid[j,id,0:ncol[j,2]-1,2])
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Read in water vapor corrected M9 and define custom stitch points
;*****************************************************************************
goto,skip_m9
FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF band_name[j] NE 'M9' THEN continue
   IF uaidname1[j] EQ 'A' THEN continue
   
   header=strarr(1)
   FOR id=0, detectors[j]-1 DO BEGIN
      infile='M9_Moeller/VIIRS_FU1_RSR_M9_Det'+strtrim(string(id+1),2)+'_3.6mpath_specCor.data'
      openr,lun,infile,/get_lun
      n_lines=File_Lines(infile)
      
      awa=''
      readf,lun,header
      FOR ix=0L, n_lines-2 DO BEGIN
         readf,lun,awa
         a1=strsplit(awa,' ',/Extract)
         
         lambda_uaid[j,id,ix,2]=a1(3)
         RSR_col_corr[j,ix,id,0,2]=a1(4)
      ENDFOR
      free_lun,lun
      
      lambda_limit_stitch[j,id,0,0,2]=1349.
      lambda_limit_stitch[j,id,0,1,2]=1397.
   ENDFOR
ENDFOR
skip_m9:
;*****************************************************************************
;  Stich FP-15 IB onto stiched FP-16
;*****************************************************************************

ncol_stitch=intarr(n_elements(uaidname1),maxdet,maxsubsets)
lambda_stitch=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)
RSR_col_stitch=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)
snr_col_stitch=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)
snr_col_stitch_avg=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)

RSR_col_stitch_2=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname3[j] EQ 'A' THEN continue
   
   IF uaidname2[j] NE 'A' THEN BEGIN
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            x=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,0] LT lambda_limit_stitch[j,id,is,0,2])
            y=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,0] GT lambda_limit_stitch[j,id,is,1,2])
            
            z=WHERE(lambda_uaid[j,id,0:ncol[j,2]-1,2] GT lambda_limit_stitch[j,id,is,0,2] $
                    AND lambda_uaid[j,id,0:ncol[j,2]-1,2] LT lambda_limit_stitch[j,id,is,1,2])
            
            ncol_stitch[j,id,is]=n_elements(x)+n_elements(y)+n_elements(z)
            
            lambda_stitch[j,0:n_elements(x)-1,id,is]=lambda_uaid[j,id,x,0]
            lambda_stitch[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=lambda_uaid[j,id,z,2]
            lambda_stitch[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=lambda_uaid[j,id,y,0]
            
            RSR_col_stitch[j,0:n_elements(x)-1,id,is]=RSR_col_stitch_ND[j,x,id,is]
            RSR_col_stitch[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=RSR_col_corr[j,z,id,is,2]
            RSR_col_stitch[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=RSR_col_stitch_ND[j,y,id,is]

            RSR_col_stitch_2[j,0:n_elements(x)-1,id,is]=RSR_col_stitch_ND_2[j,x,id,is]
            RSR_col_stitch_2[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=RSR_col_corr[j,z,id,is,2]
            RSR_col_stitch_2[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=RSR_col_stitch_ND_2[j,y,id,is]
            
            snr_col_stitch[j,0:n_elements(x)-1,id,is]=snr_col_stitch_ND[j,x,id,is]
            snr_col_stitch[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=snr_col[j,z,id,is,2]
            snr_col_stitch[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=snr_col_stitch_ND[j,y,id,is]
            
            snr_col_stitch_avg[j,0:n_elements(x)-1,id,is]=snr_col_stitch_ND_avg[j,x,id,is]
            snr_col_stitch_avg[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=snr_col[j,z,id,is,2]
            snr_col_stitch_avg[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=snr_col_stitch_ND_avg[j,y,id,is]
         ENDFOR
      ENDFOR
   ENDIF ELSE BEGIN
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR is=0, subsets[j]-1 DO BEGIN
            x=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,0] LT lambda_limit_stitch[j,id,is,0,2])
            y=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,0] GT lambda_limit_stitch[j,id,is,1,2])
            
            z=WHERE(lambda_uaid[j,id,0:ncol[j,2]-1,2] GT lambda_limit_stitch[j,id,is,0,2] $
                    AND lambda_uaid[j,id,0:ncol[j,2]-1,2] LT lambda_limit_stitch[j,id,is,1,2])
            
            ncol_stitch[j,id,is]=n_elements(x)+n_elements(y)+n_elements(z)
            
            lambda_stitch[j,0:n_elements(x)-1,id,is]=lambda_uaid[j,id,x,0]
            lambda_stitch[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=lambda_uaid[j,id,z,2]
            lambda_stitch[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=lambda_uaid[j,id,y,0]
            
            RSR_col_stitch[j,0:n_elements(x)-1,id,is]=RSR_col_corr[j,x,id,is,0]
            RSR_col_stitch[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=RSR_col_corr[j,z,id,is,2]
            RSR_col_stitch[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=RSR_col_corr[j,y,id,is,0]

            RSR_col_stitch_2[j,0:n_elements(x)-1,id,is]=RSR_col_corr[j,x,id,is,0]
            RSR_col_stitch_2[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=RSR_col_corr[j,z,id,is,2]
            RSR_col_stitch_2[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=RSR_col_corr[j,y,id,is,0]
            
            snr_col_stitch[j,0:n_elements(x)-1,id,is]=snr_col[j,x,id,is,0]
            snr_col_stitch[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=snr_col[j,z,id,is,2]
            snr_col_stitch[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=snr_col[j,y,id,is,0]
            
            snr_col_stitch_avg[j,0:n_elements(x)-1,id,is]=snr_col_avg[j,x,id,is,0]
            snr_col_stitch_avg[j,n_elements(x):n_elements(x)+n_elements(z)-1,id,is]=snr_col[j,z,id,is,2]
            snr_col_stitch_avg[j,n_elements(x)+n_elements(z):ncol_stitch[j,id,is]-1,id,is]=snr_col_avg[j,y,id,is,0]
         ENDFOR
      ENDFOR
   ENDELSE
ENDFOR

;*****************************************************************************
;  For M14, replace OOB with block function.
;*****************************************************************************

RSR_col_stitch_3=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)
RSR_col_stitch_4=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF band_name[j] NE 'M14' THEN continue

   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         x=where(lambda_stitch[j,*,id,is] NE 0)

         FOR iii=0, n_elements(x)-1 DO BEGIN
            IF lambda_stitch[j,iii,id,is] LT 6500. OR lambda_stitch[j,iii,id,is] GT 7000. THEN RSR_col_stitch_3[j,iii,id,is]=RSR_col_stitch[j,iii,id,is]
            IF lambda_stitch[j,iii,id,is] GE 6500. AND lambda_stitch[j,iii,id,is] LE 7000. THEN RSR_col_stitch_3[j,iii,id,is]=0.001

            IF lambda_stitch[j,iii,id,is] LT 6500. OR lambda_stitch[j,iii,id,is] GT 7000. THEN RSR_col_stitch_4[j,iii,id,is]=RSR_col_stitch_2[j,iii,id,is]
            IF lambda_stitch[j,iii,id,is] GE 6500. AND lambda_stitch[j,iii,id,is] LE 7000. THEN RSR_col_stitch_4[j,iii,id,is]=0.001*trans_corr[j]/trans_avg_corr[j,id,is]
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Eliminate any RSR value not corrected by RSO and remove any negatives
;*****************************************************************************

RSR_col_stitch_cut=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         FOR l=0, ncol_stitch[j,id,is]-1 DO BEGIN
            IF fpa EQ 'VisNIR' THEN BEGIN

            ENDIF
            IF fpa EQ 'SMWIR' THEN BEGIN
               IF snr_col_stitch_avg[j,l,id,is] GE snr_cutoff[j] THEN RSR_col_stitch_cut[j,l,id,is]=RSR_col_stitch[j,l,id,is]
               IF snr_col_stitch_avg[j,l,id,is] LT snr_cutoff[j] THEN RSR_col_stitch_cut[j,l,id,is]=0.
               
               IF RSR_col_stitch_cut[j,l,id,is] LT 0. THEN RSR_col_stitch_cut[j,l,id,is]=0.
            ENDIF
            IF fpa EQ 'LWIR' THEN BEGIN
               IF snr_col_stitch_avg[j,l,id,is] GE snr_cutoff[j] THEN RSR_col_stitch_cut[j,l,id,is]=RSR_col_stitch[j,l,id,is]
               IF snr_col_stitch_avg[j,l,id,is] LT snr_cutoff[j] THEN RSR_col_stitch_cut[j,l,id,is]=0.
               
               IF RSR_col_stitch_cut[j,l,id,is] LT 0. THEN RSR_col_stitch_cut[j,l,id,is]=0.
            ENDIF
         ENDFOR
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Correct for atmospheric effect in M13.
;*****************************************************************************

W=dblarr(6)
RSR_col_stitch_temp=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)
RSR_col_stitch_temp2=fltarr(n_elements(uaidname1),500,maxdet,maxsubsets)

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF band_name[j] NE 'M13' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      wave_fit_1=WHERE(lambda_stitch[j,0:ncol_stitch[j,id,0],id,0] GE 4177. AND lambda_stitch[j,0:ncol_stitch[j,id,0],id,0] LE 4185.)
      wave_fit_2=WHERE(lambda_stitch[j,0:ncol_stitch[j,id,0],id,0] GE 4253. AND lambda_stitch[j,0:ncol_stitch[j,id,0],id,0] LE 4261.)
      wave_fit_3=WHERE(lambda_stitch[j,0:ncol_stitch[j,id,0],id,0] GE 4314. AND lambda_stitch[j,0:ncol_stitch[j,id,0],id,0] LE 4322.)
      
      wave_fit=[wave_fit_1,wave_fit_2,wave_fit_3]
      
      wave_interpol=WHERE(lambda_stitch[j,0:ncol_stitch[j,id,0],id,0] GE 4177. AND lambda_stitch[j,0:ncol_stitch[j,id,0],id,0] LE 4322.)
      
      FOR iii=0, ncol_stitch[j,id,0]-1 DO RSR_col_stitch_temp[j,iii,id,0]=log(RSR_col_stitch[j,iii,id,0])
      FOR iii=0, ncol_stitch[j,id,0]-1 DO RSR_col_stitch_temp2[j,iii,id,0]=log(RSR_col_stitch_2[j,iii,id,0])

      ;fit_m13=poly_fit(lambda_stitch[j,wave_fit,id,0],RSR_col_stitch[j,wave_fit,id,0],1,/DOUBLE)
      fit_m13=poly_fit(lambda_stitch[j,wave_fit,id,0],RSR_col_stitch_temp[j,wave_fit,id,0],1,/DOUBLE)
      fit_m13_2=poly_fit(lambda_stitch[j,wave_fit,id,0],RSR_col_stitch_temp2[j,wave_fit,id,0],1,/DOUBLE)
      
      RSR_col_stitch[j,wave_interpol,id,0]=10^(fit_m13[0]+fit_m13[1]*lambda_stitch[j,wave_interpol,id,0])
      RSR_col_stitch_cut[j,wave_interpol,id,0]=10^(fit_m13[0]+fit_m13[1]*lambda_stitch[j,wave_interpol,id,0])

      RSR_col_stitch_2[j,wave_interpol,id,0]=10^(fit_m13_2[0]+fit_m13_2[1]*lambda_stitch[j,wave_interpol,id,0])
   ENDFOR
ENDFOR

;*****************************************************************************
;  Overwrite some M12 detector 1 RSR with detector 3 (due to low singal)
;*****************************************************************************

;FOR j=0, n_elements(uaidname1)-1 DO BEGIN
;   IF band_name[j] NE 'M12' THEN continue
;   
;   wave_replace=WHERE(lambda_stitch[j,0:ncol_stitch[j,0,0],0,0] GE 3340. AND lambda_stitch[j,0:ncol_stitch[j,0,0],0,0] LE 3460.)
;   
;   RSR_col_stitch[j,wave_replace,0,0]=RSR_col_stitch[j,wave_replace,2,0]
;   RSR_col_stitch_cut[j,wave_replace,0,0]=RSR_col_stitch_cut[j,wave_replace,2,0]
;ENDFOR

;*****************************************************************************
;  Average the OOB over all detectors.
;*****************************************************************************

;FOR j=0, n_elements(uaidname1)-1 DO BEGIN
;    FOR id=0, detectors[j]-1 DO BEGIN
;        FOR is=0, subsets[j]-1 DO BEGIN
;            FOR col=0, ncol_stitch[j,id,is]-1 DO BEGIN
;                IF lambda_stitch[j,col,id,is] GT lambda_limit_stitch[j,id,is,0,2] AND lambda_stitch[j,col,id,is] LT lambda_limit_stitch[j,id,is,1,2] THEN continue

;                RSR_col_stitch[j,col,id,is]=mean(RSR_col_stitch[j,col,0:detectors[j]-1,is])
;            ENDFOR
;        ENDFOR
;    ENDFOR
;ENDFOR

;*****************************************************************************
;  Calculate the IOOB
;*****************************************************************************

int_1=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
int_2=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
IOOB=fltarr(n_elements(uaidname1),maxdet,maxsubsets)

constant=0.0000217588

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname3[j] EQ 'A' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         m=0
         FOR l=1, ncol_stitch[j,id,is]-1 DO BEGIN
            rad_5800=Planck(5800.,lambda_stitch[j,l,id,is]*0.001)
            rad_300=Planck(300.,lambda_stitch[j,l,id,is]*0.001)
            
            IF lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is] LT 0 THEN continue
            IF RSR_col_stitch[j,l,id,is] LT 0 THEN continue
            
            int_1[j,id,is]=int_1[j,id,is]+RSR_col_stitch[j,l,id,is]*(constant*rad_5800+rad_300)*(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
            
            ib_col=WHERE(lambda_stitch[j,*,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,*,id,is] LT lambda_limit_corr[j,id,is,1,2])
            num_col=n_elements(ib_col)
            IF lambda_stitch[j,l,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,l,id,is] LT lambda_limit_corr[j,id,is,1,2] THEN BEGIN
               IF m EQ 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_limit_corr[j,id,is,0,2])*0.001
               
               IF m NE 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
               
               IF m EQ num_col-1 THEN int_2[j,id,is]=int_2[j,id,is]+0.01*(constant*rad_5800+rad_300) $
                                                     *(lambda_limit_corr[j,id,is,1,2]-lambda_stitch[j,l,id,is])*0.001
               m=m+1
            ENDIF
         ENDFOR
         IOOB[j,id,is]=(int_1[j,id,is]-int_2[j,id,is])/int_1[j,id,is]
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Recalculate the IOOB using the cutoff RSR
;*****************************************************************************

int_1=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
int_2=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
IOOB2=fltarr(n_elements(uaidname1),maxdet,maxsubsets)

constant=0.0000217588

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname3[j] EQ 'A' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         m=0
         FOR l=1, ncol_stitch[j,id,is]-1 DO BEGIN
            rad_5800=Planck(5800.,lambda_stitch[j,l,id,is]*0.001)
            rad_300=Planck(300.,lambda_stitch[j,l,id,is]*0.001)
            
            IF lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is] LT 0 THEN continue
            IF RSR_col_stitch_cut[j,l,id,is] LT 0 THEN continue
            
            int_1[j,id,is]=int_1[j,id,is]+RSR_col_stitch_cut[j,l,id,is]*(constant*rad_5800+rad_300)*(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
            
            ib_col=WHERE(lambda_stitch[j,*,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,*,id,is] LT lambda_limit_corr[j,id,is,1,2])
            num_col=n_elements(ib_col)
            IF lambda_stitch[j,l,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,l,id,is] LT lambda_limit_corr[j,id,is,1,2] THEN BEGIN
               IF m EQ 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch_cut[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_limit_corr[j,id,is,0,2])*0.001
               
               IF m NE 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch_cut[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
               
               IF m EQ num_col-1 THEN int_2[j,id,is]=int_2[j,id,is]+0.01*(constant*rad_5800+rad_300) $
                                                     *(lambda_limit_corr[j,id,is,1,2]-lambda_stitch[j,l,id,is])*0.001
               m=m+1
            ENDIF
         ENDFOR
         IOOB2[j,id,is]=(int_1[j,id,is]-int_2[j,id,is])/int_1[j,id,is]
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the IOOB with the average ND transmittance.
;*****************************************************************************

int_1=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
int_2=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
IOOB3=fltarr(n_elements(uaidname1),maxdet,maxsubsets)

constant=0.0000217588

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname3[j] EQ 'A' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         m=0
         FOR l=1, ncol_stitch[j,id,is]-1 DO BEGIN
            rad_5800=Planck(5800.,lambda_stitch[j,l,id,is]*0.001)
            rad_300=Planck(300.,lambda_stitch[j,l,id,is]*0.001)
            
            IF lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is] LT 0 THEN continue
            IF RSR_col_stitch_2[j,l,id,is] LT 0 THEN continue
            
            int_1[j,id,is]=int_1[j,id,is]+RSR_col_stitch_2[j,l,id,is]*(constant*rad_5800+rad_300)*(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
            
            ib_col=WHERE(lambda_stitch[j,*,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,*,id,is] LT lambda_limit_corr[j,id,is,1,2])
            num_col=n_elements(ib_col)
            IF lambda_stitch[j,l,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,l,id,is] LT lambda_limit_corr[j,id,is,1,2] THEN BEGIN
               IF m EQ 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch_2[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_limit_corr[j,id,is,0,2])*0.001
               
               IF m NE 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch_2[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
               
               IF m EQ num_col-1 THEN int_2[j,id,is]=int_2[j,id,is]+0.01*(constant*rad_5800+rad_300) $
                                                     *(lambda_limit_corr[j,id,is,1,2]-lambda_stitch[j,l,id,is])*0.001
               m=m+1
            ENDIF
         ENDFOR
         IOOB3[j,id,is]=(int_1[j,id,is]-int_2[j,id,is])/int_1[j,id,is]
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the IOOB using the modified M14 (with average ND).
;*****************************************************************************

int_1=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
int_2=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
IOOB5=fltarr(n_elements(uaidname1),maxdet,maxsubsets)

constant=0.0000217588

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname3[j] EQ 'A' THEN continue
   IF band_name[j] NE 'M14' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         m=0
         FOR l=1, ncol_stitch[j,id,is]-1 DO BEGIN
            rad_5800=Planck(5800.,lambda_stitch[j,l,id,is]*0.001)
            rad_300=Planck(300.,lambda_stitch[j,l,id,is]*0.001)
            
            IF lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is] LT 0 THEN continue
            IF RSR_col_stitch_4[j,l,id,is] LT 0 THEN continue
            
            int_1[j,id,is]=int_1[j,id,is]+RSR_col_stitch_4[j,l,id,is]*(constant*rad_5800+rad_300)*(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
            
            ib_col=WHERE(lambda_stitch[j,*,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,*,id,is] LT lambda_limit_corr[j,id,is,1,2])
            num_col=n_elements(ib_col)
            IF lambda_stitch[j,l,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,l,id,is] LT lambda_limit_corr[j,id,is,1,2] THEN BEGIN
               IF m EQ 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch_4[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_limit_corr[j,id,is,0,2])*0.001
               
               IF m NE 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch_4[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
               
               IF m EQ num_col-1 THEN int_2[j,id,is]=int_2[j,id,is]+0.01*(constant*rad_5800+rad_300) $
                                                     *(lambda_limit_corr[j,id,is,1,2]-lambda_stitch[j,l,id,is])*0.001
               m=m+1
            ENDIF
         ENDFOR
         IOOB5[j,id,is]=(int_1[j,id,is]-int_2[j,id,is])/int_1[j,id,is]
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the IOOB using the modified M14.
;*****************************************************************************

int_1=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
int_2=fltarr(n_elements(uaidname1),maxdet,maxsubsets)
IOOB4=fltarr(n_elements(uaidname1),maxdet,maxsubsets)

constant=0.0000217588

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname3[j] EQ 'A' THEN continue
   IF band_name[j] NE 'M14' THEN continue
   
   FOR id=0, detectors[j]-1 DO BEGIN
      FOR is=0, subsets[j]-1 DO BEGIN
         m=0
         FOR l=1, ncol_stitch[j,id,is]-1 DO BEGIN
            rad_5800=Planck(5800.,lambda_stitch[j,l,id,is]*0.001)
            rad_300=Planck(300.,lambda_stitch[j,l,id,is]*0.001)
            
            IF lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is] LT 0 THEN continue
            IF RSR_col_stitch_3[j,l,id,is] LT 0 THEN continue
            
            int_1[j,id,is]=int_1[j,id,is]+RSR_col_stitch_3[j,l,id,is]*(constant*rad_5800+rad_300)*(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
            
            ib_col=WHERE(lambda_stitch[j,*,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,*,id,is] LT lambda_limit_corr[j,id,is,1,2])
            num_col=n_elements(ib_col)
            IF lambda_stitch[j,l,id,is] GT lambda_limit_corr[j,id,is,0,2] AND lambda_stitch[j,l,id,is] LT lambda_limit_corr[j,id,is,1,2] THEN BEGIN
               IF m EQ 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch_3[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_limit_corr[j,id,is,0,2])*0.001
               
               IF m NE 0 THEN int_2[j,id,is]=int_2[j,id,is]+RSR_col_stitch_3[j,l,id,is]*(constant*rad_5800+rad_300) $
                                             *(lambda_stitch[j,l,id,is]-lambda_stitch[j,l-1,id,is])*0.001
               
               IF m EQ num_col-1 THEN int_2[j,id,is]=int_2[j,id,is]+0.01*(constant*rad_5800+rad_300) $
                                                     *(lambda_limit_corr[j,id,is,1,2]-lambda_stitch[j,l,id,is])*0.001
               m=m+1
            ENDIF
         ENDFOR
         IOOB4[j,id,is]=(int_1[j,id,is]-int_2[j,id,is])/int_1[j,id,is]
      ENDFOR
   ENDFOR
ENDFOR

;*****************************************************************************
;  Set up plotting environment.
;*****************************************************************************

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname1[j] EQ 'A' THEN continue
   
   set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
   set_plot,'ps'
   outfname=outpath+'RSR.stiched.'+band_name[j]+'.2.ps'
   device,/landscape,filename=outfname,/color
   !p.multi=[0,1,1]
   !p.position=0
   
   SS_1=['Lower 1%','Higher 1%']
   SS_2=['Lower 1% SS1','Lower 1% SS2','Higher 1% SS1','Higher 1% SS2']
   SS_3=['SS1','SS2']
;goto,skip_plot    
;*****************************************************************************
;  SpMA response versus wavelength.
;*****************************************************************************

   title1='SpMA RSO'
   
   plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],rso_spma[j,0:ncol[j,0]-1,0],yrange=[min(rso_spma[j,0:ncol[j,0]-1,0]),max(rso_spma[j,0:ncol[j,0]-1,0])], $
        xrange=[min(lambda_uaid[j,0,0:ncol[j,0]-1,0]),max(lambda_uaid[j,0,0:ncol[j,0]-1,0])],xtit='Wavelength (nm)',ytit='SpMA RSO',charsize=1.5,charthick=5,/nodata
   
   oplot,lambda_uaid[j,0,0:ncol[j,0]-1,0],rso_spma[j,0:ncol[j,0]-1,0]
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   
;*****************************************************************************
;  Calculated center wavelength vs detector.
;*****************************************************************************

   !p.multi=[0,2,2]
   !p.position=0
   
   D=findgen(40)
   
   IF fpa EQ 'DNB' THEN BEGIN
      center_wave_spec=[700,700,700,700]
      center_wave_spec_tol=[14,14,14,14]
   ENDIF
   IF fpa EQ 'VisNIR' THEN BEGIN
      center_wave_spec=[640,865,412,445,488,555,672,746,865]
      center_wave_spec_tol=[6,8,2,3,4,4,5,2,8]
   ENDIF
   IF fpa EQ 'SMWIR' THEN BEGIN
      center_wave_spec=[1610,3740,1240,1378,1610,2250,3700,4050]
      center_wave_spec_tol=[14,40,5,4,14,13,32,34]
   ENDIF
   IF fpa EQ 'LWIR' THEN BEGIN
      center_wave_spec=[11450,8550,10763,12013,12013]
      center_wave_spec_tol=[125,70,113,88,88]
   ENDIF
   
   title2=band_name[j]
   
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(center_lambda_corr[j,0:detectors[j]-1,0,2],min=ymin)
      IF ymax LT center_wave_spec[j]+center_wave_spec_tol[j] THEN ymax=center_wave_spec[j]+center_wave_spec_tol[j]+2
      IF ymin GT center_wave_spec[j]-center_wave_spec_tol[j] THEN ymin=center_wave_spec[j]-center_wave_spec_tol[j]-2
      
      plot,D[0:detectors[j]-1]+1,center_lambda_corr[j,0:detectors[j]-1,0,2],yrange=[ymin,ymax],xrange=[0,32],xtit='Detector',ytit='Center (nm)', $
           title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,center_lambda_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,center_lambda_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,center_lambda_corr[j,0:detectors[j]-1,1,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,center_lambda_corr[j,0:detectors[j]-1,1,2],color=lcol[1],thick=3
      
      oplot,[0,40],[center_wave_spec[j],center_wave_spec[j]],color=lcol[2],thick=3
      oplot,[0,40],[center_wave_spec[j]+center_wave_spec_tol[j],center_wave_spec[j]+center_wave_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
      oplot,[0,40],[center_wave_spec[j]-center_wave_spec_tol[j],center_wave_spec[j]-center_wave_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(center_lambda_corr[j,0:detectors[j]-1,0,2],min=ymin)
      IF ymax LT center_wave_spec[j]+center_wave_spec_tol[j] THEN ymax=center_wave_spec[j]+center_wave_spec_tol[j]+2
      IF ymin GT center_wave_spec[j]-center_wave_spec_tol[j] THEN ymin=center_wave_spec[j]-center_wave_spec_tol[j]-2
      
      plot,D[0:detectors[j]-1]+1,center_lambda_corr[j,0:detectors[j]-1,0,2],yrange=[ymin,ymax],xrange=[0,16],xtit='Detector',ytit='Center (nm)', $
           title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,center_lambda_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,center_lambda_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      
      oplot,[0,40],[center_wave_spec[j],center_wave_spec[j]],color=lcol[2],thick=3
      oplot,[0,40],[center_wave_spec[j]+center_wave_spec_tol[j],center_wave_spec[j]+center_wave_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
      oplot,[0,40],[center_wave_spec[j]-center_wave_spec_tol[j],center_wave_spec[j]-center_wave_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
   ENDIF

;*****************************************************************************
;  Calculated bandwidth vs detector.
;*****************************************************************************

   IF fpa EQ 'DNB' THEN BEGIN
      bandwidth_spec=[400,400,400,400]
      bandwidth_spec_tol=[20,20,20,20]
   ENDIF
   IF fpa EQ 'VisNIR' THEN BEGIN
      bandwidth_spec=[80,39,20,18,20,20,20,15,39]
      bandwidth_spec_tol=[6,5,2,2,3,3,3,2,5]
   ENDIF
   IF fpa EQ 'SMWIR' THEN BEGIN
      bandwidth_spec=[60,380,20,15,60,50,180,155]
      bandwidth_spec_tol=[9,30,4,3,9,6,20,20]
   ENDIF
   IF fpa EQ 'LWIR' THEN BEGIN
      bandwidth_spec=[1900,300,1000,950,950]
      bandwidth_spec_tol=[100,40,100,50,50]
   ENDIF
        
   title2=band_name[j]

   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(bandwidth_corr[j,0:detectors[j]-1,0,2],min=ymin)
      IF ymax LT bandwidth_spec[j]+bandwidth_spec_tol[j] THEN ymax=bandwidth_spec[j]+bandwidth_spec_tol[j]+2
      IF ymin GT bandwidth_spec[j]-bandwidth_spec_tol[j] THEN ymin=bandwidth_spec[j]-bandwidth_spec_tol[j]-2
      
      plot,D[0:detectors[j]-1]+1,bandwidth_corr[j,0:detectors[j]-1,0,2],yrange=[ymin,ymax],xrange=[0,32],xtit='Detector',ytit='Bandwith (nm)',title=title2, $
           charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,bandwidth_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,bandwidth_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,bandwidth_corr[j,0:detectors[j]-1,1,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,bandwidth_corr[j,0:detectors[j]-1,1,2],color=lcol[1],thick=3
      
      oplot,[0,40],[bandwidth_spec[j],bandwidth_spec[j]],color=lcol[2],thick=3
      oplot,[0,40],[bandwidth_spec[j]+bandwidth_spec_tol[j],bandwidth_spec[j]+bandwidth_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
      oplot,[0,40],[bandwidth_spec[j]-bandwidth_spec_tol[j],bandwidth_spec[j]-bandwidth_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(bandwidth_corr[j,0:detectors[j]-1,0,2],min=ymin)
      IF ymax LT bandwidth_spec[j]+bandwidth_spec_tol[j] THEN ymax=bandwidth_spec[j]+bandwidth_spec_tol[j]+2
      IF ymin GT bandwidth_spec[j]-bandwidth_spec_tol[j] THEN ymin=bandwidth_spec[j]-bandwidth_spec_tol[j]-2
      
      plot,D[0:detectors[j]-1]+1,bandwidth_corr[j,0:detectors[j]-1,0,2],yrange=[ymin,ymax],xrange=[0,16],xtit='Detector',ytit='Bandwith (nm)',title=title2, $
           charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,bandwidth_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,bandwidth_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      
      oplot,[0,40],[bandwidth_spec[j],bandwidth_spec[j]],color=lcol[2],thick=3
      oplot,[0,40],[bandwidth_spec[j]+bandwidth_spec_tol[j],bandwidth_spec[j]+bandwidth_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
      oplot,[0,40],[bandwidth_spec[j]-bandwidth_spec_tol[j],bandwidth_spec[j]-bandwidth_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
   ENDIF

;*****************************************************************************
;  Calculated 1% limits verus detector.
;*****************************************************************************

   IF fpa EQ 'DNB' THEN BEGIN
      low_limit_spec=[470,470,470,470]
      high_limit_spec=[960,960,960,960]
   ENDIF
   IF fpa EQ 'VisNIR' THEN BEGIN
      low_limit_spec=[565,802,376,417,455,523,638,721,801]
      high_limit_spec=[715,928,444,473,521,589,706,771,929]
   ENDIF
   IF fpa EQ 'SMWIR' THEN BEGIN
      low_limit_spec=[1509,3340,1205,1351,1509,2167,3410,3790]
      high_limit_spec=[1709,4140,1275,1405,1709,2333,3990,4310]
   ENDIF
   IF fpa EQ 'LWIR' THEN BEGIN
      low_limit_spec=[9900,8050,9700,11060,11060]
      high_limit_spec=[12900,9050,11740,13050,13050]
   ENDIF
   
   title2=band_name[j]
    
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(lambda_limit_corr[j,0:detectors[j]-1,*,*,2],min=ymin)
      IF ymax LT high_limit_spec[j] THEN ymax=high_limit_spec[j]+2
      IF ymin GT low_limit_spec[j] THEN ymin=low_limit_spec[j]-2
      
      plot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,0,2],yrange=[ymin,ymax],xrange=[0,32],xtit='Detector',ytit='1% limits (nm)',title=title2, $
           charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,0,2],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,1,0,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,1,0,2],color=lcol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,1,2],symsize=1,color=lcol[2],psym=symbol[2],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,1,2],color=lcol[2],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,1,1,2],symsize=1,color=lcol[3],psym=symbol[3],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,1,1,2],color=lcol[3],thick=3
      
      oplot,[0,40],[high_limit_spec[j],high_limit_spec[j]],color=lcol[2],thick=3,linestyle=2
      oplot,[0,40],[low_limit_spec[j],low_limit_spec[j]],color=lcol[2],thick=3,linestyle=2
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(lambda_limit_corr[j,0:detectors[j]-1,0,*,2],min=ymin)
      IF ymax LT high_limit_spec[j] THEN ymax=high_limit_spec[j]+2
      IF ymin GT low_limit_spec[j] THEN ymin=low_limit_spec[j]-2
      
      plot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,0,2],yrange=[ymin,ymax],xrange=[0,16],xtit='Detector',ytit='1% limits (nm)',title=title2, $
           charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,0,2],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,1,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_limit_corr[j,0:detectors[j]-1,0,1,2],color=lcol[1],thick=3
      
      oplot,[0,40],[high_limit_spec[j],high_limit_spec[j]],color=lcol[2],thick=3,linestyle=2
      oplot,[0,40],[low_limit_spec[j],low_limit_spec[j]],color=lcol[2],thick=3,linestyle=2
   ENDIF
   
;*****************************************************************************
;  IOOB
;*****************************************************************************

   IF fpa EQ 'DNB' THEN BEGIN
      ioob_spec=[0.1,0.1,0.1,0.1]
   ENDIF
   IF fpa EQ 'VisNIR' THEN BEGIN
      ioob_spec=[1.0,0.5,1.5,1.0,0.7,1.2,1.5,1.0,1.5]
   ENDIF
   IF fpa EQ 'SMWIR' THEN BEGIN
      ioob_spec=[0.7,0.5,0.8,1.0,0.7,1.0,1.1,1.3]
   ENDIF
   IF fpa EQ 'LWIR' THEN BEGIN
      ioob_spec=[0.4,0.9,0.4,0.4,0.4]
   ENDIF
   
   title2=band_name[j]
   
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(IOOB[j,0:detectors[j]-1,*]*100)
      IF ymax LT ioob_spec[j] THEN ymax=ioob_spec[j]+2
      
      plot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,yrange=[0,ymax],xrange=[0,32],xtit='Detector',ytit='IOOB (%)',title=title2, $
           charsize=1.5,charthick=5,/nodata

      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,1]*100,symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,1]*100,color=lcol[1],thick=3
      
      oplot,D[0:detectors[j]-1]+1,IOOB2[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[2],psym=symbol[2],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB2[j,0:detectors[j]-1,0]*100,color=lcol[2],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB2[j,0:detectors[j]-1,1]*100,symsize=1,color=lcol[3],psym=symbol[3],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB2[j,0:detectors[j]-1,1]*100,color=lcol[3],thick=3
      
      oplot,[0,40],[ioob_spec[j],ioob_spec[j]],color=lcol[2],thick=3,linestyle=2
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(IOOB[j,0:detectors[j]-1,0]*100)
      IF ymax LT ioob_spec[j] THEN ymax=ioob_spec[j]+2
      
      plot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,yrange=[0,ymax],xrange=[0,16],xtit='Detector',ytit='IOOB (%)',title=title2, $
           charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,color=lcol[0],thick=3
      
      oplot,D[0:detectors[j]-1]+1,IOOB2[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[2],psym=symbol[2],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB2[j,0:detectors[j]-1,0]*100,color=lcol[2],thick=3
      
      oplot,[0,40],[ioob_spec[j],ioob_spec[j]],color=lcol[2],thick=3,linestyle=2
   ENDIF
   
;*****************************************************************************
;  Calculated centroid wavelength vs detector.
;*****************************************************************************

   !p.multi=[0,1,1]
   !p.position=0
   
   title2=band_name[j]
   
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(lambda_centroid_corr[j,0:detectors[j]-1,0,2],min=ymin)
      IF ymax LT center_wave_spec[j]+center_wave_spec_tol[j] THEN ymax=center_wave_spec[j]+center_wave_spec_tol[j]+2
      IF ymin GT center_wave_spec[j]-center_wave_spec_tol[j] THEN ymin=center_wave_spec[j]-center_wave_spec_tol[j]-2
      
      plot,D[0:detectors[j]-1]+1,lambda_centroid_corr[j,0:detectors[j]-1,0,2],yrange=[ymin,ymax],xrange=[0,32],xtit='Detector', $
           ytit='Centroid (nm)',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,lambda_centroid_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_centroid_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_centroid_corr[j,0:detectors[j]-1,1,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_centroid_corr[j,0:detectors[j]-1,1,2],color=lcol[1],thick=3
      
      oplot,[0,40],[center_wave_spec[j],center_wave_spec[j]],color=lcol[2],thick=3
      oplot,[0,40],[center_wave_spec[j]+center_wave_spec_tol[j],center_wave_spec[j]+center_wave_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
      oplot,[0,40],[center_wave_spec[j]-center_wave_spec_tol[j],center_wave_spec[j]-center_wave_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
      set_legend,1,SS_3,lcol,symbol
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(lambda_centroid_corr[j,0:detectors[j]-1,0,2],min=ymin)
      IF ymax LT center_wave_spec[j]+center_wave_spec_tol[j] THEN ymax=center_wave_spec[j]+center_wave_spec_tol[j]+2
      IF ymin GT center_wave_spec[j]-center_wave_spec_tol[j] THEN ymin=center_wave_spec[j]-center_wave_spec_tol[j]-2
      
      plot,D[0:detectors[j]-1]+1,lambda_centroid_corr[j,0:detectors[j]-1,0,2],yrange=[ymin,ymax],xrange=[0,16],xtit='Detector', $
           ytit='Centroid (nm)',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,lambda_centroid_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,lambda_centroid_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      
      oplot,[0,40],[center_wave_spec[j],center_wave_spec[j]],color=lcol[2],thick=3
      oplot,[0,40],[center_wave_spec[j]+center_wave_spec_tol[j],center_wave_spec[j]+center_wave_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
      oplot,[0,40],[center_wave_spec[j]-center_wave_spec_tol[j],center_wave_spec[j]-center_wave_spec_tol[j]],linestyle=2,color=lcol[2],thick=3
   ENDIF

;*****************************************************************************
;  Calculated characterization uncertainty (center wavelength) vs detector.
;*****************************************************************************

   !p.multi=[0,2,1]
   !p.position=0
   
   IF fpa EQ 'DNB' THEN BEGIN
      char_uncert_spec=[1.0,1.0,1.0,1.0]
   ENDIF
   IF fpa EQ 'VisNIR' THEN BEGIN
      char_uncert_spec=[1.0,1.3,1.0,1.0,1.0,1.0,1.0,1.0,1.3]
   ENDIF
   IF fpa EQ 'SMWIR' THEN BEGIN
      char_uncert_spec=[2.3,3.7,2.3,1.9,3.7,2.0,1.0,1.0]
   ENDIF
   IF fpa EQ 'LWIR' THEN BEGIN
      char_uncert_spec=[20.0,11.0,10.8,6.0,6.0]
   ENDIF
   
   title2=band_name[j]+' Center'
   
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(char_uncert_center_corr[j,0:detectors[j]-1,0,2])
      IF ymax LT char_uncert_spec[j] THEN ymax=char_uncert_spec[j]*1.1
      
      plot,D[0:detectors[j]-1]+1,char_uncert_center_corr[j,0:detectors[j]-1,0,2],yrange=[0.0,ymax],xrange=[0,32],xtit='Detector', $
           ytit='Characterization Uncertainty (nm)',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,char_uncert_center_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,char_uncert_center_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,char_uncert_center_corr[j,0:detectors[j]-1,1,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,char_uncert_center_corr[j,0:detectors[j]-1,1,2],color=lcol[1],thick=3
      
      oplot,[0,40],[char_uncert_spec[j],char_uncert_spec[j]],color=lcol[2],thick=3,linestyle=2
      set_legend,1,SS_3,lcol,symbol
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(char_uncert_center_corr[j,0:detectors[j]-1,0,2])
      IF ymax LT char_uncert_spec[j] THEN ymax=char_uncert_spec[j]*1.1
      
      plot,D[0:detectors[j]-1]+1,char_uncert_center_corr[j,0:detectors[j]-1,0,2],yrange=[0.0,ymax],xrange=[0,16],xtit='Detector', $
           ytit='Characterization Uncertainty (nm)',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,char_uncert_center_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,char_uncert_center_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      
      oplot,[0,40],[char_uncert_spec[j],char_uncert_spec[j]],color=lcol[2],thick=3,linestyle=2
   ENDIF

;*****************************************************************************
;  Calculated characterization uncertainty (center wavelength) vs detector.
;*****************************************************************************

   title2=band_name[j]+' Bandwidth'
   
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2])
      IF ymax LT char_uncert_spec[j] THEN ymax=char_uncert_spec[j]*1.1
      
      plot,D[0:detectors[j]-1]+1,char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2],yrange=[0.0,ymax],xrange=[0,32],xtit='Detector', $
           ytit='Characterization Uncertainty (nm)',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,char_uncert_bandwidth_corr[j,0:detectors[j]-1,1,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,char_uncert_bandwidth_corr[j,0:detectors[j]-1,1,2],color=lcol[1],thick=3
      
      oplot,[0,40],[char_uncert_spec[j],char_uncert_spec[j]],color=lcol[2],thick=3,linestyle=2
      set_legend,1,SS_3,lcol,symbol
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2])
      IF ymax LT char_uncert_spec[j] THEN ymax=char_uncert_spec[j]*1.1
      
      plot,D[0:detectors[j]-1]+1,char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2],yrange=[0.0,ymax],xrange=[0,16],xtit='Detector', $
           ytit='Characterization Uncertainty (nm)',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2],color=lcol[0],thick=3
      
      oplot,[0,40],[char_uncert_spec[j],char_uncert_spec[j]],color=lcol[2],thick=3,linestyle=2
   ENDIF

;*****************************************************************************
;  Plot dn vs wavelength (FP-15)
;*****************************************************************************

   title1='dn vs Wavelength (FP-15)'
   
   ymax=max(dn_col[j,0:ncol[j,2]-1,*,*,2],min=ymin)
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],dn_col[j,0:ncol[j,2]-1,8,0,2],yrange=[ymin,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='dn (uncorrected)',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],dn_col[j,0:ncol[j,2]-1,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],dn_col[j,0:ncol[j,2]-1,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot RSR vs wavelength (FP-15)
;*****************************************************************************

   title1='RSR (RSO corrected) vs Wavelength (FP-15)'
   
   ymin=-1.0
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,3,3]
   !p.position=0
   
   FOR id=0, detectors[j]-1 DO BEGIN
      title2=band_name[j]
      
      plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],yrange=[0.00001,1],xrange=[xmin,xmax], $
           xtit='Wavelength (nm)',ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],color=lcol[id],thick=3
      
      XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
      IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
      IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   ENDFOR

   title1='RSR (RSO corrected) vs Wavelength (FP-15)'
   
   ymin=-1.0
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,8,0,2],yrange=[0.00001,1],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN   
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='RSR (RSO corrected) vs Wavelength (FP-15) with Atmospheric Absorption'
   
   atm=WHERE(1000*absorption[0,*] GT min(lambda_uaid[j,0,0:ncol[j,2]-1,2]) AND 1000*absorption[0,*] LT max(lambda_uaid[j,0,0:ncol[j,2]-1,2]))
   
   ymin=-1.0
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,8,0,2],yrange=[0.00001,1],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata
   
   oplot,absorption[0,atm]*1000,absorption[1,atm],color=lcol[2]
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot RSR vs wavelength, edge vs center detectors (FP-15).
;*****************************************************************************

   IF band_type[j] EQ 0 THEN BEGIN 
      center_det_1=7
      center_det_2=8
      edge_det_1=0
      edge_det_2=1
      edge_det_3=14 
      edge_det_4=15
   ENDIF
   IF band_type[j] EQ 1 THEN BEGIN 
      center_det_1=15
      center_det_2=16
      edge_det_1=0
      edge_det_2=1
      edge_det_3=30
      edge_det_4=31
   ENDIF
   
   title1='RSR (RSO corrected) vs Wavelength,edge vs center (FP-15)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,center_det_1,0,2],yrange=[0.00001,1],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata
   
   oplot,lambda_uaid[j,center_det_1,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,center_det_1,0,2],symsize=1,color=lcol[center_det_1],psym=symbol[center_det_1],thick=3
   oplot,lambda_uaid[j,center_det_1,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,center_det_1,0,2],color=lcol[center_det_1],thick=3
   oplot,lambda_uaid[j,center_det_2,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,center_det_2,0,2],symsize=1,color=lcol[center_det_2],psym=symbol[center_det_2],thick=3
   oplot,lambda_uaid[j,center_det_2,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,center_det_2,0,2],color=lcol[center_det_2],thick=3
   oplot,lambda_uaid[j,edge_det_1,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,edge_det_1,0,2],symsize=1,color=lcol[edge_det_1],psym=symbol[edge_det_1],thick=3
   oplot,lambda_uaid[j,edge_det_1,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,edge_det_1,0,2],color=lcol[edge_det_1],thick=3
   oplot,lambda_uaid[j,edge_det_2,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,edge_det_2,0,2],symsize=1,color=lcol[edge_det_2],psym=symbol[edge_det_2],thick=3
   oplot,lambda_uaid[j,edge_det_2,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,edge_det_2,0,2],color=lcol[edge_det_2],thick=3
   oplot,lambda_uaid[j,edge_det_3,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,edge_det_3,0,2],symsize=1,color=lcol[edge_det_3],psym=symbol[edge_det_3],thick=3
   oplot,lambda_uaid[j,edge_det_3,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,edge_det_3,0,2],color=lcol[edge_det_3],thick=3
   oplot,lambda_uaid[j,edge_det_4,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,edge_det_4,0,2],symsize=1,color=lcol[edge_det_4],psym=symbol[edge_det_4],thick=3
   oplot,lambda_uaid[j,edge_det_4,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,edge_det_4,0,2],color=lcol[edge_det_4],thick=3
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
;*****************************************************************************
;  Plot RSR vs wavelength, log plot (FP-15)
;*****************************************************************************

   title1='RSR vs Wavelength (FP-15)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,8,0,2],yrange=[0.00001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='RSR vs Wavelength (FP-15)) with Atmospheric Absorption'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,8,0,2],yrange=[0.00001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   oplot,absorption[0,atm]*1000,absorption[1,atm],color=lcol[2]
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],RSR_col_corr[j,0:ncol[j,2]-1,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot std and snr vs wavelength (FP-15)
;*****************************************************************************

   title1='Standard deviation vs Wavelength (FP-15)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   ymax=max(std_col[j,0:ncol[j,2]-1,0:detectors[j]-1,0,2],min=ymin)
   plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],std_col[j,0:ncol[j,2]-1,0,0,2],yrange=[ymin,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='Standard deviation',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],std_col[j,0:ncol[j,2]-1,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],std_col[j,0:ncol[j,2]-1,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='SNR vs Wavelength (FP-15)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,2]-1,2],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   ymax=max(snr_col[j,0:ncol[j,2]-1,0:detectors[j]-1,0,2],min=ymin)
   plot,lambda_uaid[j,0,0:ncol[j,2]-1,2],snr_col[j,0:ncol[j,2]-1,0,0,2],yrange=[ymin,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],snr_col[j,0:ncol[j,2]-1,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,2]-1,2],snr_col[j,0:ncol[j,2]-1,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Calculated transmittance of ND filter
;*****************************************************************************

   IF uaidname2[j] EQ 'A' THEN goto,skip_fp16_ib
   
   title1='Calculated Transimittance of ND filter versus Detector'
   title2=band_name[j]
   
   !p.multi=[0,1,1]
   !p.position=0
   
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(trans_avg_corr[j,0:detectors[j]-1,0],min=ymin)
      
      plot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],yrange=[ymin,ymax],xrange=[0,32],xtit='Detector', $
           ytit='Transimittance',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,1],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,1],color=lcol[1],thick=3
      
      XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
      set_legend,1,SS_3,lcol,symbol
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(trans_avg_corr[j,0:detectors[j]-1,0],min=ymin)
      
      plot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],yrange=[ymin,ymax],xrange=[0,16],xtit='Detector', $
           ytit='Transimittance',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],color=lcol[0],thick=3
      
      XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   ENDIF

;*****************************************************************************
;  Calculated transmittance of ND filter vs wavelength
;*****************************************************************************

   title1='Calculated Transimittance of ND filter versus Wavelength'
   title2=band_name[j]

   !p.multi=[0,1,1]
   !p.position=0

   xmax=max(lambda_uaid[j,8,0:ncol[j,1]-1,1],min=xmin)

   plot,lambda_uaid[j,8,0:ncol[j,1]-1,1],trans_col_corr_2[j,0:ncol[j,1]-1,8,0],yrange=[trans_avg_corr[j,8,0]*0.5,trans_avg_corr[j,8,0]*1.5],xrange=[xmin,xmax],xtit='Detector', $
        ytit='Transimittance',title=title2,charsize=1.5,charthick=5,/nodata

   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],trans_col_corr_2[j,0:ncol[j,1]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],trans_col_corr_2[j,0:ncol[j,1]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot dn vs wavelength (FP-16 IB)
;*****************************************************************************
   
   title1='dn vs Wavelength (FP-16 IB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,1]-1,1],min=xmin)
   ymax=max(dn_col[j,0:ncol[j,1]-1,*,*,1],min=ymin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,1]-1,1],dn_col[j,0:ncol[j,1]-1,8,0,1],yrange=[ymin,ymax],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='dn (uncorrected)',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],dn_col[j,0:ncol[j,1]-1,id,0,1],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],dn_col[j,0:ncol[j,1]-1,id,0,1],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot RSR vs wavelength, log plot (FP-16 IB)
;*****************************************************************************

   title1='RSR vs Wavelength (FP-16 IB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,1]-1,1],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,1]-1,1],RSR_col_corr[j,0:ncol[j,1]-1,8,0,1],yrange=[0.00001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],RSR_col_corr[j,0:ncol[j,1]-1,id,0,1],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],RSR_col_corr[j,0:ncol[j,1]-1,id,0,1],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot std and snr vs wavelength (FP-16 IB)
;*****************************************************************************

   title1='Standard deviation vs Wavelength (FP-16 IB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,1]-1,1],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   ymax=max(std_col[j,0:ncol[j,1]-1,0:detectors[j]-1,0,1],min=ymin)
   plot,lambda_uaid[j,0,0:ncol[j,1]-1,1],std_col[j,0:ncol[j,1]-1,0,0,1],yrange=[ymin,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='Standard deviation',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],std_col[j,0:ncol[j,1]-1,id,0,1],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],std_col[j,0:ncol[j,1]-1,id,0,1],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='SNR vs Wavelength (FP-16 IB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,1]-1,1],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   ymax=max(snr_col[j,0:ncol[j,1]-1,0:detectors[j]-1,0,1],min=ymin)
   plot,lambda_uaid[j,0,0:ncol[j,1]-1,1],snr_col[j,0:ncol[j,1]-1,0,0,1],yrange=[ymin,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],snr_col[j,0:ncol[j,1]-1,id,0,1],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],snr_col[j,0:ncol[j,1]-1,id,0,1],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
    
   skip_fp16_ib:

;*****************************************************************************
;  Plot dn vs wavelength (FP-16 OOB)
;*****************************************************************************

   title1='dn vs Wavelength (FP-16 OOB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,0]-1,0],min=xmin)
   ymax=max(dn_col[j,0:ncol[j,0]-1,*,*,0],min=ymin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],dn_col[j,0:ncol[j,0]-1,8,0,0],yrange=[ymin,ymax],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='dn (uncorrected)',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],dn_col[j,0:ncol[j,0]-1,id,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],dn_col[j,0:ncol[j,0]-1,id,0,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot RSR vs wavelength, log plot (FP-16 OOB)
;*****************************************************************************

   title1='RSR vs Wavelength (FP-16 OOB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,0]-1,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,8,0,0],yrange=[0.0000001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,id,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,id,0,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot std and snr vs wavelength (FP-16 OOB)
;*****************************************************************************

   title1='Standard deviation vs Wavelength (FP-16 OOB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,0]-1,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   ymax=max(std_col[j,0:ncol[j,0]-1,0:detectors[j]-1,0,0],min=ymin)
   plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],std_col[j,0:ncol[j,0]-1,0,0,0],yrange=[ymin,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='Standard deviation',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],std_col[j,0:ncol[j,0]-1,id,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],std_col[j,0:ncol[j,0]-1,id,0,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='Standard deviation vs Wavelength (FP-16 OOB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,0]-1,0],min=xmin)
   
   ymax=max(std_col[j,0:ncol[j,0]-1,0:detectors[j]-1,0,0])
   plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],std_col[j,0:ncol[j,0]-1,0,0,0],yrange=[0.001,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='Standard deviation',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],std_col[j,0:ncol[j,0]-1,id,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],std_col[j,0:ncol[j,0]-1,id,0,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='SNR vs Wavelength (FP-16 OOB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,0]-1,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   ymax=max(snr_col[j,0:ncol[j,0]-1,0:detectors[j]-1,0,0],min=ymin)
   plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],snr_col[j,0:ncol[j,0]-1,0,0,0],yrange=[ymin,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],snr_col[j,0:ncol[j,0]-1,id,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],snr_col[j,0:ncol[j,0]-1,id,0,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='SNR vs Wavelength (FP-16 OOB)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,0]-1,0],min=xmin)
   
   ymax=max(snr_col[j,0:ncol[j,0]-1,0:detectors[j]-1,0,0])
   plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],snr_col[j,0:ncol[j,0]-1,0,0,0],yrange=[0.001,ymax],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],snr_col[j,0:ncol[j,0]-1,id,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],snr_col[j,0:ncol[j,0]-1,id,0,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
;*****************************************************************************
;  Plot RSR vs wavelength, log plot (stitched FP-16)
;*****************************************************************************

   IF uaidname2[j] EQ 'A' THEN goto,skip_fp16_ib_stitch
   
   title1='RSR vs Wavelength (stitched FP-16)'
   title2=band_name[j]
   
   xmax=max(lambda_uaid[j,0,0:ncol[j,0]-1,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,8,0],yrange=[0.0000001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   skip_fp16_ib_stitch:

;*****************************************************************************
;  Plot RSR vs wavelength, log plot (stitched FP-16) Plot FP-15 on top
;*****************************************************************************
   
   title1='RSR vs Wavelength (stitched FP-16 with oplot FP-15) SNR=3'
   title2=band_name[j]
   
   !p.multi=[0,1,1]
   !p.position=0
   
   IF uaidname2[j] NE 'A' THEN plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,8,0],yrange=[0.00001,1], $
                                    xrange=[mean(center_lambda[j,0:detectors[j]-1,0,2])-2*mean(bandwidth_corr[j,0:detectors[j]-1,0,2]), $
                                            mean(center_lambda[j,0:detectors[j]-1,0,2])+2*mean(bandwidth_corr[j,0:detectors[j]-1,0,2])],xtit='Wavelength (nm)', $
                                    ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   IF uaidname2[j] EQ 'A' THEN plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,8,0,0],yrange=[0.00001,1], $
                                    xrange=[mean(center_lambda[j,0:detectors[j]-1,0,2])-2*mean(bandwidth_corr[j,0:detectors[j]-1,0,2]), $
                                            mean(center_lambda[j,0:detectors[j]-1,0,2])+2*mean(bandwidth_corr[j,0:detectors[j]-1,0,2])],xtit='Wavelength (nm)', $
                                    ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      IF uaidname2[j] NE 'A' THEN oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      IF uaidname2[j] NE 'A' THEN oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,id,0],color=lcol[id],thick=3

      ; IF uaidname2[j] NE 'A' THEN oplot,lambda_stitch[j,id,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j]-1,id,0],symsize=1,color=lcol[id], $
      ; psym=symbol[id],thick=3
      ; IF uaidname2[j] NE 'A' THEN oplot,lambda_stitch[j,id,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j]-1,id,0],color=lcol[id],thick=3

      IF uaidname2[j] EQ 'A' THEN oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,id,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      IF uaidname2[j] EQ 'A' THEN oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,id,0,0],color=lcol[id],thick=3
      
      xa=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,2] GE lambda_limit_stitch[j,id,0,0,2] AND lambda_uaid[j,id,0:ncol[j,0]-1,2] LE lambda_limit_stitch[j,id,0,1,2])
      
      oplot,lambda_uaid[j,id,xa,2],RSR_col_corr[j,xa,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,xa,2],RSR_col_corr[j,xa,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='RSR vs Wavelength (stitched FP-16 with oplot FP-15) 0.1% Stitch'
   title2=band_name[j]
   
   !p.multi=[0,1,1]
   !p.position=0
   
   IF uaidname2[j] NE 'A' THEN plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,8,0],yrange=[0.00001,1], $
                                    xrange=[mean(center_lambda[j,0:detectors[j]-1,0,2])-2*mean(bandwidth_corr[j,0:detectors[j]-1,0,2]), $
                                            mean(center_lambda[j,0:detectors[j]-1,0,2])+2*mean(bandwidth_corr[j,0:detectors[j]-1,0,2])],xtit='Wavelength (nm)', $
                                    ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   IF uaidname2[j] EQ 'A' THEN plot,lambda_uaid[j,0,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,8,0,0],yrange=[0.00001,1], $
                                    xrange=[mean(center_lambda[j,0:detectors[j]-1,0,2])-2*mean(bandwidth_corr[j,0:detectors[j]-1,0,2]), $
                                            mean(center_lambda[j,0:detectors[j]-1,0,2])+2*mean(bandwidth_corr[j,0:detectors[j]-1,0,2])],xtit='Wavelength (nm)', $
                                    ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog

   FOR id=0, detectors[j]-1 DO BEGIN
      IF uaidname2[j] NE 'A' THEN oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      IF uaidname2[j] NE 'A' THEN oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_stitch_ND[j,0:ncol[j,0]-1,id,0],color=lcol[id],thick=3

      ; IF uaidname2[j] NE 'A' THEN oplot,lambda_stitch[j,id,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j]-1,id,0],symsize=1,color=lcol[id], $
      ; psym=symbol[id],thick=3
      ; IF uaidname2[j] NE 'A' THEN oplot,lambda_stitch[j,id,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j]-1,id,0],color=lcol[id],thick=3

      IF uaidname2[j] EQ 'A' THEN oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,id,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      IF uaidname2[j] EQ 'A' THEN oplot,lambda_uaid[j,id,0:ncol[j,0]-1,0],RSR_col_corr[j,0:ncol[j,0]-1,id,0,0],color=lcol[id],thick=3
      
      xa=WHERE(lambda_uaid[j,id,0:ncol[j,0]-1,2] GE lambda_limit_stitch_2[j,id,0,0,2] AND lambda_uaid[j,id,0:ncol[j,0]-1,2] LE lambda_limit_stitch_2[j,id,0,1,2])
      
      oplot,lambda_uaid[j,id,xa,2],RSR_col_corr[j,xa,id,0,2],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_uaid[j,id,xa,2],RSR_col_corr[j,xa,id,0,2],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot RSR vs wavelength, log plot (stitched FP-15/16)
;*****************************************************************************

   title1='RSR vs Wavelength (stitched FP-15/16)'
   title2=band_name[j]
   
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],RSR_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[0.0000001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='RSR vs Wavelength (stitched FP-15/16 with cutoff)'
   title2=band_name[j]
   
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],RSR_col_stitch_cut[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[0.0000001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog

   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch_cut[j,0:ncol_stitch[j,id,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch_cut[j,0:ncol_stitch[j,id,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot SNR vs wavelength, log plot (stitched FP-15/16)
;*****************************************************************************

   title1='SNR vs Wavelength (stitched FP-15/16)'
   title2=band_name[j]
   
   ymax=max(snr_col_stitch[j,0:ncol_stitch[j,8,0]-1,0:detectors[j]-1,0],min=ymin)
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],snr_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[0.0001,ymax],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],snr_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],snr_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   ymax=max(snr_col_stitch[j,0:ncol_stitch[j,8,0]-1,0:detectors[j]-1,0],min=ymin)
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],snr_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[ymin,3.],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],snr_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],snr_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title2=band_name[j]+' DET=9'
   
   ymax=max(snr_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],min=ymin)
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],snr_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[0.0001,ymax],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   oplot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],snr_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],symsize=1,color=lcol[8],psym=symbol[8],thick=3
   oplot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],snr_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],color=lcol[8],thick=3
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot SNR vs wavelength, (stitched FP-15/16)
;*****************************************************************************

   title1='SNR vs Wavelength (stitched FP-15/16)'
   title2=band_name[j]
   
   ymax=max(snr_col_stitch_avg[j,0:ncol_stitch[j,8,0]-1,0:detectors[j]-1,0],min=ymin)
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],snr_col_stitch_avg[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[0.0001,ymax],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],snr_col_stitch_avg[j,0:ncol_stitch[j,id,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],snr_col_stitch_avg[j,0:ncol_stitch[j,id,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   oplot,[0.,40000.],[snr_cutoff[j],snr_cutoff[j]],symsize=1,color=lcol[1],linestyle=2,thick=3
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
   title1='SNR vs Wavelength (stitched FP-15/16)'
   title2=band_name[j]
   
   ymax=max(snr_col_stitch_avg[j,0:ncol_stitch[j,8,0]-1,0:detectors[j]-1,0],min=ymin)
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],snr_col_stitch_avg[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[0.0001,1.0],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='SNR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],snr_col_stitch_avg[j,0:ncol_stitch[j,id,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],snr_col_stitch_avg[j,0:ncol_stitch[j,id,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   oplot,[0.,40000.],[snr_cutoff[j],snr_cutoff[j]],symsize=1,color=lcol[1],linestyle=2,thick=3
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol
   
;*****************************************************************************
;  Plot atmospheric absorption
;*****************************************************************************

   title1='Atmospheric Absorption'
   
   atm=WHERE(1000*absorption[0,*] GT min(lambda_uaid[j,0,0:ncol[j,0]-1,0]) AND 1000*absorption[0,*] LT max(lambda_uaid[j,0,0:ncol[j,0]-1,0]))
   
   plot,absorption[0,atm],absorption[1,atm],yrange=[0,1],xrange=[min(absorption[0,atm]),max(absorption[0,atm])],xtit='Wavelength (um)',ytit='Absorption',charsize=1.5,charthick=5,/nodata
   
   oplot,absorption[0,atm],absorption[1,atm],color=lcol[2]
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5

;*****************************************************************************
;  Plot atmospheric absorption over RSR
;*****************************************************************************

   title1='RSR vs Wavelength (stitched FP-15/16) with Atmoshperic Absorption'
   title2=band_name[j]
   
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],RSR_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[0.0000001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   oplot,absorption[0,atm]*1000,absorption[1,atm],color=lcol[2]
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot uncorrected RSR, RSO, and corrected RSR in three panels for I5.
;*****************************************************************************
;skip_plot:
   IF band_name[j] EQ 'I5' THEN BEGIN
      !p.multi=[0,3,1]
      !p.position=0
      
      title1='I5 DET 16 RSR'
      title2='RSR uncorrected'
      
      xmax=max(lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,min=xmin)
      
      plot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,RSR_col[j,0:ncol[j,2]-1,15,0,2],yrange=[0.0,1.0],xrange=[xmin,xmax], $
           xtit='Wavelength (um)',ytit='RSR',title=title2,charsize=1.7,charthick=5,xstyle=1,xticks=3,/nodata
      
      oplot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,RSR_col[j,0:ncol[j,2]-1,15,0,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,RSR_col[j,0:ncol[j,2]-1,15,0,2],color=lcol[1],thick=3
      
      title2='RSO'
      
      ymax=max(rso_spma[j,0:ncol[j,2]-1,2],min=ymin)
      
      plot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,rso_spma[j,0:ncol[j,2]-1,2],yrange=[ymin,ymax],xrange=[xmin,xmax], $
           xtit='Wavelength (um)',ytit='RSO',title=title2,charsize=1.7,charthick=5,xstyle=1,xticks=3,/nodata
      
      oplot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,rso_spma[j,0:ncol[j,2]-1,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,rso_spma[j,0:ncol[j,2]-1,2],color=lcol[1],thick=3
      
      title2='RSR corrected'
      
      plot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,RSR_col_corr[j,0:ncol[j,2]-1,15,0,2],yrange=[0.0,1.0],xrange=[xmin,xmax], $
           xtit='Wavelength (um)',ytit='RSR',title=title2,charsize=1.7,charthick=5,xstyle=1,xticks=3,/nodata
      
      oplot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,RSR_col_corr[j,0:ncol[j,2]-1,15,0,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,lambda_uaid[j,15,0:ncol[j,2]-1,2]/1000.,RSR_col_corr[j,0:ncol[j,2]-1,15,0,2],color=lcol[1],thick=3
      
      XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   ENDIF

;*****************************************************************************
;  Overplot FP-15 and FP-16 for detector 9
;*****************************************************************************

   !p.multi=[0,1,1]
   !p.position=0
   
   title1='FP-15 and FP-16 RSR'
   title2=band_name[j]+' DET 9 RSR'
   
   xmax=max(lambda_uaid[j,9,0:ncol[j,2]-1,2]/1000.,min=xmin)
   
   plot,lambda_uaid[j,9,0:ncol[j,2]-1,2]/1000.,RSR_col_corr[j,0:ncol[j,2]-1,9,0,2],yrange=[0.0001,1.0],xrange=[xmin,xmax], $
        xtit='Wavelength (um)',ytit='RSR',title=title2,charsize=1.7,charthick=5,xstyle=1,xticks=5,/nodata,/ylog
   
   oplot,lambda_uaid[j,9,0:ncol[j,0]-1,0]/1000.,RSR_col_stitch_ND[j,0:ncol[j,0]-1,9,0],symsize=1,color=lcol[0],psym=symbol[0],thick=3
   oplot,lambda_uaid[j,9,0:ncol[j,0]-1,0]/1000.,RSR_col_stitch_ND[j,0:ncol[j,0]-1,9,0],color=lcol[0],thick=3
   oplot,lambda_uaid[j,9,0:ncol[j,2]-1,2]/1000.,RSR_col_corr[j,0:ncol[j,2]-1,9,0,2],symsize=1,color=lcol[1],psym=symbol[1],thick=3
   oplot,lambda_uaid[j,9,0:ncol[j,2]-1,2]/1000.,RSR_col_corr[j,0:ncol[j,2]-1,9,0,2],color=lcol[1],thick=3
   oplot,lambda_stitch[j,0:ncol_stitch[j,9,0]-1,9,0]/1000.,RSR_col_stitch[j,0:ncol_stitch[j,9,0]-1,9,0],symsize=1,color=lcol[2],psym=symbol[2],thick=3
   oplot,lambda_stitch[j,0:ncol_stitch[j,9,0]-1,9,0]/1000.,RSR_col_stitch[j,0:ncol_stitch[j,9,0]-1,9,0],color=lcol[2],thick=3
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   
   device,/close

;*****************************************************************************
;  Output to files.
;*****************************************************************************

   outfile2=outpath+'RSR.'+band_name[j]+'.corr.txt'
   outfile3=outpath+'RSR.'+band_name[j]+'.raw.txt'
   outfile4=outpath+'RSR.'+band_name[j]+'.corr.raw.txt'
   outfile5=outpath+'RSR.'+band_name[j]+'.comparison.txt'
   outfile6=outpath+'RSR.'+band_name[j]+'.OOB.txt'
   outfile7=outpath+'RSR.'+band_name[j]+'.v3.txt'
   
   fmt_1='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
   fmt_2='(a5, 5x, a3, 6x, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
   fmt_3='(a5, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
   fmt_4='(a5, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
   fmt_5='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d)'
   fmt_6='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d, 5x, d)'
   
   openw,lun,outfile2,/get_lun
   printf,lun,'Spetral Characteristics (RSO correction), Sender='+band_name[j]
   printf,lun,'Band    Detector   Subsample    Center    Centriod    Bandwidth    Lower 1% limit    Upper 1% limit   Transimission of ND   STDDEV of Transimission of ND   IOOB   IOOB(with zeros)   Characterization Uncertainty (MAX instead of AVG) (center and bandwidth)'
   FOR is=0, subsets[j]-1 DO BEGIN
      FOR id=0, detectors[j]-1 DO BEGIN
         printf,lun,format=fmt_1,band_name[j],id+1,is,center_lambda_corr[j,id,is,2],lambda_centroid_corr[j,id,is,2],bandwidth_corr[j,id,is,2], $
                lambda_limit_corr[j,id,is,0,2],lambda_limit_corr[j,id,is,1,2],trans_avg_corr[j,id,is],trans_std_corr[j,id,is],IOOB[j,id,is]*100,IOOB2[j,id,is]*100, $
                char_uncert_center_corr[j,id,is,2],char_uncert_bandwidth_corr[j,id,is,2]
      ENDFOR
   ENDFOR
   IF band_type[j] EQ 0 THEN printf,lun,format=fmt_2,band_name[j],'AVG',mean(center_lambda_corr[j,0:detectors[j]-1,0,2]), $
                                    mean(lambda_centroid_corr[j,0:detectors[j]-1,0,2]),mean(bandwidth_corr[j,0:detectors[j]-1,0,2]),mean(lambda_limit_corr[j,0:detectors[j]-1,0,0,2]), $
                                    mean(lambda_limit_corr[j,0:detectors[j]-1,0,1,2]),trans_corr[j],std_trans_corr[j],mean(IOOB[j,0:detectors[j]-1,0]*100), $
                                    mean(IOOB2[j,0:detectors[j]-1,0]*100),max(char_uncert_center_corr[j,0:detectors[j]-1,0,2]),max(char_uncert_bandwidth_corr[j,0:detectors[j]-1,0,2])
   IF band_type[j] EQ 1 THEN printf,lun,format=fmt_2,band_name[j],'AVG',mean(center_lambda_corr[j,0:detectors[j]-1,0:1,2]), $
                                    mean(lambda_centroid_corr[j,0:detectors[j]-1,0:1,2]),mean(bandwidth_corr[j,0:detectors[j]-1,0:1,2]),mean(lambda_limit_corr[j,0:detectors[j]-1,0:1,0,2]), $
                                    mean(lambda_limit_corr[j,0:detectors[j]-1,0:1,1,2]),trans_corr[j],std_trans_corr[j],mean(IOOB[j,0:detectors[j]-1,0:1]*100), $
                                    mean(IOOB2[j,0:detectors[j]-1,0:1]*100),max(char_uncert_center_corr[j,0:detectors[j]-1,0:1,2]),max(char_uncert_bandwidth_corr[j,0:detectors[j]-1,0:1,2])
   free_lun,lun

   openw,lun,outfile3,/get_lun
;    printf,lun,'IB RSR (no RSO correction), Sender='+band_name[j]
   printf,lun,'IB dn (no RSO correction), Sender='+band_name[j]
   printf,lun,'Band    wavelength    Illuminated detectors'
   FOR col=0, ncol[j,2]-1 DO BEGIN
    ;    IF band_type[j] EQ 0 THEN printf,lun,format=fmt_3,band_name[j],lambda_uaid[j,0,col,2],RSR_col[j,col,0,0,2],RSR_col[j,col,1,0,2], $
    ;      RSR_col[j,col,2,0,2],RSR_col[j,col,3,0,2],RSR_col[j,col,4,0,2],RSR_col[j,col,5,0,2],RSR_col[j,col,6,0,2],RSR_col[j,col,7,0,2],RSR_col[j,col,8,0,2], $
    ;      RSR_col[j,col,9,0,2],RSR_col[j,col,10,0,2],RSR_col[j,col,11,0,2],RSR_col[j,col,12,0,2],RSR_col[j,col,13,0,2],RSR_col[j,col,14,0,2],RSR_col[j,col,15,0,2]
    ;    IF band_type[j] EQ 1 THEN printf,lun,format=fmt_4,band_name[j],lambda_uaid[j,0,col,2],RSR_col[j,col,0,0,2],RSR_col[j,col,1,0,2], $
    ;      RSR_col[j,col,2,0,2],RSR_col[j,col,3,0,2],RSR_col[j,col,4,0,2],RSR_col[j,col,5,0,2],RSR_col[j,col,6,0,2],RSR_col[j,col,7,0,2],RSR_col[j,col,8,0,2], $
    ;      RSR_col[j,col,9,0,2],RSR_col[j,col,10,0,2],RSR_col[j,col,11,0,2],RSR_col[j,col,12,0,2],RSR_col[j,col,13,0,2],RSR_col[j,col,14,0,2],RSR_col[j,col,15,0,2], $
    ;      RSR_col[j,col,16,0,2],RSR_col[j,col,17,0,2],RSR_col[j,col,18,0,2],RSR_col[j,col,19,0,2],RSR_col[j,col,20,0,2],RSR_col[j,col,21,0,2],RSR_col[j,col,22,0,2], $
    ;      RSR_col[j,col,23,0,2],RSR_col[j,col,24,0,2],RSR_col[j,col,25,0,2],RSR_col[j,col,26,0,2],RSR_col[j,col,27,0,2],RSR_col[j,col,28,0,2],RSR_col[j,col,29,0,2], $
    ;      RSR_col[j,col,30,0,2],RSR_col[j,col,31,0,2]
      IF band_type[j] EQ 0 THEN printf,lun,format=fmt_3,band_name[j],lambda_uaid[j,0,col,2],dn_col[j,col,0,0,2],dn_col[j,col,1,0,2], $
                                       dn_col[j,col,2,0,2],dn_col[j,col,3,0,2],dn_col[j,col,4,0,2],dn_col[j,col,5,0,2],dn_col[j,col,6,0,2],dn_col[j,col,7,0,2],dn_col[j,col,8,0,2], $
                                       dn_col[j,col,9,0,2],dn_col[j,col,10,0,2],dn_col[j,col,11,0,2],dn_col[j,col,12,0,2],dn_col[j,col,13,0,2],dn_col[j,col,14,0,2],dn_col[j,col,15,0,2]
      IF band_type[j] EQ 1 THEN printf,lun,format=fmt_4,band_name[j],lambda_uaid[j,0,col,2],dn_col[j,col,0,0,2],dn_col[j,col,1,0,2], $
                                       dn_col[j,col,2,0,2],dn_col[j,col,3,0,2],dn_col[j,col,4,0,2],dn_col[j,col,5,0,2],dn_col[j,col,6,0,2],dn_col[j,col,7,0,2],dn_col[j,col,8,0,2], $
                                       dn_col[j,col,9,0,2],dn_col[j,col,10,0,2],dn_col[j,col,11,0,2],dn_col[j,col,12,0,2],dn_col[j,col,13,0,2],dn_col[j,col,14,0,2],dn_col[j,col,15,0,2], $
                                       dn_col[j,col,16,0,2],dn_col[j,col,17,0,2],dn_col[j,col,18,0,2],dn_col[j,col,19,0,2],dn_col[j,col,20,0,2],dn_col[j,col,21,0,2],dn_col[j,col,22,0,2], $
                                       dn_col[j,col,23,0,2],dn_col[j,col,24,0,2],dn_col[j,col,25,0,2],dn_col[j,col,26,0,2],dn_col[j,col,27,0,2],dn_col[j,col,28,0,2],dn_col[j,col,29,0,2], $
                                       dn_col[j,col,30,0,2],dn_col[j,col,31,0,2]
    ;    IF band_type[j] EQ 1 THEN printf,lun,format=fmt_4,band_name[j],lambda_uaid[j,0,col,2],dn_col[j,col,0,1,2],dn_col[j,col,1,1,2], $
    ;      dn_col[j,col,2,1,2],dn_col[j,col,3,1,2],dn_col[j,col,4,1,2],dn_col[j,col,5,1,2],dn_col[j,col,6,1,2],dn_col[j,col,7,1,2],dn_col[j,col,8,1,2], $
    ;      dn_col[j,col,9,1,2],dn_col[j,col,10,1,2],dn_col[j,col,11,1,2],dn_col[j,col,12,1,2],dn_col[j,col,13,1,2],dn_col[j,col,14,1,2],dn_col[j,col,15,1,2], $
    ;      dn_col[j,col,16,1,2],dn_col[j,col,17,1,2],dn_col[j,col,18,1,2],dn_col[j,col,19,1,2],dn_col[j,col,20,1,2],dn_col[j,col,21,1,2],dn_col[j,col,22,1,2], $
    ;      dn_col[j,col,23,1,2],dn_col[j,col,24,1,2],dn_col[j,col,25,1,2],dn_col[j,col,26,1,2],dn_col[j,col,27,1,2],dn_col[j,col,28,1,2],dn_col[j,col,29,1,2], $
    ;      dn_col[j,col,30,1,2],dn_col[j,col,31,1,2]
   ENDFOR
   free_lun,lun
   
   openw,lun,outfile4,/get_lun
   printf,lun,'IB RSR (RSO correction), Sender='+band_name[j]
   printf,lun,'Band    wavelength    Illuminated detectors'
   FOR col=0, ncol[j,2]-1 DO BEGIN
      IF band_type[j] EQ 0 THEN printf,lun,format=fmt_3,band_name[j],lambda_uaid[j,0,col,2],RSR_col_corr[j,col,0,0,2], $
                                       RSR_col_corr[j,col,1,0,2],RSR_col_corr[j,col,2,0,2],RSR_col_corr[j,col,3,0,2],RSR_col_corr[j,col,4,0,2],RSR_col_corr[j,col,5,0,2], $
                                       RSR_col_corr[j,col,6,0,2],RSR_col_corr[j,col,7,0,2],RSR_col_corr[j,col,8,0,2],RSR_col_corr[j,col,9,0,2],RSR_col_corr[j,col,10,0,2], $
                                       RSR_col_corr[j,col,11,0,2],RSR_col_corr[j,col,12,0,2],RSR_col_corr[j,col,13,0,2],RSR_col_corr[j,col,14,0,2],RSR_col_corr[j,col,15,0,2]
      IF band_type[j] EQ 1 THEN printf,lun,format=fmt_4,band_name[j],lambda_uaid[j,0,col,2],RSR_col_corr[j,col,0,0,2], $
                                       RSR_col_corr[j,col,1,0,2],RSR_col_corr[j,col,2,0,2],RSR_col_corr[j,col,3,0,2],RSR_col_corr[j,col,4,0,2],RSR_col_corr[j,col,5,0,2], $
                                       RSR_col_corr[j,col,6,0,2],RSR_col_corr[j,col,7,0,2],RSR_col_corr[j,col,8,0,2],RSR_col_corr[j,col,9,0,2],RSR_col_corr[j,col,10,0,2], $
                                       RSR_col_corr[j,col,11,0,2],RSR_col_corr[j,col,12,0,2],RSR_col_corr[j,col,13,0,2],RSR_col_corr[j,col,14,0,2],RSR_col_corr[j,col,15,0,2], $
                                       RSR_col_corr[j,col,16,0,2],RSR_col_corr[j,col,17,0,2],RSR_col_corr[j,col,18,0,2],RSR_col_corr[j,col,19,0,2],RSR_col_corr[j,col,20,0,2], $
                                       RSR_col_corr[j,col,21,0,2],RSR_col_corr[j,col,22,0,2],RSR_col_corr[j,col,23,0,2],RSR_col_corr[j,col,24,0,2],RSR_col_corr[j,col,25,0,2], $
                                       RSR_col_corr[j,col,26,0,2],RSR_col_corr[j,col,27,0,2],RSR_col_corr[j,col,28,0,2],RSR_col_corr[j,col,29,0,2],RSR_col_corr[j,col,30,0,2], $
                                       RSR_col_corr[j,col,31,0,2]
    ;    IF band_type[j] EQ 1 THEN printf,lun,format=fmt_4,band_name[j],lambda_uaid[j,0,col,2],RSR_col_corr[j,col,0,1,2], $
    ;      RSR_col_corr[j,col,1,1,2],RSR_col_corr[j,col,2,1,2],RSR_col_corr[j,col,3,1,2],RSR_col_corr[j,col,4,1,2],RSR_col_corr[j,col,5,1,2],RSR_col_corr[j,col,6,1,2], $
    ;      RSR_col_corr[j,col,7,1,2],RSR_col_corr[j,col,8,1,2],RSR_col_corr[j,col,9,1,2],RSR_col_corr[j,col,10,1,2],RSR_col_corr[j,col,11,1,2],RSR_col_corr[j,col,12,1,2], $
    ;      RSR_col_corr[j,col,13,1,2],RSR_col_corr[j,col,14,1,2],RSR_col_corr[j,col,15,1,2],RSR_col_corr[j,col,16,1,2],RSR_col_corr[j,col,17,1,2],RSR_col_corr[j,col,18,1,2], $
    ;      RSR_col_corr[j,col,19,1,2],RSR_col_corr[j,col,20,1,2],RSR_col_corr[j,col,21,1,2],RSR_col_corr[j,col,22,1,2],RSR_col_corr[j,col,23,1,2],RSR_col_corr[j,col,24,1,2], $
    ;      RSR_col_corr[j,col,25,1,2],RSR_col_corr[j,col,26,1,2],RSR_col_corr[j,col,27,1,2],RSR_col_corr[j,col,28,1,2],RSR_col_corr[j,col,29,1,2],RSR_col_corr[j,col,30,1,2], $
    ;      RSR_col_corr[j,col,31,1,2]
   ENDFOR
   free_lun,lun
   
   openw,lun,outfile5,/get_lun
   printf,lun,'0.1% limit vs SNR=3 for '+band_name[j]
   printf,lun,'Band    Detector   Subsample    Lower SNR=3    Lower 0.1% limit    Upper SNR=3    Upper 1% limit'
   FOR is=0, subsets[j]-1 DO BEGIN
      FOR id=0, detectors[j]-1 DO BEGIN
         printf,lun,format=fmt_5,band_name[j],id+1,is,lambda_limit_stitch[j,id,is,0,2],lambda_limit_stitch_2[j,id,is,0,2],lambda_limit_stitch[j,id,is,1,2],lambda_limit_stitch_2[j,id,is,1,2]
      ENDFOR
   ENDFOR
   free_lun,lun
   
   openw,lun,outfile6,/get_lun
   printf,lun,'RSR (stitched FP-15/16 with RSO correction), Sender='+band_name[j]
   printf,lun,'Band    detector    subsample    wavelength     RSR    RSR(with cutoff)    SNR    Average SNR'
;    printf,lun,'dn (FP-16 with and without RSO correction), Sender='+band_name[j]
;    printf,lun,'Band    detector    subsample    wavelength    dn   dn_RSO'
   FOR is=0, subsets[j]-1 DO BEGIN
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR col=0, ncol_stitch[j,id,0]-1 DO BEGIN
            printf,lun,format=fmt_6,band_name[j],id+1,is,lambda_stitch[j,col,id,is],RSR_col_stitch[j,col,id,is],RSR_col_stitch_cut[j,col,id,is],snr_col_stitch[j,col,id,is], $
                   snr_col_stitch_avg[j,col,id,is]
         ENDFOR
        ;    FOR col=0, ncol[j,0]-1 DO BEGIN
        ;        printf,lun,format=fmt_6,band_name[j],id,is,lambda_uaid[j,0,col,0],dn_col[j,col,id,is,0],dn_col[j,col,id,is,0]/std_col[j,col,id,is,0]
        ;    ENDFOR
      ENDFOR
   ENDFOR
   free_lun,lun
   
   openw,lun,outfile7,/get_lun
   printf,lun,'RSR, Sender='+band_name[j]
   printf,lun,'Band    detector    subsample    wavelength     RSR'
   FOR is=0, subsets[j]-1 DO BEGIN
      FOR id=0, detectors[j]-1 DO BEGIN
         FOR col=0, ncol_stitch[j,id,0]-1 DO BEGIN
            printf,lun,format=fmt_6,band_name[j],id+1,is,lambda_stitch[j,col,id,is],RSR_col_stitch[j,col,id,is]
         ENDFOR
      ENDFOR
   ENDFOR
   free_lun,lun
   
   close,/all
   
;*****************************************************************************
;  Output to sav file
;*****************************************************************************

   save,/compress,dn_col,dn_col_corr,RSR_col,RSR_col_corr,RSR_col_stitch,RSR_col_stitch_cut,std_col,snr_col,snr_col_stitch,snr_col_stitch_avg,lambda_limit,lambda_limit_corr, $
        bandwidth,bandwidth_corr,lambda_centroid,lambda_centroid_corr,center_lambda,center_lambda_corr,lambda_stitch,rso_spma,lambda_uaid, $
        trans_corr,trans_std_corr,trans_avg_corr,ncol_stitch,IOOB,filename=outpath+'rsr.sav'
ENDFOR

;*****************************************************************************
;  Make plots of the ND transmission.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+'RSR.trans.ps'
device,/landscape,filename=outfname,/color

!p.multi=[0,1,1]
!p.position=0

FOR j=0, n_elements(uaidname1)-1 DO BEGIN
   IF uaidname1[j] EQ 'A' THEN continue

;*****************************************************************************
;  Calculated transmittance of ND filter
;*****************************************************************************

   IF uaidname2[j] EQ 'A' THEN continue
   
   title1='Calculated Transimittance of ND filter versus Detector'
   title2=band_name[j]
   
   !p.multi=[0,1,1]
   !p.position=0
   
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(trans_avg_corr[j,0:detectors[j]-1,0],min=ymin)
      
      plot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],yrange=[ymin,ymax],xrange=[0,32],xtit='Detector', $
           ytit='Transimittance',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,1],symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,1],color=lcol[1],thick=3
      
      XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
      set_legend,1,SS_3,lcol,symbol
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(trans_avg_corr[j,0:detectors[j]-1,0],min=ymin)
      
      plot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],yrange=[ymin,ymax],xrange=[0,16],xtit='Detector', $
           ytit='Transimittance',title=title2,charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,trans_avg_corr[j,0:detectors[j]-1,0],color=lcol[0],thick=3
      
      XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   ENDIF

;*****************************************************************************
;  Calculated transmittance of ND filter vs wavelength
;*****************************************************************************

   title1='Calculated Transimittance of ND filter versus Wavelength'
   title2=band_name[j]

   !p.multi=[0,1,1]
   !p.position=0

   x=where(lambda_trans[j,*,8,0] NE 0)
   ymax=max(trans_wave[j,x,*,0],min=ymin)
   xmax=max(lambda_uaid[j,8,0:ncol[j,1]-1,1],min=xmin)

   plot,lambda_uaid[j,8,0:ncol[j,1]-1,1],trans_col_corr_2[j,0:ncol[j,1]-1,8,0],yrange=[ymin,ymax],xrange=[xmin,xmax],xtit='Detector', $
        ytit='Transimittance',title=title2,charsize=1.5,charthick=5,/nodata

   FOR id=0, detectors[j]-1 DO BEGIN
      x=where(lambda_trans[j,*,id,0] NE 0)

      oplot,lambda_trans[j,x,id,0],trans_wave[j,x,id,0],symsize=2.,color=lcol[id],psym=symbol[id],thick=3

      oplot,lambda_uaid[j,id,0:ncol[j,1]-1,1],trans_col_corr_2[j,0:ncol[j,1]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot difference in RSR vs wavelength, average and detector
;  dependent ND transmittance.
;*****************************************************************************

   title1='RSR Diff vs Wavelength (stitched FP-15/16)'
   title2=band_name[j]
   
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   ymax=max(RSR_col_stitch[j,0:ncol_stitch[j,8,0]-1,0:detectors[j]-1,0]-RSR_col_stitch_2[j,0:ncol_stitch[j,8,0]-1,0:detectors[j]-1,0],min=ymin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,8,0]-1,8,0],RSR_col_stitch[j,0:ncol_stitch[j,8,0]-1,8,0]-RSR_col_stitch_2[j,0:ncol_stitch[j,8,0]-1,8,0],yrange=[ymin,ymax], $
        xrange=[xmin,xmax],xtit='Wavelength (nm)',ytit='RSR Diff',title=title2,charsize=1.5,charthick=5,/nodata
   
   FOR id=0, detectors[j]-1 DO BEGIN
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0]-RSR_col_stitch_2[j,0:ncol_stitch[j,id,0]-1,id,0],symsize=1, $
            color=lcol[id],psym=symbol[id],thick=3
      oplot,lambda_stitch[j,0:ncol_stitch[j,id,0]-1,id,0],RSR_col_stitch[j,0:ncol_stitch[j,id,0]-1,id,0]-RSR_col_stitch_2[j,0:ncol_stitch[j,id,0]-1,id,0],color=lcol[id],thick=3
   ENDFOR
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   IF band_type[j] EQ 0 THEN set_legend,1,chname,lcol,symbol
   IF band_type[j] EQ 1 THEN set_legend,2,chname,lcol,symbol

;*****************************************************************************
;  Plot percent difference in RSR vs wavelength, average and detector
;  dependent ND transmittance.
;*****************************************************************************

   !p.multi=[0,1,1]
   !p.position=0

   title1='ND Trans Diff vs Wavelength (stitched FP-15/16)'
   title2=band_name[j]
   
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)

   x=where(max(abs(trans_avg_corr[j,0:detectors[j]-1,0]-trans_corr[j])) EQ abs(trans_avg_corr[j,0:detectors[j]-1,0]-trans_corr[j]))
   
   plot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],yrange=[0.0000001,1.0],xrange=[xmin,xmax], $
        xtit='Wavelength (nm)',ytit='RSR',title=title2,charsize=1.5,charthick=5,/ylog,/nodata
   
   oplot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],symsize=1,color=lcol[0],psym=symbol[0],thick=3
   oplot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],color=lcol[0],thick=3
   oplot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch_2[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],symsize=1,color=lcol[1],psym=symbol[1],thick=3
   oplot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch_2[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],color=lcol[1],thick=3

   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   set_legend,1,['Det','Avg'],lcol,symbol

;*****************************************************************************
;  Plot RSR vs wavelength, log plot (stitched FP-15/16) for M14 with
;  model OOB.
;*****************************************************************************

   title1='RSR vs Wavelength (stitched FP-15/16)'
   title2=band_name[j]
   
   xmax=max(lambda_stitch[j,0:ncol_stitch[j]-1,8,0],min=xmin)
   
   !p.multi=[0,1,1]
   !p.position=0
   
   plot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,8,0],RSR_col_stitch_3[j,0:ncol_stitch[j,x[0],0]-1,8,0],yrange=[0.0000001,1],xrange=[xmin,xmax],xtit='Wavelength (nm)', $
        ytit='RSR',title=title2,charsize=1.5,charthick=5,/nodata,/ylog
   
   oplot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch_3[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],symsize=1,color=lcol[0],psym=symbol[0],thick=3
   oplot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch_3[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],color=lcol[0],thick=3
   
   oplot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch_4[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],symsize=1,color=lcol[1],psym=symbol[1],thick=3
   oplot,lambda_stitch[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],RSR_col_stitch_4[j,0:ncol_stitch[j,x[0],0]-1,x[0],0],color=lcol[1],thick=3
   
   XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=2,charthick=5
   set_legend,1,['Det','Avg'],lcol,symbol

;*****************************************************************************
;  IOOB
;*****************************************************************************

   IF fpa EQ 'DNB' THEN BEGIN
      ioob_spec=[0.1,0.1,0.1,0.1]
   ENDIF
   IF fpa EQ 'VisNIR' THEN BEGIN
      ioob_spec=[1.0,0.5,1.5,1.0,0.7,1.2,1.5,1.0,1.5]
   ENDIF
   IF fpa EQ 'SMWIR' THEN BEGIN
      ioob_spec=[0.7,0.5,0.8,1.0,0.7,1.0,1.1,1.3]
   ENDIF
   IF fpa EQ 'LWIR' THEN BEGIN
      ioob_spec=[0.4,0.9,0.4,0.4,0.4]
   ENDIF
   
   title2=band_name[j]
   
   IF band_type[j] EQ 1 THEN BEGIN
      ymax=max(IOOB[j,0:detectors[j]-1,*]*100)
      IF ymax LT ioob_spec[j] THEN ymax=ioob_spec[j]*1.05
      
      plot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,yrange=[0,ymax],xrange=[0,32],xtit='Detector',ytit='IOOB (%)',title=title2, $
           charsize=1.5,charthick=5,/nodata

      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,color=lcol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,1]*100,symsize=1,color=lcol[1],psym=symbol[1],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,1]*100,color=lcol[1],thick=3
      
      oplot,D[0:detectors[j]-1]+1,IOOB3[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[2],psym=symbol[2],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB3[j,0:detectors[j]-1,0]*100,color=lcol[2],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB3[j,0:detectors[j]-1,1]*100,symsize=1,color=lcol[3],psym=symbol[3],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB3[j,0:detectors[j]-1,1]*100,color=lcol[3],thick=3
      
      oplot,[0,40],[ioob_spec[j],ioob_spec[j]],color=lcol[2],thick=3,linestyle=2

      set_legend,1,['SS1 Det','SS2 Det','SS1 Avg','SS2 Avg'],lcol,symbol
   ENDIF
   IF band_type[j] EQ 0 THEN BEGIN
      ymax=max(IOOB[j,0:detectors[j]-1,0]*100)
      IF ymax LT ioob_spec[j] THEN ymax=ioob_spec[j]*1.05
      
      plot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,yrange=[0,ymax],xrange=[0,16],xtit='Detector',ytit='IOOB (%)',title=title2, $
           charsize=1.5,charthick=5,/nodata
      
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[0],psym=symbol[0],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB[j,0:detectors[j]-1,0]*100,color=lcol[0],thick=3
      
      oplot,D[0:detectors[j]-1]+1,IOOB3[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[2],psym=symbol[2],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB3[j,0:detectors[j]-1,0]*100,color=lcol[2],thick=3

      oplot,D[0:detectors[j]-1]+1,IOOB4[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[3],psym=symbol[3],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB4[j,0:detectors[j]-1,0]*100,color=lcol[3],thick=3

      oplot,D[0:detectors[j]-1]+1,IOOB5[j,0:detectors[j]-1,0]*100,symsize=1,color=lcol[4],psym=symbol[4],thick=3
      oplot,D[0:detectors[j]-1]+1,IOOB5[j,0:detectors[j]-1,0]*100,color=lcol[4],thick=3
      
      oplot,[0,40],[ioob_spec[j],ioob_spec[j]],color=lcol[2],thick=3,linestyle=2

      set_legend,1,['Det','','Avg','Det (MOD)','Avg (MOD)'],lcol,symbol
   ENDIF
ENDFOR

device,/close

END
